/**
  * @file <src/modules/AutoMoDeBehaviourGenome.h>
  *
  * @package ARGoS3-AutoMoDe
  *
  * @license MIT License
  */

#ifndef AUTOMODE_BEHAVIOUR_GENOME_H
#define AUTOMODE_BEHAVIOUR_GENOME_H

#include "AutoMoDeBehaviour.h"

#include <argos3/core/control_interface/ci_controller.h>
#include <argos3/core/utility/math/range.h>
#include <argos3/core/utility/math/vector2.h>
#include <argos3/core/utility/math/rng.h>

// // ++ modified to relative path (<argos3/demiurge/epuck-dao/EpuckDAO.h>)
// #include "../../../demiurge-epuck-dao/src/EpuckDAO.h"
// // epuckDAO and NEAT-private must be at same level of directory

// #include "../../../demiurge-epuck-dao/src/ReferenceModel1Dot1.h"

#include "../NEAT/network.h"
#include "../NEAT/genome.h"

#include <map>

namespace argos {
	class AutoMoDeBehaviourGenome: public AutoMoDeBehaviour, public CCI_Controller {
		public:

			AutoMoDeBehaviourGenome(std::string label, std::string path);
			AutoMoDeBehaviourGenome(AutoMoDeBehaviourGenome* pc_behaviour);
			virtual ~AutoMoDeBehaviourGenome();

			virtual void ControlStep();
			virtual void Destroy();
			virtual void Reset();
			virtual void ResumeStep();
			virtual void Init();
			virtual void Init(TConfigurationNode& t_node);

			virtual AutoMoDeBehaviourGenome* Clone();

			inline void SetNetwork(NEAT::Network& net) {
      			m_net = &net;
			}

			UInt32 getRobotId();
			virtual void SetRobotDAO(EpuckDAO* pc_robot_dao);

		protected:
			CRange<Real> m_cWheelActuationRange;

			// Network
			NEAT::Network* m_net;
			CRange<Real> m_cNeuralNetworkOutputRange;
			std::vector<NEAT::Network*> m_cNetworkVector;

			// Path to the genome
			std::string m_strPathToGenome;
			std::string GetPathToGenome();

			// Range And Bearing Packet (which contains the id, timestamp, and the real packet which contains the range, bearing and the data)
			struct RBPacket {
				UInt32 unId;
				UInt32 unTimestamp;
				CCI_EPuckRangeAndBearingSensor::SReceivedPacket cMessage;

				RBPacket() {
				};

				RBPacket(UInt32 id, UInt32 timestamp, CCI_EPuckRangeAndBearingSensor::SReceivedPacket message) :
					unId(id),
					unTimestamp(timestamp),
					cMessage(message) {
				}
			};
			SInt32 m_nId;
			std::map<UInt32, RBPacket> m_mapMessages;
			UInt32 m_unTimeStep;

			Real m_fLeftSpeed, m_fRightSpeed;

			argos::CRandom::CRNG* m_pcRNG;

		private:
			virtual void LoadGen(std::string file);
			Real m_inputs[25];
			// Ground sensor samples obtained without using a light spot

	};

	const float m_GraySamplesLeft[4205] = {
	0.339228,
	0.339228,
	0.339228,
	0.339228,
	0.339228,
	0.339228,
	0.339228,
	0.339228,
	0.339228,
	0.339228,
	0.339228,
	0.339228,
	0.339228,
	0.353538,
	0.339228,
	0.339228,
	0.339228,
	0.339228,
	0.339228,
	0.339228,
	0.339228,
	0,
	0.363759,
	0.37398,
	0.331051,
	0.316741,
	0.394423,
	0.341272,
	0.322873,
	0.286076,
	0.331051,
	0.369892,
	0.202261,
	0.296298,
	0.187951,
	0.253368,
	0.298342,
	0.37398,
	0.302431,
	0.329006,
	0.365803,
	0.371936,
	0.355582,
	0.339228,
	0.329006,
	0.329006,
	0.286076,
	0.314696,
	0.320829,
	0.341272,
	0.351493,
	0.339228,
	0.339228,
	0.369892,
	0.365803,
	0.382158,
	0.367848,
	0.343316,
	0.363759,
	0.341272,
	0.298342,
	0,
	0.378069,
	0.267678,
	0.339228,
	0.365803,
	0.386246,
	0.382158,
	0.318785,
	0.304475,
	0.353538,
	0.339228,
	0.369892,
	0.175685,
	0.310608,
	0.181818,
	0.257457,
	0.343316,
	0.298342,
	0.273811,
	0.35967,
	0.353538,
	0.353538,
	0.353538,
	0.329006,
	0.363759,
	0.320829,
	0.324918,
	0.331051,
	0.369892,
	0.361715,
	0.369892,
	0.349449,
	0.363759,
	0.376025,
	0.37398,
	0.369892,
	0.367848,
	0.369892,
	0.351493,
	0.326962,
	0,
	0.343316,
	0.290165,
	0.296298,
	0.351493,
	0.363759,
	0.369892,
	0.298342,
	0.277899,
	0.302431,
	0.320829,
	0.345361,
	0.308564,
	0.322873,
	0.351493,
	0.118446,
	0.0591614,
	0.239058,
	0.151154,
	0.0468958,
	0.376025,
	0.322873,
	0.37398,
	0.369892,
	0.353538,
	0.351493,
	0.324918,
	0.326962,
	0.369892,
	0.357626,
	0.357626,
	0.341272,
	0.363759,
	0.369892,
	0.365803,
	0.357626,
	0.259501,
	0.339228,
	0.341272,
	0.339228,
	0,
	0.312652,
	0.185907,
	0.292209,
	0.37398,
	0.369892,
	0.380113,
	0.267678,
	0.349449,
	0.300386,
	0.296298,
	0.378069,
	0.281988,
	0.288121,
	0.298342,
	0.361715,
	0.273811,
	0.265634,
	0.0428072,
	0.0939142,
	0.37398,
	0.355582,
	0.357626,
	0.345361,
	0.351493,
	0.324918,
	0.357626,
	0.351493,
	0.353538,
	0.361715,
	0.347405,
	0.347405,
	0.369892,
	0.369892,
	0.390335,
	0.353538,
	0.292209,
	0.230881,
	0.357626,
	0.357626,
	0,
	0.369892,
	0.108224,
	0.0918699,
	0.367848,
	0.339228,
	0.369892,
	0.300386,
	0.273811,
	0.369892,
	0.37398,
	0.343316,
	0.349449,
	0.277899,
	0.353538,
	0.353538,
	0.347405,
	0.343316,
	0.369892,
	0.339228,
	0.369892,
	0.384202,
	0.353538,
	0.365803,
	0.38829,
	0.392379,
	0.367848,
	0.339228,
	0.347405,
	0.37398,
	0.367848,
	0.357626,
	0.353538,
	0.382158,
	0.357626,
	0.355582,
	0.37398,
	0.247235,
	0.314696,
	0.322873,
	0,
	0.284032,
	0.108224,
	0.234969,
	0.349449,
	0.339228,
	0.349449,
	0.361715,
	0.284032,
	0.271766,
	0.353538,
	0.300386,
	0.314696,
	0.341272,
	0.279944,
	0.157287,
	0.136844,
	0.459664,
	0.459664,
	0.459664,
	0.459664,
	0.459664,
	0.459664,
	0.459664,
	0.459664,
	0.459664,
	0.466559,
	0.459664,
	0.459664,
	0.459664,
	0.466559,
	0.459664,
	0.459664,
	0.459664,
	0.459664,
	0.459664,
	0.459664,
	0.459664,
	0.00689497,
	0.459664,
	0.335555,
	0.471156,
	0.406803,
	0.284992,
	0.39761,
	0.395311,
	0.358538,
	0.319467,
	0.418295,
	0.471156,
	0,
	0.211446,
	0.42519,
	0.455068,
	0.461963,
	0.402206,
	0.406803,
	0.197656,
	0.445875,
	0.450471,
	0.422891,
	0.464261,
	0.473454,
	0.415996,
	0.475753,
	0.459664,
	0.466559,
	0.484946,
	0.450471,
	0.461963,
	0.471156,
	0.43898,
	0.478051,
	0.484946,
	0.466559,
	0.461963,
	0.459664,
	0.457366,
	0.0183866,
	0.473454,
	0.252815,
	0.153988,
	0.475753,
	0.30108,
	0.395311,
	0.363135,
	0.386118,
	0.344748,
	0.349345,
	0.395311,
	0.266605,
	0.464261,
	0.296484,
	0.498736,
	0.289589,
	0.434383,
	0.264307,
	0.42519,
	0.491841,
	0.441278,
	0.43898,
	0.487244,
	0.482648,
	0.420593,
	0.471156,
	0.429786,
	0.386118,
	0.415996,
	0.489543,
	0.443576,
	0.459664,
	0.450471,
	0.466559,
	0.475753,
	0.436681,
	0.459664,
	0.466559,
	0.466559,
	0.0436681,
	0.496438,
	0.284992,
	0.489543,
	0.360837,
	0.429786,
	0.42519,
	0.335555,
	0.436681,
	0.468858,
	0.379223,
	0.388416,
	0.415996,
	0.393013,
	0.35624,
	0.147093,
	0.255114,
	0.422891,
	0.307975,
	0.275799,
	0.478051,
	0.464261,
	0.461963,
	0.475753,
	0.496438,
	0.434383,
	0.480349,
	0.429786,
	0.374627,
	0.42519,
	0.461963,
	0.459664,
	0.43898,
	0.443576,
	0.475753,
	0.484946,
	0.473454,
	0.432085,
	0.43898,
	0.448173,
	0.0321765,
	0.473454,
	0.351643,
	0.487244,
	0.43898,
	0.445875,
	0.471156,
	0.445875,
	0.418295,
	0.340152,
	0.4114,
	0.333257,
	0.464261,
	0.43898,
	0.372328,
	0.466559,
	0.0919329,
	0.00689497,
	0.131004,
	0.489543,
	0.480349,
	0.466559,
	0.464261,
	0.448173,
	0.496438,
	0.390715,
	0.448173,
	0.464261,
	0.393013,
	0.429786,
	0.455068,
	0.443576,
	0.420593,
	0.482648,
	0.466559,
	0.443576,
	0.436681,
	0.445875,
	0.404505,
	0.464261,
	0.0344748,
	0.461963,
	0.0459664,
	0,
	0.108021,
	0.434383,
	0.418295,
	0.43898,
	0.335555,
	0.445875,
	0.294185,
	0.443576,
	0.464261,
	0.43898,
	0.137899,
	0.0804413,
	0.243622,
	0.409101,
	0.42519,
	0.257412,
	0.496438,
	0.478051,
	0.466559,
	0.445875,
	0.496438,
	0.457366,
	0.461963,
	0.487244,
	0.38382,
	0.376925,
	0.455068,
	0.457366,
	0.420593,
	0.466559,
	0.461963,
	0.487244,
	0.459664,
	0.422891,
	0.448173,
	0.422891,
	0.0275799,
	0.436681,
	0.340152,
	0.466559,
	0.432085,
	0.409101,
	0.480349,
	0.429786,
	0.441278,
	0.422891,
	0.218341,
	0.388416,
	0.32866,
	0.464261,
	0.487244,
	0.376925,
	0.305677,
	0.432085,
	0.158584,
	0.319467,
	0.466559,
	0.489543,
	0.360837,
	0.461963,
	0.478051,
	0.443576,
	0.450471,
	0.482648,
	0.340152,
	0.418295,
	0.443576,
	0.443576,
	0.402206,
	0.478051,
	0.475753,
	0.448173,
	0.461963,
	0.466559,
	0.466559,
	0.448173,
	0.0229832,
	0.298782,
	0.324063,
	0.347047,
	0.475753,
	0.436681,
	0.39761,
	0.466559,
	0.284992,
	0.448173,
	0.216042,
	0.32866,
	0.305677,
	0.234429,
	0.319467,
	0.315105,
	0.315105,
	0.315105,
	0.300263,
	0.315105,
	0.315105,
	0.300263,
	0.315105,
	0.315105,
	0.315105,
	0.315105,
	0.315105,
	0.319346,
	0.315105,
	0.315105,
	0.315105,
	0.315105,
	0.315105,
	0.315105,
	0.315105,
	0.315105,
	0.00977479,
	0.34479,
	0.315105,
	0.204847,
	0.296022,
	0.317226,
	0.291781,
	0.336309,
	0.336309,
	0.323587,
	0.2833,
	0.308744,
	0.289661,
	0.266337,
	0.262097,
	0.308744,
	0.300263,
	0.2833,
	0.312985,
	0.300263,
	0.336309,
	0.336309,
	0.304504,
	0.312985,
	0.336309,
	0.319346,
	0.336309,
	0.336309,
	0.336309,
	0.351151,
	0.293902,
	0.289661,
	0.306624,
	0.296022,
	0.279059,
	0.308744,
	0.321466,
	0.319346,
	0.336309,
	0.321466,
	0.0330986,
	0.194245,
	0.353272,
	0.336309,
	0.336309,
	0.336309,
	0.317226,
	0.34055,
	0.266337,
	0.319346,
	0.279059,
	0.310865,
	0.351151,
	0.217569,
	0.34055,
	0.291781,
	0.317226,
	0.279059,
	0.310865,
	0.317226,
	0.323587,
	0.336309,
	0.323587,
	0.336309,
	0.351151,
	0.349031,
	0.336309,
	0.351151,
	0.355392,
	0.34055,
	0.317226,
	0.34055,
	0.336309,
	0.34479,
	0.319346,
	0.323587,
	0.346911,
	0.34479,
	0.34267,
	0.315105,
	0.0288579,
	0.177283,
	0.268458,
	0.34479,
	0.336309,
	0.336309,
	0.321466,
	0.336309,
	0.264217,
	0.336309,
	0.276939,
	0.276939,
	0.202727,
	0.351151,
	0.353272,
	0.306624,
	0.34267,
	0.323587,
	0.22393,
	0.336309,
	0.323587,
	0.323587,
	0.336309,
	0.336309,
	0.351151,
	0.351151,
	0.336309,
	0.351151,
	0.351151,
	0.34479,
	0.336309,
	0.338429,
	0.336309,
	0.34055,
	0.312985,
	0.336309,
	0.336309,
	0.359633,
	0.338429,
	0.338429,
	0.0309783,
	0.192125,
	0.336309,
	0.346911,
	0.336309,
	0.336309,
	0.34267,
	0.336309,
	0.306624,
	0.34479,
	0.355392,
	0.351151,
	0.319346,
	0.255736,
	0.363873,
	0.336309,
	0.317226,
	0.249374,
	0.310865,
	0,
	0.336309,
	0.319346,
	0.336309,
	0.351151,
	0.355392,
	0.34479,
	0.349031,
	0.353272,
	0.357512,
	0.351151,
	0.325707,
	0.336309,
	0.336309,
	0.351151,
	0.336309,
	0.336309,
	0.336309,
	0.346911,
	0.349031,
	0.346911,
	0.0224969,
	0.141237,
	0.315105,
	0.194245,
	0.321466,
	0.264217,
	0.355392,
	0.336309,
	0.298143,
	0.34479,
	0.338429,
	0.232412,
	0.349031,
	0.361753,
	0.359633,
	0.336309,
	0.34479,
	0,
	0.315105,
	0.274819,
	0.317226,
	0.336309,
	0.34055,
	0.349031,
	0.353272,
	0.357512,
	0.349031,
	0.355392,
	0.349031,
	0.323587,
	0.336309,
	0.336309,
	0.349031,
	0.351151,
	0.851554,
	0.336309,
	0.336309,
	0.34479,
	0.355392,
	0.355392,
	0.0224969,
	0.122153,
	0.336309,
	0.34267,
	0.336309,
	0.304504,
	0.321466,
	0.317226,
	0.323587,
	0.336309,
	0.34479,
	0.298143,
	0.257856,
	0.198486,
	0.361753,
	0.336309,
	0.34267,
	0.321466,
	0.183644,
	0.200606,
	0.336309,
	0.336309,
	0.336309,
	0.351151,
	0.34479,
	0.353272,
	0.357512,
	0.361753,
	0.34479,
	0.336309,
	0.312985,
	0.336309,
	0.34055,
	0.349031,
	0.336309,
	0.336309,
	0.336309,
	0.349031,
	0.351151,
	0.355392,
	0.0161358,
	0.117913,
	0.349031,
	0.240893,
	0.300263,
	0.323587,
	0.34055,
	0.353272,
	0.298143,
	0.336309,
	0.336309,
	0.34479,
	0.279059,
	0.270578,
	0.245134,
	0.413043,
	0.413043,
	0.413043,
	0.413043,
	0.413043,
	0.413043,
	0.413043,
	0.413043,
	0.413043,
	0.413043,
	0.413043,
	0.415029,
	0.413043,
	0.413043,
	0.419002,
	0.413043,
	0.413043,
	0.417016,
	0.413043,
	0.413043,
	0.413043,
	0.061441,
	0.178642,
	0.367355,
	0.363382,
	0.369341,
	0.343517,
	0.287897,
	0.266046,
	0.367355,
	0.3753,
	0.353449,
	0.373314,
	0.357422,
	0.309748,
	0.31372,
	0.3753,
	0.387219,
	0.359409,
	0.359409,
	0.277964,
	0.385233,
	0.365368,
	0.373314,
	0.377287,
	0.385233,
	0.369341,
	0.373314,
	0.365368,
	0.387219,
	0.323653,
	0.419002,
	0.401124,
	0.413043,
	0.413043,
	0.413043,
	0.397151,
	0.413043,
	0.413043,
	0.413043,
	0.393179,
	0.0852784,
	0.214398,
	0.383246,
	0.383246,
	0.379273,
	0.359409,
	0.31968,
	0.379273,
	0.204466,
	0.399138,
	0.31372,
	0.317693,
	0.275978,
	0.22433,
	0.377287,
	0.333585,
	0.281937,
	0.337558,
	0.317693,
	0.38126,
	0.385233,
	0.367355,
	0.389206,
	0.371328,
	0.379273,
	0.361395,
	0.311734,
	0.273991,
	0.371328,
	0.373314,
	0.413043,
	0.419002,
	0.413043,
	0.413043,
	0.397151,
	0.403111,
	0.403111,
	0.413043,
	0.391192,
	0.389206,
	0.079319,
	0.244195,
	0.329612,
	0.399138,
	0.323653,
	0.399138,
	0.403111,
	0.365368,
	0.387219,
	0.357422,
	0.329612,
	0.329612,
	0.273991,
	0.293856,
	0.272005,
	0.208438,
	0.357422,
	0.28591,
	0.373314,
	0.391192,
	0.389206,
	0.379273,
	0.379273,
	0.383246,
	0.355436,
	0.335571,
	0.333585,
	0.301802,
	0.359409,
	0.373314,
	0.401124,
	0.413043,
	0.405097,
	0.399138,
	0.413043,
	0.413043,
	0.405097,
	0.403111,
	0.367355,
	0.3753,
	0.0753461,
	0.230289,
	0.365368,
	0.291869,
	0.38126,
	0.389206,
	0.415029,
	0.397151,
	0.413043,
	0.379273,
	0.31968,
	0.315707,
	0.279951,
	0.353449,
	0.345504,
	0.353449,
	0.397151,
	0.383246,
	0.379273,
	0.389206,
	0.387219,
	0.379273,
	0.369341,
	0.3753,
	0.369341,
	0.363382,
	0.323653,
	0.31968,
	0.359409,
	0.371328,
	0.393179,
	0.413043,
	0.401124,
	0.424962,
	0.415029,
	0.413043,
	0.413043,
	0.413043,
	0.391192,
	0.335571,
	0.0713732,
	0.236249,
	0.315707,
	0.413043,
	0.323653,
	0.426948,
	0.391192,
	0.424962,
	0.395165,
	0.389206,
	0.367355,
	0.323653,
	0.25214,
	0.31968,
	0.367355,
	0.391192,
	0.397151,
	0.349477,
	0.335571,
	0.379273,
	0.387219,
	0.3753,
	0.351463,
	0.379273,
	0.379273,
	0.363382,
	0.323653,
	0.331598,
	0.307761,
	0.371328,
	0.397151,
	0.391192,
	0.424962,
	0.422975,
	0.422975,
	0.405097,
	0.397151,
	0.413043,
	0.391192,
	0.363382,
	0.0733597,
	0.232276,
	0.387219,
	0.246181,
	0.422975,
	0.373314,
	0.389206,
	0.417016,
	0.407084,
	0.389206,
	0.359409,
	0.307761,
	0.337558,
	0.335571,
	0.367355,
	0.214398,
	0.351463,
	0.158777,
	0.0971971,
	0.192547,
	0.395165,
	0.371328,
	0.351463,
	0.367355,
	0.367355,
	0.373314,
	0.357422,
	0.34749,
	0.303788,
	0.345504,
	0.387219,
	0.413043,
	0.417016,
	0.397151,
	0.419002,
	0.413043,
	0.403111,
	0.405097,
	0.387219,
	0.357422,
	0.0813055,
	0.230289,
	0.307761,
	0.242208,
	0.415029,
	0.3753,
	0.413043,
	0.377287,
	0.413043,
	0.367355,
	0.337558,
	0.299815,
	0.339544,
	0.367355,
	0.608965,
	0.608965,
	0.606659,
	0.606659,
	0.608965,
	0.606659,
	0.606659,
	0.608965,
	0.608965,
	0.608965,
	0.606659,
	0.606659,
	0.608965,
	0.608965,
	0.606659,
	0.606659,
	0.611271,
	0.606659,
	0.606659,
	0.606659,
	0.606659,
	0.293057,
	0.611271,
	1,
	0.608965,
	0.578989,
	0.530565,
	0.578989,
	0.606659,
	0.426799,
	0.519035,
	0.592824,
	0.549012,
	0.532871,
	0.576683,
	0.565153,
	0.535177,
	0.578989,
	0.528259,
	0.519035,
	0.537482,
	0.565153,
	0.565153,
	0.5836,
	0.590518,
	0.553624,
	0.581295,
	0.578989,
	0.604354,
	0.560541,
	0.618189,
	0.622801,
	0.59513,
	0.597436,
	0.604354,
	0.585906,
	0.565153,
	0.604354,
	0.565153,
	0.574377,
	0.606659,
	0.267692,
	0.613577,
	0.472917,
	0.55593,
	0.382987,
	0.382987,
	0.604354,
	0.611271,
	0.602048,
	1,
	0.431411,
	0.590518,
	0.336869,
	0.320728,
	0.535177,
	0.35301,
	0.426799,
	1,
	0.380681,
	0.592824,
	0.592824,
	0.597436,
	0.597436,
	0.625107,
	0.588212,
	0.578989,
	0.615883,
	0.608965,
	0.608965,
	0.551318,
	0.549012,
	0.604354,
	0.602048,
	0.606659,
	0.604354,
	0.59513,
	0.611271,
	0.59513,
	0.576683,
	0.576683,
	0.258468,
	0.532871,
	0.436023,
	0.632024,
	0.581295,
	0.530565,
	0.59513,
	0.615883,
	0.509812,
	0.535177,
	0.525953,
	0.5836,
	0.5836,
	0.472917,
	0.606659,
	0.31381,
	0.461388,
	0.5444,
	0.40374,
	0.606659,
	0.592824,
	0.606659,
	0.620495,
	0.606659,
	0.627413,
	0.599742,
	0.629718,
	0.604354,
	0.63433,
	0.588212,
	0.537482,
	0.549012,
	0.5836,
	0.585906,
	0.578989,
	0.613577,
	0.613577,
	0.608965,
	0.565153,
	0.581295,
	0.272304,
	0.604354,
	0.606659,
	0.491364,
	0.597436,
	0.592824,
	0.539788,
	0.525953,
	0.523647,
	0.588212,
	0.429105,
	0.602048,
	0.36454,
	0.521341,
	0.602048,
	0.597436,
	0.615883,
	0.530565,
	0.491364,
	0.588212,
	0.581295,
	0.602048,
	0.599742,
	0.627413,
	0.625107,
	0.606659,
	0.620495,
	0.611271,
	0.625107,
	0.611271,
	0.581295,
	0.572071,
	0.597436,
	0.567459,
	0.576683,
	0.606659,
	0.5836,
	0.592824,
	0.576683,
	0.553624,
	0.267692,
	0.608965,
	0.470611,
	0.618189,
	0.542094,
	0.514423,
	0.567459,
	0.574377,
	0.525953,
	0.567459,
	0.625107,
	0.468305,
	0.590518,
	0.316116,
	0.442941,
	0.565153,
	0.63433,
	0.528259,
	0.581295,
	0.576683,
	0.5836,
	0.597436,
	0.597436,
	0.622801,
	0.606659,
	0.618189,
	0.615883,
	0.578989,
	0.622801,
	0.611271,
	0.576683,
	0.576683,
	0.569765,
	0.569765,
	0.562847,
	0.590518,
	0.613577,
	0.599742,
	0.565153,
	0.562847,
	0.31381,
	0.539788,
	0.604354,
	0.408352,
	0.581295,
	0.406046,
	0.578989,
	0.588212,
	0.532871,
	0.495976,
	0.592824,
	0.468305,
	0.530565,
	0.482141,
	0.608965,
	0.316116,
	0.472917,
	0.486753,
	0.371458,
	0.59513,
	0.574377,
	0.608965,
	0.608965,
	0.655083,
	0.620495,
	0.608965,
	0.613577,
	0.59513,
	0.620495,
	0.627413,
	0.592824,
	0.590518,
	0.565153,
	0.565153,
	0.553624,
	0.608965,
	0.618189,
	0.604354,
	0.581295,
	0.562847,
	0.288445,
	0.588212,
	0.592824,
	0.470611,
	1,
	0.553624,
	0.572071,
	0.567459,
	0.5836,
	0.55593,
	0.456776,
	0.572071,
	0.459082,
	0.509812,
	0.356728,
	0.356728,
	0.356728,
	0.356728,
	0.356728,
	0.356728,
	0.356728,
	0.339298,
	0.356728,
	0.356728,
	0.356728,
	0.356728,
	0.356728,
	0.356728,
	0.356728,
	0.356728,
	0.356728,
	0.356728,
	0.356728,
	0.348013,
	0.356728,
	0.0371833,
	0.170811,
	0.295724,
	0.20567,
	0.345108,
	0.147571,
	0.159191,
	0.208575,
	0.127237,
	0.127237,
	0.153381,
	0.255055,
	0.365443,
	0.275389,
	0.350918,
	0.362538,
	0.287009,
	0.196956,
	0.353823,
	0.202765,
	0.362538,
	0.307344,
	0.350918,
	0.353823,
	0.368348,
	0.371253,
	0.362538,
	0.379967,
	0.356728,
	0.374158,
	0.365443,
	0.356728,
	0.371253,
	0.365443,
	0.348013,
	0.356728,
	0.333488,
	0.345108,
	0.324773,
	0.348013,
	0.0255635,
	0.269579,
	0.20567,
	0.20567,
	0.345108,
	0.162096,
	0.150476,
	0.391587,
	0.266674,
	0.281199,
	0.353823,
	0.327678,
	0.275389,
	0.362538,
	0.353823,
	0.333488,
	0.298629,
	0.220195,
	0.260864,
	0.348013,
	0.353823,
	0.313154,
	0.345108,
	0.350918,
	0.365443,
	0.377062,
	0.362538,
	0.365443,
	0.356728,
	0.345108,
	0.371253,
	0.333488,
	0.374158,
	0.371253,
	0.356728,
	0.318963,
	0.324773,
	0.333488,
	0.330583,
	0.356728,
	0.0284685,
	0.25796,
	0.127237,
	0.0313734,
	0.0981873,
	0.356728,
	0.162096,
	0.156286,
	0.353823,
	0.2231,
	0.368348,
	0.333488,
	0.313154,
	0.371253,
	0.313154,
	0.313154,
	0.196956,
	0.310249,
	0.321868,
	0.292819,
	0.356728,
	0.307344,
	0.333488,
	0.345108,
	0.356728,
	0.368348,
	0.356728,
	0.377062,
	0.356728,
	0.371253,
	0.368348,
	0.345108,
	0.382872,
	0.356728,
	0.350918,
	0.278294,
	0.292819,
	0.336393,
	0.318963,
	0.356728,
	0.0197536,
	0.301534,
	0.208575,
	0.144666,
	0.307344,
	0.2231,
	0.368348,
	0.231815,
	0.130142,
	0.118522,
	0.348013,
	0.22891,
	0.382872,
	0.289914,
	0.318963,
	0.388682,
	0.199861,
	0.350918,
	0.226005,
	0.316059,
	0.362538,
	0.330583,
	0.313154,
	0.348013,
	0.371253,
	0.379967,
	0.377062,
	0.397397,
	0.350918,
	0.368348,
	0.374158,
	0.374158,
	0.353823,
	0.377062,
	0.368348,
	0.292819,
	0.298629,
	0.356728,
	0.342203,
	0.350918,
	0.0139437,
	0.231815,
	0.316059,
	0.25215,
	0.167906,
	0.356728,
	0.310249,
	0.388682,
	0.304439,
	0.333488,
	0.313154,
	0.278294,
	0.356728,
	0.345108,
	0.382872,
	0.220195,
	0.316059,
	0.336393,
	0.281199,
	0.260864,
	0.342203,
	0.316059,
	0.342203,
	0.339298,
	0.385777,
	0.391587,
	0.374158,
	0.379967,
	0.368348,
	0.382872,
	0.362538,
	0.374158,
	0.379967,
	0.350918,
	0.348013,
	0.304439,
	0.330583,
	0.365443,
	0.365443,
	0.182431,
	0.00232392,
	0.202765,
	0.054613,
	0.054613,
	0.336393,
	0.0778526,
	0.165001,
	0.371253,
	0.356728,
	0.272484,
	0.385777,
	0.20567,
	0.374158,
	0.101092,
	0.0517081,
	0.20567,
	0.298629,
	0.333488,
	0.138857,
	0.0604229,
	0.333488,
	0.30394,
	0.30394,
	0.30394,
	0.30394,
	0.30394,
	0.30394,
	0.30394,
	0.30394,
	0.30394,
	0.30394,
	0.30394,
	0.30394,
	0.30394,
	0.30394,
	0.30394,
	0.30394,
	0.30394,
	0.30394,
	0.30394,
	0.30394,
	0.792494,
	0,
	0.124472,
	0.222182,
	0.30394,
	0.30394,
	0.30394,
	0.30394,
	0.170336,
	0.319893,
	0.305934,
	0.264058,
	0.30394,
	0.278017,
	0.270041,
	0.264058,
	0.282005,
	0.305934,
	0.30394,
	0.254088,
	0.30394,
	0.309923,
	0.268047,
	0.319893,
	0.30394,
	0.30394,
	0.270041,
	0.278017,
	0.30394,
	0.30394,
	0.30394,
	0.30394,
	0.30394,
	0.276023,
	0.276023,
	0.270041,
	0.30394,
	0.311917,
	0.280011,
	0.272035,
	0.276023,
	0,
	0.150395,
	0.230159,
	0.309923,
	0.17233,
	0.280011,
	0.30394,
	0.30394,
	0.30394,
	0.30394,
	0.218194,
	0.270041,
	0.222182,
	0.30394,
	0.321887,
	0.280011,
	0.280011,
	0.30394,
	0.266053,
	0.315905,
	0.309923,
	0.30394,
	0.30394,
	0.305934,
	0.30394,
	0.30394,
	0.278017,
	0.30394,
	0.270041,
	0.264058,
	0.30394,
	0.30394,
	0.282005,
	0.30394,
	0.812435,
	0.30394,
	0.30394,
	0.30394,
	0.268047,
	0.274029,
	0,
	0.234147,
	0.160365,
	0.266053,
	0.236141,
	0.212212,
	0.270041,
	0.30394,
	0.30394,
	0.30394,
	0.248106,
	0.134442,
	0.282005,
	0.274029,
	0.305934,
	0.254088,
	0.30394,
	0.30394,
	0.30394,
	0.30394,
	0.309923,
	0.305934,
	0.30394,
	0.30394,
	0.30394,
	0.30394,
	0.274029,
	0.30394,
	0.282005,
	0.274029,
	0.280011,
	0.30394,
	0.30394,
	0.30394,
	0.282005,
	0.30394,
	0.307929,
	0.313911,
	0.7905,
	0.282005,
	0,
	0.307929,
	0.176318,
	0.278017,
	0.30394,
	0.278017,
	0.240129,
	0.283999,
	0.282005,
	0.283999,
	0.264058,
	0.176318,
	0.258076,
	0.0347372,
	0.0287549,
	0.30394,
	0.20623,
	0.30394,
	0.315905,
	0.30394,
	0.30394,
	0.30394,
	0.30394,
	0.30394,
	0.305934,
	0.270041,
	0.274029,
	0.26007,
	0.264058,
	0.258076,
	0.270041,
	0.30394,
	0.276023,
	0.30394,
	0.30394,
	0.30394,
	0.307929,
	0.313911,
	0.30394,
	0.226171,
	0,
	0.30394,
	0.146407,
	0.158371,
	0.30394,
	0.252094,
	0.17233,
	0.30394,
	0.278017,
	0.278017,
	0.266053,
	0,
	0.282005,
	0.30394,
	0.30394,
	0.276023,
	0.30394,
	0.30394,
	0.30394,
	0.268047,
	0.315905,
	0.307929,
	0.30394,
	0.305934,
	0.311917,
	0.30394,
	0.30394,
	0.270041,
	0.272035,
	0.258076,
	0.282005,
	0.30394,
	0.26007,
	0.283999,
	0.30394,
	0.30394,
	0.307929,
	0.319893,
	0.30394,
	0.264058,
	0,
	0.309923,
	0.148401,
	0.12846,
	0.30394,
	0.262064,
	0.120483,
	0.180306,
	0.196259,
	0.282005,
	0.272035,
	0.156377,
	0.246112,
	0.230159,
	0.30394,
	0.282005,
	0.270041,
	0.266053,
	0.258076,
	0.30394,
	0.30394,
	0.282005,
	0.30394,
	0.305934,
	0.315905,
	0.30394,
	0.30394,
	0.268047,
	0.30394,
	0.274029,
	0.270041,
	0.30394,
	0.30394,
	0.307929,
	0.321887,
	0.313911,
	0.307929,
	0.313911,
	0.278017,
	0.276023,
	0,
	0.1823,
	0.214206,
	0.202241,
	0.384214,
	0.386254,
	0.386254,
	0.386254,
	0.384214,
	0.384214,
	0.386254,
	0.384214,
	0.386254,
	0.386254,
	0.386254,
	0.384214,
	0.384214,
	0.386254,
	0.398498,
	0.384214,
	0.384214,
	0.384214,
	0.384214,
	0.384214,
	0.384214,
	0.0883193,
	0.380132,
	0.237287,
	0.267897,
	0.380132,
	0.396457,
	0.325035,
	0.369929,
	0.396457,
	0.390335,
	0.394417,
	0.347482,
	0.351563,
	0.327075,
	0.312791,
	0.300547,
	0.37401,
	0.361766,
	0.378092,
	0.392376,
	0.402579,
	0.382173,
	0.353604,
	0.351563,
	0.335238,
	0.306669,
	0.333197,
	0.367888,
	0.388295,
	0.398498,
	0.376051,
	0.406661,
	0.390335,
	0.37197,
	0.410742,
	0.396457,
	0.386254,
	0.343401,
	0.378092,
	0.382173,
	0.0658722,
	0.263815,
	0.306669,
	0.388295,
	0.384214,
	0.351563,
	0.394417,
	0.390335,
	0.378092,
	0.386254,
	0.378092,
	0.384214,
	0.345441,
	0.304628,
	0.314831,
	0.361766,
	0.382173,
	0.267897,
	0.392376,
	0.400539,
	0.398498,
	0.384214,
	0.37197,
	0.363807,
	0.349522,
	0.335238,
	0.333197,
	0.357685,
	0.382173,
	0.394417,
	0.386254,
	0.398498,
	0.384214,
	0.392376,
	0.390335,
	0.398498,
	0.392376,
	0.384214,
	0.37401,
	0.396457,
	0.0638315,
	0.251571,
	0.314831,
	0.400539,
	0.392376,
	0.276059,
	0.398498,
	0.400539,
	0.384214,
	0.386254,
	0.331157,
	0.376051,
	0.329116,
	0.327075,
	0.343401,
	0.359726,
	0.388295,
	0.388295,
	0.398498,
	0.400539,
	0.396457,
	0.392376,
	0.382173,
	0.37401,
	0.347482,
	0.337279,
	0.312791,
	0.343401,
	0.388295,
	0.388295,
	0.402579,
	0.398498,
	0.408701,
	0.396457,
	0.390335,
	0.394417,
	0.394417,
	0.380132,
	0.378092,
	0.382173,
	0.0638315,
	0.229124,
	0.34136,
	0.398498,
	0.396457,
	0.337279,
	0.396457,
	0.398498,
	0.369929,
	0.384214,
	0.37401,
	0.327075,
	0.300547,
	0.349522,
	0.320953,
	0.363807,
	0.361766,
	0.357685,
	0.394417,
	0.271978,
	0.394417,
	0.392376,
	0.367888,
	0.386254,
	0.359726,
	0.353604,
	0.339319,
	0.327075,
	0.37197,
	0.394417,
	0.398498,
	0.392376,
	0.386254,
	0.392376,
	0.386254,
	0.398498,
	0.380132,
	0.378092,
	0.376051,
	0.382173,
	0.0577096,
	0.196474,
	0.359726,
	0.396457,
	0.398498,
	0.398498,
	0.382173,
	0.325035,
	0.349522,
	0.380132,
	0.31075,
	0.345441,
	0.345441,
	0.363807,
	0.378092,
	0.390335,
	0.390335,
	0.322994,
	0.400539,
	0.398498,
	0.400539,
	0.396457,
	0.410742,
	0.400539,
	0.378092,
	0.37197,
	0.357685,
	0.333197,
	0.34136,
	0.386254,
	0.396457,
	0.378092,
	0.353604,
	0.398498,
	0.392376,
	0.396457,
	0.402579,
	0.37401,
	0.378092,
	0.398498,
	0.0638315,
	0.218921,
	0.331157,
	0.398498,
	0.386254,
	0.325035,
	0.398498,
	0.34136,
	0.367888,
	0.392376,
	0.269937,
	0.320953,
	0.349522,
	0.376051,
	0.369929,
	0.367888,
	0.363807,
	0.403229,
	0.414034,
	0.403229,
	0.403229,
	0.403229,
	0.403229,
	0.403229,
	0.403229,
	0.403229,
	0.403229,
	0.403229,
	0.411873,
	0.403229,
	0.403229,
	0.403229,
	0.403229,
	0.403229,
	0.403229,
	0.403229,
	0.403229,
	0.403229,
	0.150387,
	0.38594,
	0.264922,
	0.271405,
	0.342719,
	0.368652,
	0.439966,
	0.431322,
	0.403229,
	0.403229,
	0.38594,
	0.396745,
	0.377296,
	0.403229,
	0.422678,
	0.366491,
	0.427,
	0.403229,
	0.390262,
	0.429161,
	0.420517,
	0.439966,
	0.442127,
	0.424839,
	0.424839,
	0.439966,
	0.411873,
	0.390262,
	0.401068,
	0.418356,
	0.427,
	0.429161,
	0.433483,
	0.429161,
	0.411873,
	0.368652,
	0.34488,
	0.318948,
	0.366491,
	0.370813,
	0.143904,
	0.368652,
	0.234667,
	0.221701,
	0.38594,
	0.392423,
	0.433483,
	0.422678,
	0.422678,
	0.396745,
	0.403229,
	0.381618,
	0.396745,
	0.366491,
	0.390262,
	0.396745,
	0.429161,
	0.411873,
	0.351364,
	0.431322,
	0.442127,
	0.429161,
	0.422678,
	0.411873,
	0.439966,
	0.433483,
	0.429161,
	0.394584,
	0.392423,
	0.403229,
	0.424839,
	0.433483,
	0.442127,
	0.439966,
	0.414034,
	0.38594,
	0.360008,
	0.349203,
	0.351364,
	0.366491,
	0.139582,
	0.34488,
	0.288693,
	0.245473,
	0.390262,
	0.38594,
	0.442127,
	0.409712,
	0.439966,
	0.403229,
	0.396745,
	0.403229,
	0.403229,
	0.409712,
	0.418356,
	0.375135,
	0.351364,
	0.422678,
	0.288693,
	0.429161,
	0.442127,
	0.414034,
	0.431322,
	0.414034,
	0.44861,
	0.439966,
	0.431322,
	0.420517,
	0.396745,
	0.407551,
	0.433483,
	0.431322,
	0.452933,
	0.429161,
	0.424839,
	0.392423,
	0.36433,
	0.366491,
	0.360008,
	0.379457,
	0.154709,
	0.377296,
	0.303821,
	0.347042,
	0.390262,
	0.372974,
	0.357847,
	0.381618,
	0.452933,
	0.390262,
	0.401068,
	0.411873,
	0.372974,
	0.439966,
	0.44861,
	0.381618,
	0.392423,
	0.390262,
	0.416195,
	0.407551,
	0.416195,
	0.429161,
	0.414034,
	0.411873,
	0.439966,
	0.439966,
	0.439966,
	0.418356,
	0.411873,
	0.422678,
	0.439966,
	0.44861,
	0.427,
	0.427,
	0.398906,
	0.362169,
	0.327592,
	0.349203,
	0.370813,
	0.396745,
	0.143904,
	0.334075,
	0.238989,
	0.230345,
	0.366491,
	0.351364,
	0.396745,
	0.379457,
	0.407551,
	0.383779,
	0.411873,
	0.420517,
	0.394584,
	0.381618,
	0.377296,
	0.429161,
	0.347042,
	0.403229,
	0.383779,
	0.414034,
	0.420517,
	0.416195,
	0.414034,
	0.433483,
	0.435644,
	0.431322,
	0.442127,
	0.424839,
	0.424839,
	0.414034,
	0.435644,
	0.435644,
	0.424839,
	0.403229,
	0.383779,
	0.360008,
	0.316787,
	0.312465,
	0.403229,
	0.416195,
	0.143904,
	0.321109,
	0.30166,
	0.310304,
	0.247634,
	0.377296,
	0.439966,
	0.321109,
	0.446449,
	0.38594,
	0.409712,
	0.403229,
	0.427,
	0.435644,
	0.401068,
	0.418356,
	0.30166,
	0.416195,
	0.420517,
	0.291524,
	0.291524,
	0.291524,
	0.291524,
	0.291524,
	0.291524,
	0.291524,
	0.291524,
	0.291524,
	0.283673,
	0.291524,
	0.291524,
	0.293486,
	0.291524,
	0.291524,
	0.291524,
	0.291524,
	0.291524,
	0.291524,
	0.291524,
	0.291524,
	0.0776009,
	0.279748,
	0.207132,
	0.281711,
	0.283673,
	0.287598,
	0.289561,
	0.27386,
	0.283673,
	0.26601,
	0.279748,
	0.275823,
	0.27386,
	0.277785,
	0.256197,
	0.283673,
	0.285636,
	0.281711,
	0.27386,
	0.271898,
	0.287598,
	0.283673,
	0.287598,
	0.289561,
	0.285636,
	0.295449,
	0.293486,
	0.285636,
	0.287598,
	0.289561,
	0.283673,
	0.299374,
	0.289561,
	0.291524,
	0.289561,
	0.293486,
	0.287598,
	0.283673,
	0.281711,
	0.287598,
	0.0363865,
	0.181618,
	0.230683,
	0.267972,
	0.269935,
	0.289561,
	0.285636,
	0.252272,
	0.281711,
	0.275823,
	0.293486,
	0.287598,
	0.248347,
	0.279748,
	0.287598,
	0.269935,
	0.299374,
	0.285636,
	0.269935,
	0.281711,
	0.293486,
	0.301337,
	0.293486,
	0.293486,
	0.299374,
	0.299374,
	0.287598,
	0.281711,
	0.283673,
	0.295449,
	0.299374,
	0.293486,
	0.299374,
	0.291524,
	0.293486,
	0.287598,
	0.287598,
	0.291524,
	0.305262,
	0.269935,
	0.0363865,
	0.128628,
	0.271898,
	0.299374,
	0.305262,
	0.289561,
	0.297411,
	0.283673,
	0.281711,
	0.289561,
	0.291524,
	0.289561,
	0.285636,
	0.293486,
	0.299374,
	0.289561,
	0.295449,
	0.291524,
	0.291524,
	0.293486,
	0.289561,
	0.303299,
	0.291524,
	0.293486,
	0.291524,
	0.293486,
	0.287598,
	0.281711,
	0.287598,
	0.295449,
	0.299374,
	0.303299,
	0.299374,
	0.293486,
	0.295449,
	0.291524,
	0.287598,
	0.289561,
	0.301337,
	0.277785,
	0.0304987,
	0.120778,
	0.216945,
	0.27386,
	0.299374,
	0.238534,
	0.275823,
	0.279748,
	0.299374,
	0.291524,
	0.293486,
	0.289561,
	0.281711,
	0.301337,
	0.289561,
	0.228721,
	0.254234,
	0.289561,
	0.269935,
	0.269935,
	0.295449,
	0.297411,
	0.297411,
	0.285636,
	0.299374,
	0.287598,
	0.289561,
	0.281711,
	0.285636,
	0.283673,
	0.291524,
	0.293486,
	0.287598,
	0.289561,
	0.275823,
	0.275823,
	0.293486,
	0.299374,
	0.299374,
	0.275823,
	0.0363865,
	0.122741,
	0.279748,
	0.293486,
	0.289561,
	0.295449,
	0.299374,
	0.283673,
	0.283673,
	0.26601,
	0.277785,
	0.287598,
	0.269935,
	0.281711,
	0.291524,
	0.285636,
	0.289561,
	0.283673,
	0.262085,
	0.295449,
	0.293486,
	0.299374,
	0.295449,
	0.297411,
	0.297411,
	0.299374,
	0.295449,
	0.285636,
	0.289561,
	0.283673,
	0.295449,
	0.299374,
	0.289561,
	0.289561,
	0.293486,
	0.293486,
	0.283673,
	0.295449,
	0.299374,
	0.285636,
	0.0422743,
	0.116853,
	0.191431,
	0.189469,
	0.285636,
	0.299374,
	0.293486,
	0.281711,
	0.283673,
	0.289561,
	0.289561,
	0.297411,
	0.285636,
	0.262085,
	0.301337,
	0.299374,
	0.289561,
	0.216945,
	0.269935,
	0.27386,
	0.285636,
	0.289561,
	0.293486,
	0.301337,
	0.301337,
	0.299374,
	0.295449,
	0.295449,
	0.933469,
	0.933469,
	0.933469,
	0.933469,
	0.933469,
	0.933469,
	0.933469,
	0.933469,
	0.933469,
	0.933469,
	0.933469,
	0.933469,
	0.933469,
	0.933469,
	0.933469,
	0.933469,
	0.933469,
	0.933469,
	0.933469,
	0.933469,
	0.933469,
	0.0157927,
	0.869445,
	0.858774,
	0.928133,
	0.901457,
	0.928133,
	0.906792,
	0.922798,
	0.576002,
	0.853439,
	0.917463,
	0.917463,
	0.922798,
	0.922798,
	0.922798,
	0.922798,
	0.928133,
	0.928133,
	0.928133,
	0.922798,
	0.928133,
	0.922798,
	0.928133,
	0.917463,
	0.922798,
	0.928133,
	0.928133,
	0.928133,
	0.928133,
	0.922798,
	0.922798,
	0.928133,
	0.922798,
	0.928133,
	0.933469,
	0.933469,
	0.933469,
	0.933469,
	0.885451,
	0.912127,
	0.207864,
	0.837433,
	0.901457,
	0.922798,
	0.928133,
	0.922798,
	0.864109,
	0.917463,
	0.922798,
	0.917463,
	0.922798,
	0.922798,
	0.922798,
	0.885451,
	0.933469,
	0.928133,
	0.933469,
	0.928133,
	0.928133,
	0.901457,
	0.933469,
	0.922798,
	0.928133,
	0.922798,
	0.928133,
	0.928133,
	0.928133,
	0.928133,
	0.922798,
	0.922798,
	0.922798,
	0.922798,
	0.901457,
	0.928133,
	0.928133,
	0.928133,
	0.928133,
	0.928133,
	0.922798,
	0.917463,
	0.197194,
	0.890786,
	0.896121,
	0.922798,
	0.933469,
	0.917463,
	0.890786,
	0.71472,
	0.378595,
	0.922798,
	0.922798,
	0.928133,
	0.922798,
	0.928133,
	0.922798,
	0.922798,
	0.917463,
	0.912127,
	0.917463,
	0.928133,
	0.933469,
	0.928133,
	0.928133,
	0.917463,
	0.922798,
	0.922798,
	0.928133,
	0.922798,
	0.922798,
	0.917463,
	0.922798,
	0.928133,
	0.917463,
	0.933469,
	0.933469,
	0.933469,
	0.928133,
	0.928133,
	0.928133,
	0.901457,
	0.186523,
	0.896121,
	0.495972,
	0.229206,
	0.826762,
	0.906792,
	0.928133,
	0.848103,
	0.906792,
	0.901457,
	0.917463,
	0.933469,
	0.922798,
	0.853439,
	0.928133,
	0.928133,
	0.922798,
	0.922798,
	0.869445,
	0.869445,
	0.933469,
	0.928133,
	0.928133,
	0.933469,
	0.928133,
	0.928133,
	0.928133,
	0.928133,
	0.928133,
	0.922798,
	0.917463,
	0.922798,
	0.928133,
	0.912127,
	0.922798,
	0.928133,
	0.928133,
	0.922798,
	0.917463,
	0.928133,
	0.218535,
	0.853439,
	0.250547,
	0.357253,
	0.928133,
	0.928133,
	0.922798,
	0.917463,
	0.922798,
	0.922798,
	0.917463,
	0.933469,
	0.901457,
	0.928133,
	0.928133,
	0.928133,
	0.917463,
	0.928133,
	0.912127,
	0.928133,
	0.933469,
	0.928133,
	0.928133,
	0.933469,
	0.928133,
	0.928133,
	0.933469,
	0.933469,
	0.928133,
	0.922798,
	0.917463,
	0.928133,
	0.928133,
	0.928133,
	0.917463,
	0.917463,
	0.922798,
	0.922798,
	0.928133,
	0.928133,
	0.234541,
	0.858774,
	0.890786,
	0.922798,
	0.885451,
	0.933469,
	0.933469,
	0.928133,
	0.917463,
	0.922798,
	0.922798,
	0.928133,
	0.912127,
	0.906792,
	0.928133,
	0.928133,
	0.928133,
	0.928133,
	0.917463,
	0.928133,
	0.933469,
	0.922798,
	0.928133,
	0.917463,
	0.922798,
	0.928133,
	0.933469,
	0.933469,
	0.928133,
	0.928133,
	0.922798,
	0.922798,
	0.933469,
	0.933469,
	0.933469,
	0.928133,
	0.922798,
	0.928133,
	0.928133,
	0.933469,
	0.22387,
	0.869445,
	0.922798,
	0.917463,
	0.917463,
	0.928133,
	0.917463,
	0.901457,
	0.917463,
	0.912127,
	0.928133,
	0.933469,
	0.928133,
	0.832097,
	0.821427,
	0.346655,
	0.34453,
	0.34453,
	0.34453,
	0.34453,
	0.34453,
	0.34453,
	0.34453,
	0.34453,
	0.348779,
	0.34453,
	0.34453,
	0.34453,
	0.34453,
	0.34453,
	0.327533,
	0.34453,
	0.34453,
	0.34453,
	0.34453,
	0.34453,
	0.195806,
	0.189432,
	0.204305,
	0.214928,
	0.127818,
	0.248922,
	0.234049,
	0.282916,
	0.34453,
	0.272293,
	0.312661,
	0.340281,
	0.333907,
	0.295664,
	0.325408,
	0.312661,
	0.265919,
	0.265919,
	0.270168,
	0.291414,
	0.302038,
	0.253171,
	0.34453,
	0.348779,
	0.355153,
	0.348779,
	0.333907,
	0.348779,
	0.346655,
	0.350904,
	0.295664,
	0.312661,
	0.325408,
	0.325408,
	0.28929,
	0.319035,
	0.321159,
	0.31691,
	0.299913,
	0.31691,
	0.140566,
	0.274417,
	0.121444,
	0.214928,
	0.234049,
	0.151189,
	0.333907,
	0.327533,
	0.299913,
	0.248922,
	0.223426,
	0.306287,
	0.25742,
	0.314785,
	0.248922,
	0.365776,
	0.28929,
	0.319035,
	0.34453,
	0.355153,
	0.348779,
	0.306287,
	0.327533,
	0.34453,
	0.34453,
	0.333907,
	0.304162,
	0.31691,
	0.308411,
	0.274417,
	0.297788,
	0.304162,
	0.312661,
	0.331782,
	0.304162,
	0.329658,
	0.325408,
	0.325408,
	0.321159,
	0.308411,
	0.172435,
	0.282916,
	0.140566,
	0.125693,
	0.231925,
	0.17031,
	0.291414,
	0.123569,
	0.31691,
	0.0959483,
	0.270168,
	0.314785,
	0.25742,
	0.291414,
	0.338156,
	0.359403,
	0.321159,
	0.238299,
	0.350904,
	0.306287,
	0.348779,
	0.34453,
	0.331782,
	0.348779,
	0.359403,
	0.348779,
	0.304162,
	0.319035,
	0.310536,
	0.28929,
	0.280791,
	0.302038,
	0.31691,
	0.336032,
	0.310536,
	0.331782,
	0.331782,
	0.312661,
	0.331782,
	0.323284,
	0.155438,
	0.270168,
	0.127818,
	0.161812,
	0.333907,
	0.236174,
	0.26167,
	0.26167,
	0.291414,
	0.255296,
	0.28929,
	0.293539,
	0.304162,
	0.291414,
	0.25742,
	0.340281,
	0.325408,
	0.234049,
	0.355153,
	0.253171,
	0.350904,
	0.350904,
	0.319035,
	0.333907,
	0.359403,
	0.329658,
	0.336032,
	0.325408,
	0.302038,
	0.272293,
	0.28929,
	0.297788,
	0.327533,
	0.325408,
	0.306287,
	0.333907,
	0.338156,
	0.331782,
	0.327533,
	0.312661,
	0.151189,
	0.242548,
	0.11507,
	0.327533,
	0.214928,
	0.2298,
	0.276542,
	0.153313,
	0.304162,
	0.225551,
	0.231925,
	0.282916,
	0.244672,
	0.287165,
	0.265919,
	0.365776,
	0.2298,
	0.34453,
	0.253171,
	0.338156,
	0.34453,
	0.340281,
	0.346655,
	0.346655,
	0.361527,
	0.355153,
	0.31691,
	0.323284,
	0.306287,
	0.278667,
	0.282916,
	0.282916,
	0.325408,
	0.348779,
	0.338156,
	0.325408,
	0.308411,
	0.321159,
	0.312661,
	0.319035,
	0.149064,
	0.268043,
	0.11507,
	0.20218,
	0.323284,
	0.280791,
	0.274417,
	0.246797,
	0.282916,
	0.253171,
	0.25742,
	0.255296,
	0.299913,
	0.28929,
	0.323284,
	0.361527,
	0.28929,
	0.340281,
	0.333907,
	0.219177,
	0.333907,
	0.325408,
	0.329658,
	0.323284,
	0.363652,
	0.340281,
	0.323284,
	0.327533,
	0.308411,
	0.287165,
	0.259545,
	0.31691,
	0.327533,
	0.333907,
	0.325408,
	0.325408,
	0.325408,
	0.325408,
	0.321159,
	0.312661,
	0.140566,
	0.274417,
	0.121444,
	0.323284,
	0.274417,
	0.297788,
	0.282916,
	0.157563,
	0.265919,
	0.278667,
	0.306287,
	0.287165,
	0.293539,
	0.28929,
	0.176684,
	0.587176,
	0.587176,
	0.587176,
	0.587176,
	0.587176,
	0.587176,
	0.587176,
	0.587176,
	0.587176,
	0.587176,
	0.587176,
	0.587176,
	0.587176,
	0.587176,
	0.587176,
	0.581924,
	0.587176,
	0.587176,
	0.587176,
	0.587176,
	0.587176,
	0.083027,
	0.272083,
	0.487396,
	0.503151,
	0.513654,
	0.571421,
	0.487396,
	0.487396,
	0.524157,
	0.424378,
	0.392868,
	0.335101,
	0.576673,
	0.581924,
	0.39812,
	0.539912,
	0.545163,
	0.592427,
	0.53466,
	0.555666,
	0.576673,
	0.581924,
	0.581924,
	0.592427,
	0.576673,
	0.581924,
	0.576673,
	0.56617,
	0.524157,
	0.550415,
	0.503151,
	0.487396,
	0.513654,
	0.508403,
	0.545163,
	0.539912,
	0.513654,
	0.476893,
	0.476893,
	0.518906,
	0.0987816,
	0.177555,
	0.581924,
	0.576673,
	0.581924,
	0.576673,
	0.513654,
	0.550415,
	0.513654,
	0.335101,
	0.571421,
	0.524157,
	0.56617,
	0.539912,
	0.487396,
	0.592427,
	0.597679,
	0.581924,
	0.539912,
	0.518906,
	0.576673,
	0.581924,
	0.592427,
	0.581924,
	0.581924,
	0.571421,
	0.571421,
	0.56617,
	0.560918,
	0.571421,
	0.539912,
	0.524157,
	0.56617,
	0.550415,
	0.587176,
	0.576673,
	0.587176,
	0.592427,
	0.56617,
	0.503151,
	0.0987816,
	0.203813,
	0.581924,
	0.581924,
	0.576673,
	0.581924,
	0.508403,
	0.550415,
	0.371862,
	0.508403,
	0.487396,
	0.53466,
	0.60293,
	0.592427,
	0.461139,
	0.587176,
	0.587176,
	0.576673,
	0.529409,
	0.529409,
	0.571421,
	0.587176,
	0.60293,
	0.576673,
	0.592427,
	0.592427,
	0.581924,
	0.576673,
	0.550415,
	0.581924,
	0.529409,
	0.53466,
	0.581924,
	0.571421,
	0.581924,
	0.571421,
	0.576673,
	0.592427,
	0.46639,
	0.571421,
	0.0987816,
	0.214316,
	0.581924,
	0.592427,
	0.56617,
	0.560918,
	0.524157,
	0.581924,
	0.461139,
	0.503151,
	0.487396,
	0.518906,
	0.576673,
	0.539912,
	0.518906,
	0.53466,
	0.56617,
	0.581924,
	0.503151,
	0.539912,
	0.56617,
	0.597679,
	0.581924,
	0.581924,
	0.60293,
	0.581924,
	0.576673,
	0.56617,
	0.560918,
	0.571421,
	0.508403,
	0.56617,
	0.581924,
	0.576673,
	0.581924,
	0.571421,
	0.576673,
	0.592427,
	0.46639,
	0.508403,
	0.0935301,
	0.251077,
	0.581924,
	0.424378,
	0.539912,
	0.56617,
	0.508403,
	0.581924,
	0.53466,
	0.324598,
	0.476893,
	0.503151,
	0.581924,
	0.571421,
	0.560918,
	0.592427,
	0.592427,
	0.576673,
	0.550415,
	0.555666,
	0.56617,
	0.581924,
	0.592427,
	0.597679,
	0.60293,
	0.571421,
	0.550415,
	0.545163,
	0.555666,
	0.581924,
	0.555666,
	0.555666,
	0.592427,
	0.581924,
	0.581924,
	0.56617,
	0.581924,
	0.581924,
	0.581924,
	0.560918,
	0.0777754,
	0.23007,
	0.581924,
	0.555666,
	0.581924,
	0.571421,
	0.524157,
	0.539912,
	0.545163,
	0.555666,
	0.503151,
	0.550415,
	0.560918,
	0.555666,
	0.371862,
	0.513654,
	0.592427,
	0.576673,
	0.56617,
	0.476893,
	0.56617,
	0.581924,
	0.576673,
	0.592427,
	0.592427,
	0.581924,
	0.53466,
	0.560918,
	0.555666,
	0.571421,
	0.509436,
	0.509436,
	0.509436,
	0.509436,
	0.509436,
	0.509436,
	0.509436,
	0.509436,
	0.509436,
	0.509436,
	0.509436,
	0.509436,
	0.509436,
	0.509436,
	0.509436,
	0.509436,
	0.509436,
	0.509436,
	0.509436,
	0.509436,
	0.509436,
	0,
	0.198346,
	0.56647,
	0.54573,
	0.576839,
	0.563877,
	0.54573,
	0.537953,
	0.540545,
	0.493882,
	0.475735,
	0.447218,
	0.478327,
	0.54573,
	0.535361,
	0.537953,
	0.46018,
	0.558692,
	0.512029,
	0.488697,
	0.530176,
	0.54573,
	0.504252,
	0.537953,
	0.524991,
	0.509436,
	0.5561,
	0.517214,
	0.54573,
	0.519806,
	0.569062,
	0.54573,
	0.537953,
	0.535361,
	0.491289,
	0.506844,
	0.509436,
	0.5561,
	0.587209,
	0.524991,
	0,
	0.454996,
	0.385,
	0.522399,
	0.473143,
	0.423887,
	0.486105,
	0.550915,
	0.56647,
	0.532768,
	0.524991,
	0.369446,
	0.54573,
	0.537953,
	0.452403,
	0.429071,
	0.488697,
	0.372038,
	0.418702,
	0.372038,
	0.54573,
	0.5561,
	0.527583,
	0.54573,
	0.558692,
	0.540545,
	0.524991,
	0.558692,
	0.561285,
	0.550915,
	0.571654,
	0.553508,
	0.537953,
	0.517214,
	0.496474,
	0.530176,
	0.537953,
	0.537953,
	0.553508,
	0.54573,
	0.00909942,
	0.47055,
	0.530176,
	0.364261,
	0.506844,
	0.315005,
	0.530176,
	0.5561,
	0.540545,
	0.483512,
	0.597579,
	0.527583,
	0.535361,
	0.532768,
	0.473143,
	0.379815,
	0.436849,
	0.47055,
	0.473143,
	0.478327,
	0.537953,
	0.569062,
	0.530176,
	0.561285,
	0.558692,
	0.530176,
	0.524991,
	0.558692,
	0.589801,
	0.571654,
	0.561285,
	0.561285,
	0.54573,
	0.524991,
	0.517214,
	0.537953,
	0.527583,
	0.563877,
	0.563877,
	0.532768,
	0.00391458,
	0.431664,
	0.506844,
	0.400555,
	0.512029,
	0,
	0.483512,
	0.46018,
	0.571654,
	0.524991,
	0.563877,
	0.532768,
	0.5561,
	0.519806,
	0.379815,
	1,
	0.250194,
	0.353891,
	0.486105,
	0.504252,
	0.532768,
	0.543138,
	0.535361,
	0.54573,
	0.553508,
	0.543138,
	0.522399,
	0.543138,
	0.563877,
	0.56647,
	0.54573,
	0.563877,
	0.524991,
	0.54573,
	0.519806,
	0.54573,
	0.553508,
	0.561285,
	0.571654,
	0.563877,
	0,
	0.346114,
	0.392778,
	0.5561,
	0.527583,
	0.439441,
	0.452403,
	0.532768,
	0.5561,
	0.527583,
	0.56647,
	0.408332,
	0.569062,
	0.457588,
	0.462773,
	0.423887,
	0.416109,
	0.475735,
	0.535361,
	0.522399,
	0.532768,
	0.524991,
	0.532768,
	0.54573,
	0.561285,
	0.499067,
	0.535361,
	0.558692,
	0.558692,
	0.5561,
	0.537953,
	0.54573,
	0.553508,
	0.561285,
	0.563877,
	0.574247,
	0.5561,
	0.558692,
	0.569062,
	0.582024,
	0,
	0.338337,
	0.449811,
	0.579432,
	0.478327,
	0.408332,
	0.478327,
	0.496474,
	0.558692,
	0.527583,
	0,
	0.527583,
	0.550915,
	0.530176,
	0.493882,
	0.499067,
	0.486105,
	0.540545,
	0.530176,
	0.535361,
	0.558692,
	0.537953,
	0.519806,
	0.540545,
	0.558692,
	0.550915,
	0.550915,
	0.54573,
	0.561285,
	0.548323,
	0.56647,
	0.553508,
	0.54573,
	0.558692,
	0.54573,
	0.582024,
	0.550915,
	0.54573,
	0.571654,
	0.5561,
	0,
	0.32019,
	0.281303,
	0.524991,
	0.304635,
	0.504252,
	0.509436,
	0.517214,
	0.54573,
	0.452403,
	0.514621,
	0.491289,
	0.442034,
	0.491289,
	0.46018,
	0.234126,
	0.237318,
	0.237318,
	0.234126,
	0.237318,
	0.234126,
	0.237318,
	0.237318,
	0.240511,
	0.234126,
	0.237318,
	0.237318,
	0.234126,
	0.237318,
	0.237318,
	0.237318,
	0.237318,
	0.237318,
	0.237318,
	0.234126,
	0.237318,
	0.131971,
	0.246895,
	0.240511,
	0.176664,
	0.19901,
	0.214972,
	0.259665,
	0.259665,
	0.240511,
	0.237318,
	0.246895,
	0.227741,
	0.240511,
	0.246895,
	0.243703,
	0.259665,
	0.237318,
	0.234126,
	0.237318,
	0.234126,
	0.269242,
	0.269242,
	0.262857,
	0.259665,
	0.259665,
	0.25328,
	0.259665,
	0.243703,
	0.234126,
	0.246895,
	0.243703,
	0.227741,
	0.234126,
	0.250088,
	0.237318,
	0.202203,
	0.218164,
	0.230934,
	0.246895,
	0.25328,
	0.0681245,
	0.246895,
	0.237318,
	0.21178,
	0.205395,
	0.259665,
	0.262857,
	0.250088,
	0.230934,
	0.243703,
	0.259665,
	0.259665,
	0.243703,
	0.234126,
	0.240511,
	0.250088,
	0.240511,
	0,
	0.234126,
	0.240511,
	0.259665,
	0.25328,
	0.240511,
	0.237318,
	0.240511,
	0.243703,
	0.243703,
	0.243703,
	0.246895,
	0.262857,
	0.250088,
	0.234126,
	0.237318,
	0.262857,
	0.250088,
	0.224549,
	0.234126,
	0.224549,
	0.250088,
	0.243703,
	0.0649322,
	0.25328,
	0.230934,
	0.224549,
	0.240511,
	0.243703,
	1,
	0.234126,
	0.218164,
	0.208587,
	0.262857,
	0.250088,
	0.256472,
	0.250088,
	0.237318,
	0.250088,
	0.234126,
	0.259665,
	0.25328,
	0.266049,
	0.262857,
	0.250088,
	0.250088,
	0.240511,
	0.243703,
	0.25328,
	0.243703,
	0.259665,
	0.243703,
	0.250088,
	0.230934,
	0.234126,
	0.237318,
	0.266049,
	0.246895,
	0.240511,
	0.234126,
	0.240511,
	0.246895,
	0.243703,
	0.0745092,
	0.246895,
	0.237318,
	0.234126,
	0.259665,
	0.240511,
	0.224549,
	0.246895,
	0.237318,
	0.234126,
	0.224549,
	0.224549,
	0.224549,
	0.221357,
	0.243703,
	0.240511,
	0.240511,
	0.259665,
	0.259665,
	0.259665,
	0.262857,
	0.25328,
	0.25328,
	0.243703,
	0.246895,
	0.259665,
	0.237318,
	0.240511,
	0.262857,
	0.262857,
	0.259665,
	0.243703,
	0.227741,
	0.240511,
	0.269242,
	0.243703,
	0.224549,
	0.234126,
	0.240511,
	0.243703,
	0.0681245,
	0.234126,
	0.237318,
	0.205395,
	0.11601,
	0.266049,
	0.224549,
	0.246895,
	0.243703,
	0.25328,
	0.259665,
	0.262857,
	0.25328,
	0.221357,
	0.227741,
	0.224549,
	0.227741,
	0.234126,
	0.259665,
	0.25328,
	0.262857,
	0.259665,
	0.262857,
	0.250088,
	0.243703,
	0.259665,
	0.250088,
	0.246895,
	0.240511,
	0.262857,
	0.259665,
	0.246895,
	0.237318,
	0.25328,
	0.243703,
	0.243703,
	0.224549,
	0.224549,
	0.240511,
	0.234126,
	0.0585475,
	0.25328,
	0.109625,
	0.173472,
	0.15751,
	0.221357,
	0.240511,
	0.237318,
	0.230934,
	0.234126,
	0.262857,
	0.259665,
	0.259665,
	0.250088,
	0.25328,
	0.250088,
	0.227741,
	0.262857,
	0.266049,
	0.25328,
	0.259665,
	0.259665,
	0.259665,
	0.21178,
	0.230934,
	0.259665,
	0.243703,
	0.250088,
	0.259665,
	0.266049,
	0.256472,
	0.243703,
	0.234126,
	0.25328,
	0.25328,
	0.256472,
	0.21178,
	0.218164,
	0.240511,
	0.230934,
	0.0649322,
	0.243703,
	0.112817,
	0.25328,
	0.192626,
	0.237318,
	0.250088,
	0.234126,
	0.237318,
	0.25328,
	0.259665,
	0.25328,
	0.250088,
	0.243703,
	0.259665,
	0.523001,
	0.525502,
	0.523001,
	0.523001,
	0.523001,
	0.523001,
	0.523001,
	0.523001,
	0.523001,
	0.523001,
	1,
	0.523001,
	0.523001,
	0.523001,
	0.523001,
	0.523001,
	0.523001,
	0.523001,
	0.523001,
	0.523001,
	0.523001,
	0,
	0.292868,
	0.497986,
	0.482978,
	0.525502,
	0.475473,
	0.502989,
	0.450459,
	0.487981,
	0.482978,
	0.475473,
	0.475473,
	0.427946,
	0.43545,
	0.502989,
	0.455462,
	0.457963,
	0.523001,
	0.523001,
	0.517998,
	0.545514,
	0.545514,
	0.538009,
	0.543012,
	0.523001,
	0.530505,
	0.538009,
	0.525502,
	0.422943,
	0.485479,
	0.507992,
	1,
	0.538009,
	0.495485,
	0.510494,
	0.472972,
	0.462966,
	0.487981,
	0.482978,
	0.507992,
	0,
	0.22533,
	0.480476,
	0.480476,
	0.450459,
	0.440453,
	0.480476,
	0.515496,
	0.520499,
	0.555519,
	0.517998,
	0.535508,
	0.477975,
	0.485479,
	0,
	0.512995,
	0.442955,
	0.457963,
	0.360407,
	0.465468,
	0.563024,
	0.457963,
	0.512995,
	0.523001,
	0.530505,
	0.502989,
	0.500488,
	0.515496,
	0.540511,
	0.545514,
	0.535508,
	0.543012,
	0.533006,
	0.497986,
	0.517998,
	0.492983,
	0.475473,
	0.457963,
	0.500488,
	0.495485,
	0,
	0.110263,
	0.45296,
	0.360407,
	0.495485,
	0.457963,
	0.507992,
	0.460465,
	0.465468,
	0.507992,
	0.290367,
	0.285364,
	0.495485,
	0.205318,
	0.210321,
	0.465468,
	0.33039,
	0.33039,
	0.512995,
	0.480476,
	0.530505,
	0.510494,
	0.550517,
	0.538009,
	0.525502,
	0.512995,
	0.490482,
	0.507992,
	0.505491,
	0.430448,
	0.500488,
	0.497986,
	0.517998,
	0.517998,
	0.490482,
	0.492983,
	0.470471,
	0.510494,
	0.500488,
	0.482978,
	0,
	0.135278,
	0.510494,
	0.320384,
	0.335393,
	0.495485,
	0.502989,
	0.485479,
	0.427946,
	0.462966,
	0.360407,
	0.345399,
	0.507992,
	1,
	0.437952,
	0.357906,
	0.250344,
	0.502989,
	0.41794,
	0.495485,
	0.543012,
	0.517998,
	0.550517,
	0.538009,
	0.538009,
	0.525502,
	0.497986,
	0.540511,
	0.523001,
	0.507992,
	0.492983,
	0.492983,
	0.538009,
	0.517998,
	0.490482,
	0.500488,
	0.482978,
	0.515496,
	0,
	0.490482,
	0,
	0.102759,
	0.530505,
	0.407935,
	0.515496,
	0.507992,
	0.457963,
	0.442955,
	0.355404,
	0.565525,
	0.457963,
	0.533006,
	0.500488,
	0.352903,
	0.437952,
	0.520499,
	0.495485,
	0.510494,
	0.545514,
	0.533006,
	0.568027,
	0.543012,
	0.515496,
	0.530505,
	0.558021,
	0.500488,
	0.507992,
	0.540511,
	0.530505,
	0.533006,
	0.545514,
	0.525502,
	0.530505,
	0.500488,
	0.517998,
	0.490482,
	0.505491,
	0.515496,
	0.500488,
	0.43545,
	0,
	0.145284,
	0.495485,
	0.482978,
	0.512995,
	0.455462,
	0.500488,
	0.520499,
	0.550517,
	0.545514,
	0.528004,
	0.385422,
	0.487981,
	0.507992,
	0.475473,
	0.495485,
	0.517998,
	0.538009,
	0.540511,
	0.555519,
	0.550517,
	0.548015,
	1,
	0.535508,
	0.520499,
	0.497986,
	0.512995,
	0.517998,
	0.517998,
	0.538009,
	0.553018,
	0.555519,
	0.555519,
	0.490482,
	0.523001,
	0.502989,
	0.445456,
	0.487981,
	0.475473,
	0.490482,
	0,
	0.112765,
	0.525502,
	0.477975,
	0.445456,
	0.477975,
	0.490482,
	0.487981,
	0.545514,
	0.550517,
	0.550517,
	0.495485,
	0.517998,
	0.535508,
	0.500488};

	const float m_GraySamplesCenter[4205] = {
	0.23196,
	0.23196,
	0.23196,
	0.23196,
	0.23196,
	0.23196,
	0.23196,
	0.23196,
	0.23196,
	0.22996,
	0.23196,
	0.23196,
	0.23196,
	0.23196,
	0.23196,
	0.22996,
	0.22596,
	0.23196,
	0.23196,
	0.23196,
	0.23196,
	0.02396,
	0.23996,
	0.22996,
	0.20596,
	0.21796,
	0.24196,
	0.19596,
	0.20796,
	0.19796,
	0.20396,
	0.20196,
	0.13596,
	0.18396,
	0.13796,
	0.18396,
	0.19396,
	0.21796,
	0.18196,
	0.20396,
	0.21996,
	0.22596,
	0.21996,
	0.21196,
	0.20796,
	0.21396,
	0.19596,
	0.21796,
	0.20396,
	0.22796,
	0.22996,
	0.22396,
	0.22596,
	0.22396,
	0.22796,
	0.23396,
	0.21796,
	0.22996,
	0.21996,
	0.21796,
	0.21996,
	0.01196,
	0.19796,
	0.17996,
	0.21996,
	0.21996,
	0.23196,
	0.22996,
	0.23796,
	0.20796,
	0.23796,
	0.20196,
	0.23796,
	0.12996,
	0.18996,
	0.12196,
	0.17996,
	0.21996,
	0.17396,
	0.16396,
	0.21396,
	0.22396,
	0.20796,
	0.20796,
	0.20396,
	0.23196,
	0.20396,
	0.20396,
	0.22396,
	0.23596,
	0.23196,
	0.23596,
	0.22796,
	0.23396,
	0.23996,
	0.23996,
	0.21996,
	0.23396,
	0.23796,
	0.21996,
	0.21996,
	0.00796002,
	0.22996,
	0.18996,
	0.16596,
	0.21996,
	0.22996,
	0.23596,
	0.19596,
	0.18796,
	0.23596,
	0.20996,
	0.21196,
	0.20196,
	0.19996,
	0.21996,
	0.07596,
	0.05196,
	0.14996,
	0.08596,
	0.03996,
	0.21996,
	0.19396,
	0.21796,
	0.22196,
	0.21396,
	0.21996,
	0.21996,
	0.21396,
	0.21996,
	0.22796,
	0.21996,
	0.21796,
	0.21996,
	0.22796,
	0.21996,
	0.21996,
	0.16196,
	0.22196,
	0.21396,
	0.20796,
	0.01196,
	0.20596,
	0.12596,
	0.17396,
	0.22796,
	0.21996,
	0.23196,
	0.17996,
	0.21796,
	0.19796,
	0.18996,
	0.22796,
	0.19396,
	0.18996,
	0.20196,
	0.21996,
	0.17596,
	0.14796,
	0.03596,
	0.05796,
	0.22796,
	0.21596,
	0.21196,
	0.20796,
	0.22396,
	0.22196,
	0.23796,
	0.21396,
	0.23396,
	0.23196,
	0.21396,
	0.20996,
	0.21196,
	0.23196,
	0.23796,
	0.21596,
	0.16396,
	0.15596,
	0.23196,
	0.23596,
	0.01596,
	0.23196,
	0.07396,
	0.07196,
	0.22396,
	0.20196,
	0.23596,
	0.18396,
	0.17396,
	0.23996,
	0.23196,
	0.21396,
	0.21596,
	0.17796,
	0.20596,
	0.21596,
	0.21196,
	0.19996,
	0.21996,
	0.20396,
	0.22396,
	0.21996,
	0.21796,
	0.21196,
	0.23396,
	0.23996,
	0.23596,
	0.21396,
	0.22396,
	0.23396,
	0.22796,
	0.22396,
	0.21996,
	0.21796,
	0.23396,
	0.21196,
	0.21996,
	0.16196,
	0.20796,
	0.20196,
	0.01396,
	0.20796,
	0.10996,
	0.14796,
	0.21996,
	0.21996,
	0.20996,
	0.22396,
	0.18196,
	0.19596,
	0.22796,
	0.20796,
	0.21396,
	0.20196,
	0.18396,
	0.10196,
	0.09196,
	0.399222,
	0.388872,
	0.388872,
	0.388872,
	0.386802,
	0.388872,
	0.388872,
	0.399222,
	0.388872,
	0.388872,
	0.388872,
	0.388872,
	0.388872,
	0.372312,
	0.386802,
	0.388872,
	0.388872,
	0.388872,
	0.388872,
	0.386802,
	0.388872,
	0.0307603,
	0.343331,
	0.287441,
	0.399222,
	0.384732,
	0.266741,
	0.351611,
	0.275021,
	0.250181,
	0.252251,
	0.343331,
	0.399222,
	0.0121302,
	0.192221,
	0.337121,
	0.355752,
	0.364032,
	0.328841,
	0.339191,
	0.152891,
	0.366102,
	0.374382,
	0.345401,
	0.374382,
	0.372312,
	0.364032,
	0.399222,
	0.372312,
	0.368172,
	0.376452,
	0.376452,
	0.388872,
	0.368172,
	0.382662,
	0.376452,
	0.380592,
	0.386802,
	0.345401,
	0.345401,
	0.359892,
	0.0204102,
	0.291581,
	0.202571,
	0.134261,
	0.372312,
	0.260531,
	0.347471,
	0.328841,
	0.312281,
	0.287441,
	0.301931,
	0.345401,
	0.266741,
	0.335051,
	0.225341,
	0.382662,
	0.254321,
	0.364032,
	0.243971,
	0.347471,
	0.388872,
	0.364032,
	0.364032,
	0.384732,
	0.361962,
	0.295721,
	0.337121,
	0.332981,
	0.341261,
	0.359892,
	0.386802,
	0.386802,
	0.388872,
	0.399222,
	0.380592,
	0.399222,
	0.351611,
	0.337121,
	0.328841,
	0.345401,
	0.0390403,
	0.345401,
	0.214991,
	0.405432,
	0.297791,
	0.359892,
	0.361962,
	0.293651,
	0.399222,
	0.407502,
	0.341261,
	0.332981,
	0.268811,
	0.285371,
	0.256391,
	0.140471,
	0.227411,
	0.364032,
	0.260531,
	0.241901,
	0.399222,
	0.376452,
	0.378522,
	0.378522,
	0.372312,
	0.343331,
	0.328841,
	0.345401,
	0.353682,
	0.380592,
	0.374382,
	0.388872,
	0.374382,
	0.376452,
	0.384732,
	0.401292,
	0.364032,
	0.339191,
	0.328841,
	0.341261,
	0.0390403,
	0.337121,
	0.289511,
	0.376452,
	0.376452,
	0.370242,
	0.378522,
	0.390942,
	0.339191,
	0.285371,
	0.357822,
	0.293651,
	0.357822,
	0.343331,
	0.258461,
	0.378522,
	0.0990705,
	0.0328303,
	0.113561,
	0.386802,
	0.399222,
	0.388872,
	0.382662,
	0.374382,
	0.376452,
	0.275021,
	0.326771,
	0.310211,
	0.328841,
	0.370242,
	0.399222,
	0.378522,
	0.368172,
	0.386802,
	0.386802,
	0.382662,
	0.320561,
	0.343331,
	0.328841,
	0.351611,
	0.0390403,
	0.353682,
	0.0535304,
	0,
	0.107351,
	0.361962,
	0.335051,
	0.364032,
	0.295721,
	0.376452,
	0.270881,
	0.341261,
	0.341261,
	0.301931,
	0.115631,
	0.0597404,
	0.217061,
	0.332981,
	0.320561,
	0.229481,
	0.399222,
	0.399222,
	0.399222,
	0.386802,
	0.388872,
	0.332981,
	0.339191,
	0.366102,
	0.335051,
	0.328841,
	0.388872,
	0.380592,
	0.372312,
	0.372312,
	0.376452,
	0.388872,
	0.351611,
	0.328841,
	0.372312,
	0.341261,
	0.0369703,
	0.372312,
	0.268811,
	0.384732,
	0.339191,
	0.347471,
	0.372312,
	0.349541,
	0.384732,
	0.353682,
	0.223271,
	0.351611,
	0.252251,
	0.316421,
	0.378522,
	0.304001,
	0.241901,
	0.345401,
	0.144611,
	0.285371,
	0.390942,
	0.388872,
	0.314351,
	0.399222,
	0.378522,
	0.347471,
	0.341261,
	0.316421,
	0.345401,
	0.366102,
	0.347471,
	0.382662,
	0.347471,
	0.372312,
	0.368172,
	0.368172,
	0.366102,
	0.374382,
	0.351611,
	0.372312,
	0.0307603,
	0.254321,
	0.270881,
	0.281231,
	0.361962,
	0.326771,
	0.337121,
	0.388872,
	0.272951,
	0.411642,
	0.217061,
	0.260531,
	0.229481,
	0.186011,
	0.281231,
	0.357685,
	0.359894,
	0.359894,
	0.359894,
	0.357685,
	0.359894,
	0.357685,
	0.362102,
	0.359894,
	0.359894,
	0.359894,
	0.359894,
	0.359894,
	0.357685,
	0.359894,
	0.357685,
	0.359894,
	0.359894,
	0.373145,
	0.359894,
	0.359894,
	0.015371,
	0.386396,
	0.342226,
	0.22076,
	0.333392,
	0.346643,
	0.300265,
	0.331184,
	0.326767,
	0.342226,
	0.331184,
	0.362102,
	0.346643,
	0.311307,
	0.313516,
	0.344435,
	0.348852,
	0.317933,
	0.366519,
	0.320141,
	0.357685,
	0.370936,
	0.359894,
	0.326767,
	0.373145,
	0.366519,
	0.397438,
	0.390813,
	0.37977,
	0.384187,
	0.331184,
	0.313516,
	0.331184,
	0.331184,
	0.324558,
	0.342226,
	0.337809,
	0.357685,
	0.340018,
	0.335601,
	0.0286219,
	0.216343,
	0.377562,
	0.37977,
	0.362102,
	0.373145,
	0.357685,
	0.373145,
	0.313516,
	0.348852,
	0.311307,
	0.326767,
	0.390813,
	0.238428,
	0.388604,
	0.317933,
	0.368728,
	0.313516,
	0.364311,
	0.370936,
	0.37977,
	0.373145,
	0.377562,
	0.377562,
	0.390813,
	0.340018,
	0.346643,
	0.375353,
	0.373145,
	0.370936,
	0.359894,
	0.324558,
	0.373145,
	0.373145,
	0.359894,
	0.359894,
	0.362102,
	0.362102,
	0.35106,
	0.320141,
	0.00653708,
	0.189841,
	0.291431,
	0.348852,
	0.373145,
	0.364311,
	0.373145,
	0.359894,
	0.30689,
	0.381979,
	0.29364,
	0.295848,
	0.216343,
	0.381979,
	0.362102,
	0.35106,
	0.381979,
	0.370936,
	0.256095,
	0.384187,
	0.375353,
	0.375353,
	0.381979,
	0.373145,
	0.401855,
	0.37977,
	0.342226,
	0.381979,
	0.390813,
	0.386396,
	0.357685,
	0.370936,
	0.386396,
	0.384187,
	0.340018,
	0.368728,
	0.370936,
	0.381979,
	0.357685,
	0.357685,
	0.0175795,
	0.216343,
	0.346643,
	0.364311,
	0.368728,
	0.373145,
	0.37977,
	0.384187,
	0.342226,
	0.37977,
	0.386396,
	0.384187,
	0.357685,
	0.27818,
	0.399647,
	0.362102,
	0.346643,
	0.284806,
	0.342226,
	0.364311,
	0.357685,
	0.362102,
	0.370936,
	0.377562,
	0.386396,
	0.377562,
	0.373145,
	0.384187,
	0.381979,
	0.377562,
	0.364311,
	0.368728,
	0.384187,
	0.384187,
	0.359894,
	0.364311,
	0.373145,
	0.386396,
	0.368728,
	0.357685,
	0.010954,
	0.154505,
	0.320141,
	0.218551,
	0.357685,
	0.295848,
	0.401855,
	0.384187,
	0.344435,
	0.386396,
	0.37977,
	0.245053,
	0.386396,
	0.37977,
	0.390813,
	0.368728,
	0.377562,
	0.373145,
	0.353269,
	0.317933,
	0.368728,
	0.384187,
	0.368728,
	0.373145,
	0.390813,
	0.386396,
	0.362102,
	0.375353,
	0.397438,
	0.357685,
	0.373145,
	0.377562,
	0.39523,
	0.39523,
	0.373145,
	0.35106,
	0.373145,
	0.366519,
	0.366519,
	0.377562,
	0.00653708,
	0.134629,
	0.373145,
	0.373145,
	0.366519,
	0.359894,
	0.357685,
	0.366519,
	0.357685,
	0.373145,
	0.375353,
	0.324558,
	0.258304,
	0.203092,
	0.390813,
	0.331184,
	0.373145,
	0.373145,
	0.216343,
	0.238428,
	0.368728,
	0.370936,
	0.368728,
	0.381979,
	0.375353,
	0.377562,
	0.377562,
	0.388604,
	0.384187,
	0.366519,
	0.364311,
	0.381979,
	0.381979,
	0.384187,
	0.373145,
	0.390813,
	0.377562,
	0.364311,
	0.377562,
	0.375353,
	0.0043286,
	0.13242,
	0.390813,
	0.271555,
	0.346643,
	0.37977,
	0.37977,
	0.390813,
	0.346643,
	0.370936,
	0.366519,
	0.364311,
	0.287014,
	0.282597,
	0.280389,
	0.418584,
	0.418584,
	0.42483,
	0.420666,
	0.420666,
	0.418584,
	0.420666,
	0.420666,
	0.418584,
	0.420666,
	0.418584,
	0.420666,
	0.420666,
	0.420666,
	0.420666,
	0.428994,
	0.418584,
	0.420666,
	0.420666,
	0.420666,
	0.420666,
	0.0500718,
	0.181237,
	0.364452,
	0.360288,
	0.358206,
	0.358206,
	0.324894,
	0.283255,
	0.381108,
	0.364452,
	0.35196,
	0.395682,
	0.374862,
	0.322812,
	0.314484,
	0.38319,
	0.370698,
	0.37278,
	0.389436,
	0.285337,
	0.40401,
	0.395682,
	0.381108,
	0.399846,
	0.381108,
	0.381108,
	0.3936,
	0.385272,
	0.399846,
	0.387354,
	0.406092,
	0.399846,
	0.399846,
	0.416502,
	0.356124,
	0.418584,
	0.416502,
	0.406092,
	0.416502,
	0.416502,
	0.0667277,
	0.214549,
	0.3936,
	0.399846,
	0.395682,
	0.366534,
	0.333222,
	0.356124,
	0.197893,
	0.3936,
	0.285337,
	0.36237,
	0.314484,
	0.247861,
	0.381108,
	0.314484,
	0.308238,
	0.345714,
	0.34155,
	0.416502,
	0.40401,
	0.401928,
	0.3936,
	0.364452,
	0.368616,
	0.364452,
	0.35196,
	0.306156,
	0.381108,
	0.349878,
	0.401928,
	0.40401,
	0.401928,
	0.416502,
	0.418584,
	0.428994,
	0.406092,
	0.401928,
	0.408174,
	0.389436,
	0.0729737,
	0.247861,
	0.339468,
	0.418584,
	0.337386,
	0.416502,
	0.416502,
	0.360288,
	0.376944,
	0.356124,
	0.314484,
	0.349878,
	0.318648,
	0.322812,
	0.266599,
	0.197893,
	0.406092,
	0.264517,
	0.389436,
	0.408174,
	0.418584,
	0.3936,
	0.395682,
	0.381108,
	0.354042,
	0.354042,
	0.36237,
	0.349878,
	0.374862,
	0.370698,
	0.397764,
	0.399846,
	0.416502,
	0.401928,
	0.426912,
	0.420666,
	0.422748,
	0.391518,
	0.379026,
	0.397764,
	0.0771377,
	0.212467,
	0.354042,
	0.291582,
	0.401928,
	0.416502,
	0.431076,
	0.38319,
	0.406092,
	0.381108,
	0.34155,
	0.34155,
	0.312402,
	0.391518,
	0.345714,
	0.349878,
	0.408174,
	0.399846,
	0.401928,
	0.3936,
	0.401928,
	0.387354,
	0.381108,
	0.395682,
	0.366534,
	0.381108,
	0.347796,
	0.370698,
	0.374862,
	0.379026,
	0.376944,
	0.408174,
	0.416502,
	0.416502,
	0.426912,
	0.416502,
	0.416502,
	0.406092,
	0.408174,
	0.385272,
	0.0771377,
	0.245779,
	0.326976,
	0.420666,
	0.33114,
	0.426912,
	0.416502,
	0.418584,
	0.389436,
	0.387354,
	0.337386,
	0.376944,
	0.283255,
	0.349878,
	0.37278,
	0.397764,
	0.401928,
	0.374862,
	0.376944,
	0.3936,
	0.416502,
	0.399846,
	0.379026,
	0.391518,
	0.379026,
	0.376944,
	0.374862,
	0.374862,
	0.354042,
	0.349878,
	0.381108,
	0.408174,
	0.40401,
	0.426912,
	0.426912,
	0.422748,
	0.418584,
	0.418584,
	0.40401,
	0.387354,
	0.0750557,
	0.247861,
	0.381108,
	0.260353,
	0.428994,
	0.387354,
	0.397764,
	0.42483,
	0.3936,
	0.374862,
	0.391518,
	0.349878,
	0.379026,
	0.35196,
	0.374862,
	0.218713,
	0.389436,
	0.158335,
	0.0979575,
	0.204139,
	0.416502,
	0.408174,
	0.37278,
	0.38319,
	0.385272,
	0.385272,
	0.38319,
	0.389436,
	0.345714,
	0.387354,
	0.368616,
	0.389436,
	0.401928,
	0.389436,
	0.42483,
	0.426912,
	0.416502,
	0.43524,
	0.391518,
	0.379026,
	0.0833836,
	0.243697,
	0.322812,
	0.258271,
	0.426912,
	0.387354,
	0.399846,
	0.374862,
	0.385272,
	0.356124,
	0.360288,
	0.335304,
	0.374862,
	0.374862,
	0.426544,
	0.4364,
	0.426544,
	0.426544,
	0.418659,
	0.426544,
	0.426544,
	0.426544,
	0.426544,
	0.426544,
	0.426544,
	0.426544,
	0.426544,
	0.426544,
	0.426544,
	0.426544,
	0.426544,
	0.426544,
	0.426544,
	0.426544,
	0.426544,
	0.178175,
	0.418659,
	0.396976,
	0.412746,
	0.404861,
	0.363466,
	0.42063,
	0.430486,
	0.28659,
	0.375293,
	0.412746,
	0.395005,
	0.379236,
	0.410774,
	0.418659,
	0.395005,
	0.404861,
	0.385149,
	0.375293,
	0.389092,
	0.418659,
	0.408803,
	0.410774,
	0.42063,
	0.426544,
	0.42063,
	0.42063,
	0.424573,
	0.396976,
	0.432458,
	0.412746,
	0.422602,
	0.416688,
	0.424573,
	0.42063,
	0.389092,
	0.410774,
	0.398947,
	0.412746,
	0.4364,
	0.146636,
	0.430486,
	0.365437,
	0.404861,
	0.249138,
	0.251109,
	0.408803,
	0.412746,
	0.424573,
	0.42063,
	0.280677,
	0.389092,
	0.193945,
	0.197887,
	0.359524,
	0.229426,
	0.28659,
	0.412746,
	0.249138,
	0.422602,
	0.426544,
	0.428515,
	0.424573,
	0.426544,
	0.422602,
	0.404861,
	0.414717,
	0.414717,
	0.442313,
	0.375293,
	0.375293,
	0.422602,
	0.424573,
	0.414717,
	0.414717,
	0.416688,
	0.434429,
	0.412746,
	0.385149,
	0.396976,
	0.15255,
	0.385149,
	0.270821,
	0.460054,
	0.398947,
	0.351639,
	0.414717,
	0.426544,
	0.365437,
	0.343754,
	0.359524,
	0.426544,
	0.398947,
	0.30236,
	0.430486,
	0.188031,
	0.3201,
	0.359524,
	0.272792,
	0.428515,
	0.42063,
	0.430486,
	0.426544,
	0.422602,
	0.428515,
	0.416688,
	0.428515,
	0.414717,
	0.440342,
	0.40289,
	0.365437,
	0.375293,
	0.416688,
	0.412746,
	0.375293,
	0.432458,
	0.430486,
	0.434429,
	0.408803,
	0.408803,
	0.168319,
	0.432458,
	0.422602,
	0.337841,
	0.426544,
	0.432458,
	0.389092,
	0.355581,
	0.365437,
	0.379236,
	0.284619,
	0.430486,
	0.221541,
	0.347697,
	0.430486,
	0.414717,
	0.422602,
	0.351639,
	0.347697,
	0.412746,
	0.42063,
	0.426544,
	0.410774,
	0.426544,
	0.426544,
	0.424573,
	0.412746,
	0.408803,
	0.424573,
	0.444285,
	0.40289,
	0.395005,
	0.408803,
	0.42063,
	0.426544,
	0.430486,
	0.426544,
	0.430486,
	0.412746,
	0.414717,
	0.174233,
	0.418659,
	0.306302,
	0.444285,
	0.389092,
	0.377264,
	0.42063,
	0.395005,
	0.365437,
	0.416688,
	0.434429,
	0.30236,
	0.424573,
	0.190002,
	0.306302,
	0.393034,
	0.440342,
	0.357553,
	0.404861,
	0.42063,
	0.422602,
	0.430486,
	0.410774,
	0.442313,
	0.428515,
	0.414717,
	0.426544,
	0.379236,
	0.42063,
	0.430486,
	0.398947,
	0.404861,
	0.396976,
	0.408803,
	0.40289,
	0.432458,
	0.438371,
	0.424573,
	0.406832,
	0.398947,
	0.211685,
	0.383178,
	0.428515,
	0.258994,
	0.416688,
	0.326014,
	0.404861,
	0.418659,
	0.381207,
	0.345726,
	0.438371,
	0.296446,
	0.363466,
	0.318129,
	0.440342,
	0.190002,
	0.329956,
	0.314187,
	0.235339,
	0.424573,
	0.418659,
	0.428515,
	0.404861,
	0.442313,
	0.428515,
	0.428515,
	0.426544,
	0.416688,
	0.4364,
	0.446256,
	0.416688,
	0.424573,
	0.408803,
	0.408803,
	0.395005,
	0.4364,
	0.446256,
	0.424573,
	0.414717,
	0.40289,
	0.18606,
	0.424573,
	0.412746,
	0.316158,
	0.400919,
	0.40289,
	0.404861,
	0.404861,
	0.428515,
	0.363466,
	0.322071,
	0.375293,
	0.312215,
	0.333898,
	0.386433,
	0.386433,
	0.386433,
	0.386433,
	0.386433,
	0.384157,
	0.386433,
	0.386433,
	0.386433,
	0.386433,
	0.386433,
	0.375051,
	0.386433,
	0.386433,
	0.390986,
	0.386433,
	0.386433,
	0.386433,
	0.386433,
	0.386433,
	0.386433,
	0.0768496,
	0.206601,
	0.324972,
	0.245299,
	0.38188,
	0.190667,
	0.174733,
	0.211154,
	0.158798,
	0.151969,
	0.188391,
	0.28855,
	0.38188,
	0.311313,
	0.347735,
	0.375051,
	0.311313,
	0.231641,
	0.386433,
	0.23847,
	0.400091,
	0.372775,
	0.38188,
	0.375051,
	0.386433,
	0.388709,
	0.375051,
	0.400091,
	0.375051,
	0.386433,
	0.386433,
	0.375051,
	0.377328,
	0.379604,
	0.375051,
	0.375051,
	0.386433,
	0.384157,
	0.375051,
	0.38188,
	0.0677442,
	0.324972,
	0.23847,
	0.254405,
	0.345459,
	0.188391,
	0.188391,
	0.384157,
	0.302208,
	0.295379,
	0.375051,
	0.375051,
	0.290826,
	0.393262,
	0.377328,
	0.336353,
	0.340906,
	0.268063,
	0.281721,
	0.390986,
	0.397815,
	0.350011,
	0.38188,
	0.375051,
	0.38188,
	0.400091,
	0.395538,
	0.388709,
	0.390986,
	0.38188,
	0.375051,
	0.340906,
	0.375051,
	0.375051,
	0.390986,
	0.350011,
	0.375051,
	0.375051,
	0.379604,
	0.395538,
	0.0677442,
	0.299932,
	0.172456,
	0.0791259,
	0.138311,
	0.395538,
	0.172456,
	0.17018,
	0.375051,
	0.245299,
	0.390986,
	0.375051,
	0.33863,
	0.375051,
	0.347735,
	0.306761,
	0.243023,
	0.350011,
	0.377328,
	0.33863,
	0.386433,
	0.375051,
	0.375051,
	0.375051,
	0.375051,
	0.379604,
	0.379604,
	0.393262,
	0.386433,
	0.390986,
	0.386433,
	0.375051,
	0.375051,
	0.354564,
	0.375051,
	0.302208,
	0.343182,
	0.384157,
	0.33863,
	0.388709,
	0.0609151,
	0.329524,
	0.254405,
	0.177009,
	0.336353,
	0.256681,
	0.390986,
	0.243023,
	0.156522,
	0.142864,
	0.404644,
	0.247576,
	0.386433,
	0.322695,
	0.331801,
	0.384157,
	0.240747,
	0.388709,
	0.277168,
	0.375051,
	0.384157,
	0.375051,
	0.937309,
	0.384157,
	0.393262,
	0.402367,
	0.400091,
	0.402367,
	0.379604,
	0.384157,
	0.390986,
	0.390986,
	0.379604,
	0.38188,
	0.375051,
	0.336353,
	0.33863,
	0.384157,
	0.384157,
	0.375051,
	0.0449807,
	0.283997,
	0.375051,
	0.329524,
	0.204325,
	0.375051,
	0.324972,
	0.388709,
	0.304484,
	0.322695,
	0.315866,
	0.304484,
	0.375051,
	0.395538,
	0.402367,
	0.256681,
	0.379604,
	0.375051,
	0.336353,
	0.299932,
	0.390986,
	0.350011,
	0.375051,
	0.386433,
	0.388709,
	0.409196,
	0.388709,
	0.40692,
	0.395538,
	0.386433,
	0.375051,
	0.400091,
	0.38188,
	0.384157,
	0.329524,
	0.35684,
	0.377328,
	0.402367,
	0.377328,
	0.204325,
	0.0381516,
	0.224812,
	0.131482,
	0.099613,
	0.375051,
	0.092784,
	0.204325,
	0.38188,
	0.377328,
	0.295379,
	0.395538,
	0.227089,
	0.384157,
	0.122377,
	0.0814023,
	0.208878,
	0.322695,
	0.375051,
	0.172456,
	0.108718,
	0.388709,
	0.33598,
	0.33598,
	0.33598,
	0.33598,
	0.33598,
	0.33598,
	0.33598,
	0.33598,
	0.33598,
	0.338037,
	0.33598,
	0.33598,
	0.33598,
	0.33598,
	0.33598,
	0.33598,
	0.33598,
	0.33598,
	0.33598,
	0.33598,
	0.338037,
	0,
	0.1817,
	0.255755,
	0.331866,
	0.344208,
	0.323638,
	0.338037,
	0.204328,
	0.327752,
	0.346265,
	0.315409,
	0.342151,
	0.325695,
	0.329809,
	0.307181,
	0.333923,
	0.346265,
	0.344208,
	0.319524,
	0.342151,
	0.342151,
	0.327752,
	0.348322,
	0.344208,
	0.33598,
	0.319524,
	0.331866,
	0.348322,
	0.342151,
	0.342151,
	0.323638,
	0.313352,
	0.325695,
	0.35038,
	0.319524,
	0.325695,
	0.348322,
	0.315409,
	0.294839,
	0.315409,
	0,
	0.198157,
	0.268097,
	0.342151,
	0.21667,
	0.317466,
	0.352437,
	0.33598,
	0.327752,
	0.327752,
	0.288668,
	0.333923,
	0.292782,
	0.348322,
	0.348322,
	0.329809,
	0.311295,
	0.338037,
	0.319524,
	0.340094,
	0.327752,
	0.340094,
	0.344208,
	0.342151,
	0.340094,
	0.338037,
	0.317466,
	0.340094,
	0.333923,
	0.323638,
	0.342151,
	0.346265,
	0.352437,
	0.356551,
	0.340094,
	0.327752,
	0.352437,
	0.327752,
	0.294839,
	0.315409,
	0,
	0.290725,
	0.187872,
	0.307181,
	0.292782,
	0.261926,
	0.338037,
	0.344208,
	0.35038,
	0.352437,
	0.296896,
	0.1817,
	0.327752,
	0.319524,
	0.358608,
	0.292782,
	0.327752,
	0.331866,
	0.331866,
	0.327752,
	0.35038,
	0.342151,
	0.342151,
	0.348322,
	0.329809,
	0.360665,
	0.319524,
	0.346265,
	0.338037,
	0.340094,
	0.338037,
	0.315409,
	0.342151,
	0.360665,
	0.348322,
	0.348322,
	0.331866,
	0.333923,
	0.313352,
	0.338037,
	0.00479293,
	0.292782,
	0.1961,
	0.340094,
	0.35038,
	0.340094,
	0.30101,
	0.338037,
	0.342151,
	0.354494,
	0.340094,
	0.235184,
	0.296896,
	0.0623907,
	0.0665048,
	0.35038,
	0.257812,
	0.340094,
	0.333923,
	0.338037,
	0.342151,
	0.346265,
	0.342151,
	0.344208,
	0.344208,
	0.311295,
	0.333923,
	0.315409,
	0.333923,
	0.323638,
	0.333923,
	0.327752,
	0.348322,
	0.362722,
	0.352437,
	0.354494,
	0.338037,
	0.344208,
	0.329809,
	0.261926,
	0.0150782,
	0.333923,
	0.179643,
	0.1961,
	0.352437,
	0.317466,
	0.23107,
	0.338037,
	0.331866,
	0.327752,
	0.325695,
	0.319524,
	0.333923,
	0.338037,
	0.344208,
	0.311295,
	0.364779,
	0.327752,
	0.331866,
	0.303067,
	0.344208,
	0.352437,
	0.354494,
	0.352437,
	0.340094,
	0.331866,
	0.342151,
	0.333923,
	0.338037,
	0.323638,
	0.35038,
	0.344208,
	0.340094,
	0.348322,
	0.358608,
	0.356551,
	0.329809,
	0.342151,
	0.338037,
	0.294839,
	0.0294777,
	0.338037,
	0.185814,
	0.173472,
	0.340094,
	0.303067,
	0.1817,
	0.21667,
	0.249583,
	0.348322,
	0.33598,
	0.208442,
	0.296896,
	0.26604,
	0.342151,
	0.323638,
	0.329809,
	0.303067,
	0.315409,
	0.340094,
	0.342151,
	0.348322,
	0.352437,
	0.356551,
	0.344208,
	0.33598,
	0.333923,
	0.331866,
	0.346265,
	0.340094,
	0.33598,
	0.344208,
	0.346265,
	0.35038,
	0.354494,
	0.354494,
	0.329809,
	0.323638,
	0.296896,
	0.327752,
	0,
	0.222842,
	0.226956,
	0.218727,
	0.393858,
	0.388049,
	0.389985,
	0.391921,
	0.391921,
	0.40354,
	0.389985,
	0.391921,
	0.389985,
	0.391921,
	0.391921,
	0.391921,
	0.391921,
	0.391921,
	0.391921,
	0.353194,
	0.391921,
	0.391921,
	0.391921,
	0.391921,
	0.391921,
	0.105339,
	0.374494,
	0.250566,
	0.29123,
	0.366749,
	0.37643,
	0.329958,
	0.316403,
	0.378367,
	0.374494,
	0.360939,
	0.328021,
	0.335767,
	0.351258,
	0.347385,
	0.329958,
	0.380303,
	0.370621,
	0.370621,
	0.380303,
	0.384176,
	0.374494,
	0.366749,
	0.366749,
	0.364812,
	0.35513,
	0.359003,
	0.347385,
	0.360939,
	0.366749,
	0.366749,
	0.386112,
	0.37643,
	0.37643,
	0.386112,
	0.38224,
	0.378367,
	0.362876,
	0.326085,
	0.380303,
	0.0937203,
	0,
	0.320276,
	0.38224,
	0.389985,
	0.343512,
	0.388049,
	0.37643,
	0.360939,
	0.370621,
	0.326085,
	0.339639,
	0.364812,
	0.339639,
	0.339639,
	0.37643,
	0.38224,
	0.29123,
	0.37643,
	0.37643,
	0.37643,
	0.378367,
	0.372558,
	0.339639,
	0.370621,
	0.364812,
	0.359003,
	0.329958,
	0.349321,
	0.359003,
	0.374494,
	0.38224,
	0.37643,
	0.38224,
	0.384176,
	0.37643,
	0.380303,
	0.370621,
	0.37643,
	0.38224,
	0.0956567,
	0.26993,
	0.326085,
	0.389985,
	0.389985,
	0.29123,
	0.380303,
	0.38224,
	0.357067,
	0.357067,
	0.297039,
	0.351258,
	0.351258,
	0.366749,
	0.362876,
	0.378367,
	0.388049,
	0.389985,
	0.388049,
	0.37643,
	0.386112,
	0.38224,
	0.389985,
	0.380303,
	0.366749,
	0.380303,
	0.351258,
	0.324148,
	0.353194,
	0.349321,
	0.364812,
	0.366749,
	0.380303,
	0.389985,
	0.380303,
	0.380303,
	0.370621,
	0.37643,
	0.38224,
	0.374494,
	0.101466,
	0.252503,
	0.349321,
	0.391921,
	0.38224,
	0.335767,
	0.38224,
	0.370621,
	0.35513,
	0.35513,
	0.359003,
	0.326085,
	0.33383,
	0.38224,
	0.33383,
	0.374494,
	0.366749,
	0.360939,
	0.380303,
	0.271866,
	0.386112,
	0.388049,
	0.364812,
	0.399667,
	0.397731,
	0.370621,
	0.372558,
	0.351258,
	0.359003,
	0.353194,
	0.366749,
	0.366749,
	0.374494,
	0.378367,
	0.380303,
	0.370621,
	0.399667,
	0.38224,
	0.38224,
	0.380303,
	0.0879112,
	0.223457,
	0.372558,
	0.391921,
	0.389985,
	0.38224,
	0.347385,
	0.300912,
	0.337703,
	0.337703,
	0.289294,
	0.362876,
	0.364812,
	0.389985,
	0.389985,
	0.391921,
	0.378367,
	0.31253,
	0.38224,
	0.395794,
	0.389985,
	0.38224,
	0.370621,
	0.389985,
	0.380303,
	0.388049,
	0.380303,
	0.364812,
	0.357067,
	0.353194,
	0.35513,
	0.359003,
	0.374494,
	0.386112,
	0.374494,
	0.393858,
	0.391921,
	0.380303,
	0.374494,
	0.374494,
	0.0898476,
	0.240884,
	0.349321,
	0.405476,
	0.380303,
	0.320276,
	0.364812,
	0.320276,
	0.349321,
	0.351258,
	0.29123,
	0.341576,
	0.388049,
	0.388049,
	0.378367,
	0.362876,
	0.351258,
	0.455323,
	0.455323,
	0.455323,
	0.455323,
	0.455323,
	0.468584,
	0.455323,
	0.455323,
	0.455323,
	0.455323,
	0.455323,
	0.455323,
	0.461953,
	0.455323,
	0.455323,
	0.455323,
	0.461953,
	0.455323,
	0.455323,
	0.455323,
	0.455323,
	0.176851,
	0.442062,
	0.320507,
	0.338188,
	0.402281,
	0.419962,
	0.495105,
	0.497315,
	0.448693,
	0.461953,
	0.461953,
	0.444273,
	0.455323,
	0.470794,
	0.455323,
	0.406701,
	0.470794,
	0.435432,
	0.395651,
	0.442062,
	0.455323,
	0.459743,
	0.453113,
	0.466373,
	0.488474,
	0.486264,
	0.475214,
	0.448693,
	0.444273,
	0.481844,
	0.479634,
	0.497315,
	0.479634,
	0.481844,
	0.468584,
	0.446483,
	0.411121,
	0.395651,
	0.426592,
	0.439852,
	0.161381,
	0.448693,
	0.287356,
	0.260835,
	0.435432,
	0.442062,
	0.488474,
	0.450903,
	0.468584,
	0.468584,
	0.461953,
	0.461953,
	0.484054,
	0.413331,
	0.448693,
	0.479634,
	0.468584,
	0.450903,
	0.366919,
	0.448693,
	0.455323,
	0.455323,
	0.470794,
	0.473004,
	0.495105,
	0.499525,
	0.475214,
	0.455323,
	0.453113,
	0.481844,
	0.461953,
	0.488474,
	0.490684,
	0.479634,
	0.477424,
	0.450903,
	0.419962,
	0.406701,
	0.417751,
	0.419962,
	0.165801,
	0.417751,
	0.327138,
	0.276306,
	0.439852,
	0.439852,
	0.495105,
	0.450903,
	0.475214,
	0.497315,
	0.470794,
	0.470794,
	0.461953,
	0.473004,
	0.419962,
	0.38681,
	0.413331,
	0.470794,
	0.329348,
	0.453113,
	0.455323,
	0.470794,
	0.479634,
	0.461953,
	0.479634,
	0.486264,
	0.488474,
	0.466373,
	0.461953,
	0.475214,
	0.492895,
	0.497315,
	0.506155,
	0.481844,
	0.481844,
	0.444273,
	0.431012,
	0.433222,
	0.437642,
	0.448693,
	0.183482,
	0.459743,
	0.342608,
	0.404491,
	0.455323,
	0.402281,
	0.39123,
	0.426592,
	0.490684,
	0.473004,
	0.461953,
	0.481844,
	0.433222,
	0.481844,
	0.497315,
	0.422172,
	0.419962,
	0.431012,
	0.473004,
	0.450903,
	0.470794,
	0.484054,
	0.479634,
	0.475214,
	0.481844,
	0.473004,
	0.490684,
	0.488474,
	0.477424,
	0.488474,
	0.497315,
	0.501735,
	0.503945,
	0.484054,
	0.442062,
	0.437642,
	0.39344,
	0.424382,
	0.422172,
	0.461953,
	0.154751,
	0.360289,
	0.285146,
	0.274096,
	0.426592,
	0.411121,
	0.455323,
	0.404491,
	0.455323,
	0.411121,
	0.470794,
	0.486264,
	0.444273,
	0.433222,
	0.411121,
	0.477424,
	0.358079,
	0.459743,
	0.417751,
	0.455323,
	0.486264,
	0.475214,
	0.479634,
	0.470794,
	0.479634,
	0.479634,
	0.484054,
	0.484054,
	0.486264,
	0.470794,
	0.479634,
	0.495105,
	0.479634,
	0.461953,
	0.439852,
	0.431012,
	0.39344,
	0.435432,
	0.479634,
	0.477424,
	0.154751,
	0.349239,
	0.364709,
	0.3846,
	0.302827,
	0.455323,
	0.495105,
	0.347028,
	0.466373,
	0.431012,
	0.448693,
	0.481844,
	0.481844,
	0.484054,
	0.419962,
	0.442062,
	0.324928,
	0.422172,
	0.486264,
	0.298934,
	0.298934,
	0.298934,
	0.298934,
	0.298934,
	0.298934,
	0.301101,
	0.298934,
	0.298934,
	0.298934,
	0.305436,
	0.301101,
	0.301101,
	0.305436,
	0.301101,
	0.298934,
	0.298934,
	0.298934,
	0.298934,
	0.298934,
	0.301101,
	0.0908441,
	0.296766,
	0.225235,
	0.281593,
	0.294598,
	0.292431,
	0.296766,
	0.28376,
	0.288096,
	0.268587,
	0.290263,
	0.28376,
	0.28376,
	0.285928,
	0.257749,
	0.288096,
	0.288096,
	0.28376,
	0.28376,
	0.270755,
	0.277258,
	0.301101,
	0.290263,
	0.298934,
	0.279425,
	0.294598,
	0.307604,
	0.290263,
	0.294598,
	0.294598,
	0.294598,
	0.296766,
	0.290263,
	0.298934,
	0.298934,
	0.294598,
	0.298934,
	0.28376,
	0.290263,
	0.292431,
	0.0366541,
	0.168878,
	0.227403,
	0.259917,
	0.270755,
	0.294598,
	0.294598,
	0.262084,
	0.290263,
	0.285928,
	0.298934,
	0.298934,
	0.281593,
	0.292431,
	0.290263,
	0.277258,
	0.301101,
	0.294598,
	0.279425,
	0.28376,
	0.301101,
	0.296766,
	0.301101,
	0.301101,
	0.296766,
	0.288096,
	0.303269,
	0.298934,
	0.301101,
	0.303269,
	0.301101,
	0.301101,
	0.294598,
	0.292431,
	0.301101,
	0.281593,
	0.305436,
	0.294598,
	0.292431,
	0.281593,
	0.0388217,
	0.140699,
	0.296766,
	0.292431,
	0.298934,
	0.294598,
	0.301101,
	0.292431,
	0.294598,
	0.301101,
	0.294598,
	0.301101,
	0.296766,
	0.296766,
	0.305436,
	0.303269,
	0.311939,
	0.296766,
	0.296766,
	0.301101,
	0.314107,
	0.298934,
	0.298934,
	0.307604,
	0.292431,
	0.294598,
	0.309772,
	0.298934,
	0.301101,
	0.301101,
	0.301101,
	0.296766,
	0.298934,
	0.292431,
	0.305436,
	0.292431,
	0.296766,
	0.294598,
	0.303269,
	0.290263,
	0.0409893,
	0.132028,
	0.21223,
	0.253414,
	0.298934,
	0.244744,
	0.28376,
	0.279425,
	0.301101,
	0.294598,
	0.303269,
	0.301101,
	0.301101,
	0.294598,
	0.290263,
	0.240408,
	0.26642,
	0.28376,
	0.279425,
	0.285928,
	0.305436,
	0.294598,
	0.296766,
	0.298934,
	0.27509,
	0.288096,
	0.305436,
	0.294598,
	0.301101,
	0.290263,
	0.301101,
	0.296766,
	0.294598,
	0.292431,
	0.272922,
	0.28376,
	0.288096,
	0.296766,
	0.298934,
	0.290263,
	0.0366541,
	0.134196,
	0.277258,
	0.288096,
	0.288096,
	0.290263,
	0.294598,
	0.292431,
	0.288096,
	0.27509,
	0.28376,
	0.303269,
	0.272922,
	0.296766,
	0.298934,
	0.301101,
	0.303269,
	0.28376,
	0.26642,
	0.298934,
	0.301101,
	0.298934,
	0.301101,
	0.307604,
	0.309772,
	0.292431,
	0.309772,
	0.305436,
	0.305436,
	0.292431,
	0.294598,
	0.294598,
	0.288096,
	0.298934,
	0.290263,
	0.28376,
	0.288096,
	0.292431,
	0.294598,
	0.290263,
	0.0474921,
	0.129861,
	0.199224,
	0.199224,
	0.27509,
	0.294598,
	0.305436,
	0.285928,
	0.294598,
	0.294598,
	0.298934,
	0.303269,
	0.292431,
	0.255582,
	0.296766,
	0.301101,
	0.294598,
	0.216565,
	0.272922,
	0.26642,
	0.28376,
	0.292431,
	0.290263,
	0.303269,
	0.298934,
	0.292431,
	0.301101,
	0.298934,
	0.724293,
	0.724293,
	0.726896,
	0.724293,
	0.724293,
	0.724293,
	0.737309,
	0.724293,
	0.724293,
	0.726896,
	0.724293,
	0.724293,
	0.724293,
	0.724293,
	0.724293,
	0.724293,
	0.724293,
	0.726896,
	0.734706,
	0.724293,
	0.724293,
	0.0266309,
	0.521242,
	0.484797,
	0.6488,
	0.518639,
	0.667022,
	0.531655,
	0.599339,
	0.266127,
	0.435336,
	0.599339,
	0.654006,
	0.674832,
	0.677435,
	0.654006,
	0.680039,
	0.687848,
	0.65661,
	0.627974,
	0.646197,
	0.703468,
	0.651403,
	0.677435,
	0.6488,
	0.693055,
	0.703468,
	0.693055,
	0.682642,
	0.693055,
	0.578513,
	0.667022,
	0.674832,
	0.687848,
	0.693055,
	0.700864,
	0.71388,
	0.661816,
	0.664419,
	0.474384,
	0.552481,
	0.0969177,
	0.437939,
	0.523845,
	0.630577,
	0.677435,
	0.654006,
	0.49521,
	0.625371,
	0.682642,
	0.627974,
	0.661816,
	0.6488,
	0.630577,
	0.482194,
	0.661816,
	0.654006,
	0.724293,
	0.703468,
	0.674832,
	0.526449,
	0.724293,
	0.667022,
	0.682642,
	0.5681,
	0.635784,
	0.65661,
	0.693055,
	0.659213,
	0.64099,
	0.646197,
	0.651403,
	0.680039,
	0.536862,
	0.687848,
	0.71388,
	0.674832,
	0.674832,
	0.65661,
	0.622768,
	0.583719,
	0.0917113,
	0.492607,
	0.521242,
	0.635784,
	0.680039,
	0.627974,
	0,
	0.34943,
	0.219269,
	0.614958,
	0.620165,
	0.659213,
	0.633181,
	0.667022,
	0.612355,
	0.651403,
	0.601942,
	0.581116,
	0.601942,
	0.682642,
	0.698261,
	0.695658,
	0.71388,
	0.706071,
	0.609752,
	0.659213,
	0.65661,
	0.625371,
	0.64099,
	0.643593,
	0.651403,
	0.698261,
	0.588926,
	0.700864,
	0.71388,
	0.680039,
	0.6488,
	0.654006,
	0.646197,
	0.547274,
	0.0917113,
	0.521242,
	0.224475,
	0.109934,
	0.380668,
	0.560291,
	0.703468,
	0.437939,
	0.57591,
	0.542068,
	0.609752,
	0.677435,
	0.635784,
	0.440543,
	0.661816,
	0.643593,
	0.677435,
	0.674832,
	0.463971,
	0.4874,
	0.71388,
	0.674832,
	0.71388,
	0.716484,
	0.614958,
	0.661816,
	0.693055,
	0.693055,
	0.677435,
	0.630577,
	0.622768,
	0.706071,
	0.71388,
	0.57591,
	0.583719,
	0.635784,
	0.664419,
	0.630577,
	0.557687,
	0.6488,
	0.104727,
	0.458765,
	0.104727,
	0.161998,
	0.654006,
	0.71388,
	0.630577,
	0.651403,
	0.687848,
	0.674832,
	0.620165,
	0.719087,
	0.508226,
	0.604545,
	0.604545,
	0.643593,
	0.700864,
	0.680039,
	0.596736,
	0.680039,
	0.711277,
	0.698261,
	0.698261,
	0.6488,
	0.604545,
	0.617561,
	0.667022,
	0.685245,
	0.693055,
	0.674832,
	0.664419,
	0.693055,
	0.706071,
	0.682642,
	0.570703,
	0.604545,
	0.609752,
	0.607148,
	0.630577,
	0.674832,
	0.0943145,
	0.476988,
	0.469178,
	0.630577,
	0.505623,
	0.711277,
	0.706071,
	0.703468,
	0.64099,
	0.638387,
	0.667022,
	0.695658,
	0.573307,
	0.518639,
	0.635784,
	0.627974,
	0.698261,
	0.674832,
	0.64099,
	0.687848,
	0.687848,
	0.654006,
	0.693055,
	0.711277,
	0.625371,
	0.651403,
	0.680039,
	0.693055,
	0.711277,
	0.690451,
	0.651403,
	0.667022,
	0.71388,
	0.72169,
	0.693055,
	0.693055,
	0.646197,
	0.635784,
	0.674832,
	0.659213,
	0.0969177,
	0.393685,
	0.607148,
	0.560291,
	0.588926,
	0.6488,
	0.630577,
	0.622768,
	0.661816,
	0.588926,
	0.677435,
	0.700864,
	0.667022,
	0.391081,
	0.409304,
	0.284727,
	0.282864,
	0.282864,
	0.282864,
	0.282864,
	0.282864,
	0.282864,
	0.282864,
	0.282864,
	0.282864,
	0.282864,
	0.282864,
	0.282864,
	0.282864,
	0.282864,
	0.282864,
	0.282864,
	0.282864,
	0.282864,
	0.282864,
	0.282864,
	0.161767,
	0.145,
	0.176671,
	0.176671,
	0.135684,
	0.241877,
	0.213932,
	0.253055,
	0.264234,
	0.236288,
	0.269823,
	0.295905,
	0.28659,
	0.279138,
	0.26796,
	0.251192,
	0.230699,
	0.215795,
	0.234425,
	0.197164,
	0.228836,
	0.228836,
	0.284727,
	0.292179,
	0.297768,
	0.282864,
	0.297768,
	0.284727,
	0.282864,
	0.295905,
	0.271686,
	0.249329,
	0.260507,
	0.269823,
	0.260507,
	0.28659,
	0.271686,
	0.269823,
	0.284727,
	0.269823,
	0.141274,
	0.241877,
	0.122643,
	0.193438,
	0.204617,
	0.145,
	0.260507,
	0.26237,
	0.228836,
	0.266097,
	0.232562,
	0.28659,
	0.219521,
	0.236288,
	0.223247,
	0.288453,
	0.256781,
	0.275412,
	0.279138,
	0.28659,
	0.282864,
	0.275412,
	0.269823,
	0.290316,
	0.294042,
	0.28659,
	0.253055,
	0.290316,
	0.295905,
	0.277275,
	0.271686,
	0.241877,
	0.256781,
	0.269823,
	0.26796,
	0.28659,
	0.281001,
	0.284727,
	0.290316,
	0.269823,
	0.145,
	0.254918,
	0.141274,
	0.137547,
	0.215795,
	0.145,
	0.217658,
	0.113328,
	0.249329,
	0.137547,
	0.251192,
	0.290316,
	0.238151,
	0.238151,
	0.282864,
	0.281001,
	0.279138,
	0.217658,
	0.301494,
	0.269823,
	0.292179,
	0.295905,
	0.273549,
	0.288453,
	0.294042,
	0.281001,
	0.241877,
	0.277275,
	0.284727,
	0.264234,
	0.282864,
	0.249329,
	0.264234,
	0.264234,
	0.256781,
	0.282864,
	0.279138,
	0.282864,
	0.282864,
	0.275412,
	0.148726,
	0.238151,
	0.130095,
	0.154315,
	0.279138,
	0.20089,
	0.213932,
	0.230699,
	0.236288,
	0.266097,
	0.264234,
	0.264234,
	0.275412,
	0.234425,
	0.221384,
	0.28659,
	0.277275,
	0.217658,
	0.294042,
	0.226973,
	0.282864,
	0.288453,
	0.26796,
	0.290316,
	0.290316,
	0.284727,
	0.256781,
	0.294042,
	0.279138,
	0.269823,
	0.271686,
	0.251192,
	0.260507,
	0.273549,
	0.275412,
	0.28659,
	0.28659,
	0.290316,
	0.281001,
	0.279138,
	0.146863,
	0.219521,
	0.122643,
	0.279138,
	0.193438,
	0.197164,
	0.266097,
	0.141274,
	0.234425,
	0.204617,
	0.210206,
	0.264234,
	0.226973,
	0.230699,
	0.22511,
	0.295905,
	0.202754,
	0.299631,
	0.223247,
	0.292179,
	0.290316,
	0.284727,
	0.284727,
	0.288453,
	0.299631,
	0.273549,
	0.260507,
	0.281001,
	0.279138,
	0.277275,
	0.279138,
	0.232562,
	0.269823,
	0.271686,
	0.281001,
	0.275412,
	0.271686,
	0.290316,
	0.281001,
	0.284727,
	0.148726,
	0.234425,
	0.122643,
	0.191575,
	0.269823,
	0.251192,
	0.22511,
	0.195301,
	0.215795,
	0.279138,
	0.241877,
	0.234425,
	0.26796,
	0.219521,
	0.284727,
	0.290316,
	0.24374,
	0.295905,
	0.290316,
	0.208343,
	0.277275,
	0.28659,
	0.279138,
	0.288453,
	0.28659,
	0.290316,
	0.264234,
	0.284727,
	0.279138,
	0.277275,
	0.277275,
	0.241877,
	0.26237,
	0.266097,
	0.281001,
	0.292179,
	0.277275,
	0.275412,
	0.28659,
	0.281001,
	0.148726,
	0.234425,
	0.130095,
	0.284727,
	0.234425,
	0.254918,
	0.256781,
	0.143137,
	0.215795,
	0.260507,
	0.275412,
	0.238151,
	0.249329,
	0.232562,
	0.158041,
	0.463691,
	0.463691,
	0.455133,
	0.463691,
	0.463691,
	0.463691,
	0.463691,
	0.463691,
	0.455133,
	0.463691,
	0.463691,
	0.463691,
	0.46797,
	0.463691,
	0.463691,
	0.463691,
	0.463691,
	0.463691,
	0.463691,
	0.463691,
	0.463691,
	0.0186572,
	0.198382,
	0.455133,
	0.4209,
	0.41662,
	0.455133,
	0.386666,
	0.4209,
	0.429458,
	0.335316,
	0.292524,
	0.258291,
	0.46797,
	0.46797,
	0.288245,
	0.395224,
	0.408062,
	0.438016,
	0.429458,
	0.446575,
	0.455133,
	0.455133,
	0.446575,
	0.450854,
	0.433737,
	0.46797,
	0.450854,
	0.446575,
	0.425179,
	0.4209,
	0.378108,
	0.403783,
	0.4209,
	0.403783,
	0.412341,
	0.425179,
	0.390945,
	0.36527,
	0.378108,
	0.395224,
	0.0314947,
	0.112799,
	0.47225,
	0.450854,
	0.463691,
	0.459412,
	0.412341,
	0.433737,
	0.412341,
	0.245453,
	0.450854,
	0.438016,
	0.455133,
	0.4209,
	0.395224,
	0.455133,
	0.450854,
	0.438016,
	0.438016,
	0.41662,
	0.459412,
	0.459412,
	0.455133,
	0.450854,
	0.450854,
	0.455133,
	0.463691,
	0.47225,
	0.459412,
	0.463691,
	0.438016,
	0.455133,
	0.459412,
	0.463691,
	0.47225,
	0.446575,
	0.463691,
	0.455133,
	0.46797,
	0.395224,
	0.0357739,
	0.142753,
	0.459412,
	0.450854,
	0.459412,
	0.47225,
	0.412341,
	0.446575,
	0.288245,
	0.408062,
	0.382387,
	0.438016,
	0.485087,
	0.463691,
	0.339595,
	0.455133,
	0.459412,
	0.429458,
	0.429458,
	0.433737,
	0.450854,
	0.459412,
	0.450854,
	0.442295,
	0.459412,
	0.46797,
	0.459412,
	0.480808,
	0.450854,
	0.450854,
	0.433737,
	0.429458,
	0.46797,
	0.46797,
	0.463691,
	0.446575,
	0.450854,
	0.463691,
	0.378108,
	0.46797,
	0.0357739,
	0.155591,
	0.47225,
	0.455133,
	0.446575,
	0.455133,
	0.412341,
	0.455133,
	0.343874,
	0.395224,
	0.403783,
	0.403783,
	0.450854,
	0.408062,
	0.390945,
	0.429458,
	0.442295,
	0.442295,
	0.378108,
	0.446575,
	0.438016,
	0.455133,
	0.450854,
	0.446575,
	0.463691,
	0.46797,
	0.46797,
	0.463691,
	0.450854,
	0.463691,
	0.433737,
	0.429458,
	0.47225,
	0.455133,
	0.463691,
	0.459412,
	0.459412,
	0.463691,
	0.360991,
	0.412341,
	0.0272155,
	0.185545,
	0.463691,
	0.335316,
	0.438016,
	0.463691,
	0.382387,
	0.463691,
	0.425179,
	0.241174,
	0.386666,
	0.395224,
	0.455133,
	0.463691,
	0.446575,
	0.476529,
	0.459412,
	0.450854,
	0.429458,
	0.446575,
	0.450854,
	0.455133,
	0.442295,
	0.455133,
	0.47225,
	0.450854,
	0.446575,
	0.429458,
	0.455133,
	0.47225,
	0.450854,
	0.429458,
	0.480808,
	0.450854,
	0.446575,
	0.446575,
	0.463691,
	0.476529,
	0.446575,
	0.438016,
	0.0229364,
	0.15987,
	0.47225,
	0.455133,
	0.459412,
	0.459412,
	0.403783,
	0.4209,
	0.425179,
	0.446575,
	0.408062,
	0.442295,
	0.433737,
	0.446575,
	0.279687,
	0.412341,
	0.455133,
	0.455133,
	0.455133,
	0.382387,
	0.455133,
	0.46797,
	0.450854,
	0.463691,
	0.47225,
	0.480808,
	0.41662,
	0.446575,
	0.463691,
	0.442295,
	0.314954,
	0.31717,
	0.31717,
	0.323819,
	0.31717,
	0.31717,
	0.319387,
	0.31717,
	0.31717,
	0.31717,
	0.31717,
	0.31717,
	0.31717,
	0.31717,
	0.31717,
	0.31717,
	0.314954,
	0.31717,
	0.31717,
	0.31717,
	0.31717,
	0,
	0.124368,
	0.323819,
	0.332683,
	0.326035,
	0.323819,
	0.326035,
	0.310522,
	0.301658,
	0.279496,
	0.297225,
	0.270632,
	0.297225,
	0.326035,
	0.334899,
	0.343764,
	0.288361,
	0.314954,
	0.290577,
	0.303874,
	0.328251,
	0.337115,
	0.312738,
	0.328251,
	0.334899,
	0.323819,
	0.334899,
	0.328251,
	0.328251,
	0.319387,
	0.328251,
	0.337115,
	0.31717,
	0.326035,
	0.310522,
	0.321603,
	0.334899,
	0.330467,
	0.321603,
	0.301658,
	0.0246432,
	0.290577,
	0.224094,
	0.328251,
	0.30609,
	0.27728,
	0.297225,
	0.321603,
	0.330467,
	0.303874,
	0.319387,
	0.219661,
	0.312738,
	0.301658,
	0.261768,
	0.257335,
	0.312738,
	0.244039,
	0.268416,
	0.235174,
	0.321603,
	0.332683,
	0.310522,
	0.321603,
	0.330467,
	0.314954,
	0.275064,
	0.319387,
	0.328251,
	0.323819,
	0.330467,
	0.330467,
	0.319387,
	0.319387,
	0.319387,
	0.337115,
	0.328251,
	0.326035,
	0.328251,
	0.314954,
	0.0357238,
	0.288361,
	0.301658,
	0.23739,
	0.326035,
	0.217445,
	0.323819,
	0.328251,
	0.323819,
	0.290577,
	0.323819,
	0.292793,
	0.310522,
	0.297225,
	0.27728,
	0.23739,
	0.279496,
	0.301658,
	0.297225,
	0.301658,
	0.323819,
	0.330467,
	0.334899,
	0.343764,
	0.326035,
	0.319387,
	0.283929,
	0.334899,
	0.323819,
	0.334899,
	0.337115,
	0.343764,
	0.323819,
	0.330467,
	0.330467,
	0.341548,
	0.330467,
	0.348196,
	0.337115,
	0.310522,
	0.040156,
	0.275064,
	0.319387,
	0.250687,
	0.326035,
	0.308306,
	0.290577,
	0.283929,
	0.332683,
	0.31717,
	0.323819,
	0.323819,
	0.308306,
	0.295009,
	0.235174,
	0.292793,
	0.179771,
	0.241823,
	0.295009,
	0.310522,
	0.323819,
	0.319387,
	0.330467,
	0.334899,
	0.323819,
	0.319387,
	0.290577,
	0.326035,
	0.332683,
	0.332683,
	0.334899,
	0.334899,
	0.330467,
	0.323819,
	0.323819,
	0.339332,
	0.330467,
	0.341548,
	0.332683,
	0.319387,
	0.0357238,
	0.235174,
	0.252903,
	0.341548,
	0.341548,
	0.275064,
	0.295009,
	0.334899,
	0.337115,
	0.301658,
	0.319387,
	0.246255,
	0.312738,
	0.261768,
	0.279496,
	0.263984,
	0.268416,
	0.297225,
	0.332683,
	0.31717,
	0.326035,
	0.334899,
	0.326035,
	0.337115,
	0.332683,
	0.295009,
	0.308306,
	0.330467,
	0.339332,
	0.334899,
	0.350412,
	0.332683,
	0.334899,
	0.328251,
	0.321603,
	0.341548,
	0.332683,
	0.334899,
	0.339332,
	0.330467,
	0.0312915,
	0.208581,
	0.261768,
	0.337115,
	0.295009,
	0.252903,
	0.303874,
	0.295009,
	0.341548,
	0.319387,
	0.299442,
	0.319387,
	0.319387,
	0.308306,
	0.303874,
	0.310522,
	0.314954,
	0.321603,
	0.330467,
	0.319387,
	0.30609,
	0.330467,
	0.323819,
	0.334899,
	0.328251,
	0.314954,
	0.308306,
	0.330467,
	0.332683,
	0.341548,
	0.341548,
	0.343764,
	0.348196,
	0.339332,
	0.341548,
	0.337115,
	0.332683,
	0.330467,
	0.326035,
	0.337115,
	0.0357238,
	0.232958,
	0.210797,
	0.332683,
	0.219661,
	0.334899,
	0.343764,
	0.341548,
	0.343764,
	0.310522,
	0.334899,
	0.308306,
	0.27728,
	0.303874,
	0.295009,
	0.253322,
	0.253322,
	0.253322,
	0.253322,
	0.253322,
	0.253322,
	0.253322,
	0.253322,
	0.253322,
	0.253322,
	0.253322,
	0.253322,
	0.256487,
	0.253322,
	0.253322,
	0.253322,
	0.259651,
	0.253322,
	0.253322,
	0.256487,
	0.253322,
	0.145741,
	0.259651,
	0.240666,
	0.190039,
	0.218517,
	0.234337,
	0.275471,
	0.278636,
	0.256487,
	0.256487,
	0.265979,
	0.234337,
	0.256487,
	0.253322,
	0.253322,
	0.262815,
	0.250158,
	0.246994,
	0.253322,
	0.253322,
	0.275471,
	0.2818,
	0.275471,
	0.262815,
	0.262815,
	0.265979,
	0.275471,
	0.265979,
	0.256487,
	0.265979,
	0.253322,
	0.234337,
	0.240666,
	0.259651,
	0.259651,
	0.231173,
	0.240666,
	0.253322,
	0.275471,
	0.265979,
	0.0856221,
	0.275471,
	0.259651,
	0.237502,
	0.221681,
	0.262815,
	0.259651,
	0.253322,
	0.253322,
	0.259651,
	0.275471,
	0.275471,
	0.265979,
	0.253322,
	0.256487,
	0.272307,
	0.262815,
	0.272307,
	0.253322,
	0.24383,
	0.272307,
	0.269143,
	0.256487,
	0.259651,
	0.256487,
	0.262815,
	0.262815,
	0.262815,
	0.265979,
	0.278636,
	0.265979,
	0.259651,
	0.240666,
	0.269143,
	0.262815,
	0.240666,
	0.259651,
	0.246994,
	0.265979,
	0.265979,
	0.0856221,
	0.269143,
	0.256487,
	0.250158,
	0.256487,
	0.24383,
	0.253322,
	0.237502,
	0.234337,
	0.228009,
	0.275471,
	0.265979,
	0.275471,
	0.272307,
	0.256487,
	0.275471,
	0.256487,
	0.275471,
	0.269143,
	0.272307,
	0.272307,
	0.262815,
	0.269143,
	0.269143,
	0.253322,
	0.269143,
	0.262815,
	0.275471,
	0.256487,
	0.272307,
	0.24383,
	0.253322,
	0.253322,
	0.265979,
	0.246994,
	0.246994,
	0.259651,
	0.265979,
	0.269143,
	0.259651,
	0.0856221,
	0.269143,
	0.265979,
	0.253322,
	0.231173,
	0.24383,
	0.224845,
	0.253322,
	0.256487,
	0.253322,
	0.237502,
	0.246994,
	0.240666,
	0.231173,
	0.262815,
	0.269143,
	0.259651,
	0.2818,
	0.272307,
	0.265979,
	0.275471,
	0.272307,
	0.269143,
	0.265979,
	0.262815,
	0.269143,
	0.256487,
	0.253322,
	0.2818,
	0.275471,
	0.278636,
	0.262815,
	0.246994,
	0.240666,
	0.272307,
	0.272307,
	0.250158,
	0.259651,
	0.269143,
	0.272307,
	0.0887862,
	0.272307,
	0.262815,
	0.231173,
	0.12992,
	0.265979,
	0.224845,
	0.269143,
	0.265979,
	0.275471,
	0.2818,
	0.2818,
	0.275471,
	0.24383,
	0.24383,
	0.246994,
	0.250158,
	0.250158,
	0.272307,
	0.262815,
	0.272307,
	0.272307,
	0.2818,
	0.275471,
	0.272307,
	0.278636,
	0.265979,
	0.265979,
	0.259651,
	0.275471,
	0.275471,
	0.269143,
	0.259651,
	0.262815,
	0.256487,
	0.24383,
	0.253322,
	0.259651,
	0.262815,
	0.265979,
	0.0824579,
	0.269143,
	0.136249,
	0.196368,
	0.158398,
	0.228009,
	0.246994,
	0.259651,
	0.262815,
	0.262815,
	0.275471,
	0.2818,
	0.275471,
	0.275471,
	0.275471,
	0.278636,
	0.240666,
	0.272307,
	0.272307,
	0.259651,
	0.265979,
	0.269143,
	0.275471,
	0.228009,
	0.253322,
	0.275471,
	0.272307,
	0.275471,
	0.275471,
	0.275471,
	0.275471,
	0.262815,
	0.259651,
	0.259651,
	0.256487,
	0.256487,
	0.250158,
	0.256487,
	0.265979,
	0.269143,
	0.0887862,
	0.265979,
	0.139413,
	0.278636,
	0.186875,
	0.231173,
	0.256487,
	0.259651,
	0.259651,
	0.272307,
	0.275471,
	0.265979,
	0.272307,
	0.275471,
	0.275471,
	0.304078,
	0.304078,
	0.304078,
	0.304078,
	0.304078,
	0.304078,
	0.304078,
	0.304078,
	0.304078,
	0.304078,
	0.304078,
	0.302135,
	0.304078,
	0.304078,
	0.304078,
	0.300192,
	0.304078,
	0.304078,
	0.300192,
	0.304078,
	0.304078,
	0,
	0.17975,
	0.255512,
	0.251627,
	0.288537,
	0.255512,
	0.26911,
	0.265225,
	0.274938,
	0.280766,
	0.274938,
	0.249684,
	0.232201,
	0.226373,
	0.26911,
	0.255512,
	0.236086,
	0.288537,
	0.29825,
	0.29825,
	0.309905,
	0.280766,
	0.274938,
	0.286594,
	0.290479,
	0.290479,
	0.302135,
	0.302135,
	0.228316,
	0.26911,
	0.29825,
	0.284651,
	0.29825,
	0.29825,
	0.304078,
	0.288537,
	0.280766,
	0.278824,
	0.296307,
	0.288537,
	0.00102958,
	0.117586,
	0.284651,
	0.278824,
	0.251627,
	0.257455,
	0.276881,
	0.274938,
	0.267168,
	0.288537,
	0.267168,
	0.276881,
	0.274938,
	0.280766,
	0.278824,
	0.302135,
	0.241914,
	0.25357,
	0.203062,
	0.232201,
	0.304078,
	0.257455,
	0.290479,
	0.276881,
	0.302135,
	0.284651,
	0.286594,
	0.290479,
	0.288537,
	0.284651,
	0.290479,
	0.286594,
	0.292422,
	0.282709,
	0.296307,
	0.278824,
	0.274938,
	0.274938,
	0.284651,
	0.276881,
	0,
	0.0670785,
	0.280766,
	0.210832,
	0.280766,
	0.274938,
	0.280766,
	0.239971,
	0.236086,
	0.274938,
	0.154496,
	0.162267,
	0.257455,
	0.111759,
	0.105931,
	0.274938,
	0.173922,
	0.162267,
	0.288537,
	0.259397,
	0.300192,
	0.292422,
	0.304078,
	0.304078,
	0.307963,
	0.282709,
	0.282709,
	0.288537,
	0.282709,
	0.208889,
	0.26134,
	0.267168,
	0.296307,
	0.280766,
	0.304078,
	0.280766,
	0.274938,
	0.294364,
	0.286594,
	0.284651,
	0,
	0.0787342,
	0.300192,
	0.183635,
	0.201119,
	0.280766,
	0.274938,
	0.25357,
	0.21666,
	0.241914,
	0.195291,
	0.205004,
	0.284651,
	0.292422,
	0.245799,
	0.193348,
	0.150611,
	0.249684,
	0.230258,
	0.257455,
	0.292422,
	0.300192,
	0.292422,
	0.304078,
	0.300192,
	0.280766,
	0.29825,
	0.29825,
	0.292422,
	0.25357,
	0.259397,
	0.251627,
	0.304078,
	0.288537,
	0.296307,
	0.288537,
	0.280766,
	0.29825,
	0.288537,
	0.280766,
	0,
	0.059308,
	0.29825,
	0.236086,
	0.292422,
	0.304078,
	0.259397,
	0.226373,
	0.17975,
	0.296307,
	0.251627,
	0.282709,
	0.282709,
	0.208889,
	0.259397,
	0.276881,
	0.278824,
	0.271053,
	0.292422,
	0.278824,
	0.307963,
	0.29825,
	0.280766,
	0.296307,
	0.288537,
	0.276881,
	0.300192,
	0.290479,
	0.286594,
	0.286594,
	0.294364,
	0.280766,
	0.29825,
	0.286594,
	0.302135,
	0.276881,
	0.278824,
	0.302135,
	0.282709,
	0.259397,
	0,
	0.0806768,
	0.265225,
	0.276881,
	0.304078,
	0.251627,
	0.282709,
	0.29825,
	0.286594,
	0.286594,
	0.276881,
	0.218602,
	0.274938,
	0.265225,
	0.274938,
	0.278824,
	0.278824,
	0.300192,
	0.296307,
	0.300192,
	0.315733,
	0.29825,
	0.292422,
	0.304078,
	0.300192,
	0.302135,
	0.29825,
	0.296307,
	0.284651,
	0.292422,
	0.284651,
	0.29825,
	0.302135,
	0.274938,
	0.29825,
	0.284651,
	0.280766,
	0.286594,
	0.274938,
	0.292422,
	0,
	0.0651359,
	0.29825,
	0.26911,
	0.265225,
	0.274938,
	0.290479,
	0.274938,
	0.284651,
	0.280766,
	0.296307,
	0.286594,
	0.29825,
	0.307963,
	0.284651};

	const float m_GraySamplesRight[4205] = {
	0.872714,
	0.899225,
	0.872714,
	0.872714,
	0.899225,
	0.872714,
	0.872714,
	0.892597,
	0.872714,
	0.872714,
	0.889283,
	0.872714,
	0.872714,
	0.872714,
	0.882655,
	0.872714,
	0.872714,
	0.856144,
	0.872714,
	0.872714,
	0.892597,
	0.150285,
	0.862772,
	0.879341,
	0.813063,
	0.79318,
	0.8694,
	0.786552,
	0.829633,
	0.786552,
	0.756727,
	0.826319,
	0.561208,
	0.73353,
	0.518127,
	0.630799,
	0.723588,
	0.783238,
	0.670566,
	0.667252,
	0.766669,
	0.763355,
	0.789866,
	0.720274,
	0.769983,
	0.832947,
	0.720274,
	0.783238,
	0.85283,
	0.859458,
	0.859458,
	0.889283,
	0.846202,
	0.866086,
	0.859458,
	0.85283,
	0.823005,
	0.859458,
	0.832947,
	0.85283,
	0.769983,
	0.0707515,
	0.85283,
	0.710333,
	0.836261,
	0.826319,
	0.879341,
	0.819691,
	0.859458,
	0.819691,
	0.8694,
	0.813063,
	0.836261,
	0.544638,
	0.594347,
	0.47836,
	0.627485,
	0.779925,
	0.653997,
	0.587719,
	0.687136,
	0.773297,
	0.746786,
	0.644055,
	0.753413,
	0.816377,
	0.789866,
	0.750099,
	0.832947,
	0.846202,
	0.823005,
	0.872714,
	0.85283,
	0.819691,
	0.813063,
	0.882655,
	0.829633,
	0.849516,
	0.832947,
	0.839575,
	0.803122,
	0.0906349,
	0.8694,
	0.756727,
	0.67388,
	0.779925,
	0.899225,
	0.849516,
	0.713647,
	0.779925,
	0.876027,
	0.813063,
	0.803122,
	0.746786,
	0.756727,
	0.766669,
	0.253016,
	0.153599,
	0.624172,
	0.325921,
	0.150285,
	0.746786,
	0.716961,
	0.786552,
	0.826319,
	0.763355,
	0.813063,
	0.826319,
	0.839575,
	0.829633,
	0.862772,
	0.866086,
	0.826319,
	0.79318,
	0.819691,
	0.806436,
	0.720274,
	0.620858,
	0.832947,
	0.836261,
	0.816377,
	0.0972627,
	0.823005,
	0.518127,
	0.624172,
	0.829633,
	0.829633,
	0.862772,
	0.624172,
	0.879341,
	0.700391,
	0.730216,
	0.859458,
	0.803122,
	0.743472,
	0.756727,
	0.813063,
	0.624172,
	0.524755,
	0.153599,
	0.236446,
	0.813063,
	0.753413,
	0.763355,
	0.816377,
	0.842888,
	0.823005,
	0.876027,
	0.849516,
	0.859458,
	0.806436,
	0.859458,
	0.760041,
	0.85283,
	0.872714,
	0.859458,
	0.746786,
	0.663938,
	0.600974,
	0.849516,
	0.885969,
	0.0972627,
	0.885969,
	1,
	0.312666,
	0.80975,
	0.769983,
	0.819691,
	0.73353,
	0.713647,
	0.846202,
	0.892597,
	0.839575,
	0.859458,
	0.647369,
	0.773297,
	0.763355,
	0.799808,
	0.726902,
	0.753413,
	0.743472,
	0.823005,
	0.776611,
	0.740158,
	0.79318,
	0.816377,
	0.846202,
	0.846202,
	0.819691,
	0.823005,
	0.885969,
	0.872714,
	0.829633,
	0.786552,
	0.806436,
	0.826319,
	0.803122,
	0.796494,
	0.640741,
	0.786552,
	0.773297,
	0.110518,
	0.769983,
	0.41871,
	0.498244,
	0.806436,
	0.799808,
	0.836261,
	0.806436,
	0.700391,
	0.700391,
	0.85283,
	0.783238,
	0.79318,
	0.726902,
	0.670566,
	0.412082,
	0.465105,
	0.68141,
	0.707984,
	0.68141,
	0.68141,
	0.68141,
	0.68141,
	0.68141,
	0.68141,
	0.663694,
	0.68141,
	0.68141,
	0.675505,
	0.68141,
	0.68141,
	0.68141,
	0.68141,
	0.68141,
	0.645979,
	0.68141,
	0.68141,
	0.678457,
	0.0465927,
	0.699126,
	0.533778,
	0.68141,
	0.722747,
	0.521968,
	0.666647,
	0.536731,
	0.403862,
	0.406815,
	0.634168,
	0.693221,
	0,
	0.380241,
	0.640073,
	0.640073,
	0.678457,
	0.610547,
	0.610547,
	0.318236,
	0.690268,
	0.68141,
	0.640073,
	0.648931,
	0.702079,
	0.619405,
	0.660742,
	0.702079,
	0.6696,
	0.637121,
	0.722747,
	0.693221,
	0.687315,
	0.687315,
	0.68141,
	0.702079,
	0.699126,
	0.595784,
	0.548541,
	0.554447,
	0.00525578,
	0.542636,
	0.332999,
	0.238514,
	0.690268,
	0.477678,
	0.651884,
	0.557399,
	0.610547,
	0.598736,
	0.58102,
	0.663694,
	0.504252,
	0.575115,
	0.350715,
	0.696173,
	0.462915,
	0.690268,
	0,
	0.660742,
	0.705031,
	0.663694,
	0.705031,
	0.702079,
	0.628263,
	1,
	0.510157,
	0.651884,
	0.684363,
	0.622357,
	0.719794,
	0.684363,
	0.672552,
	0.6696,
	0.687315,
	0.699126,
	0.634168,
	0.566257,
	0.586926,
	0.572163,
	0.084977,
	0.648931,
	0.36843,
	0.713889,
	0.533778,
	0.628263,
	0.634168,
	0.542636,
	0.713889,
	0.719794,
	0.699126,
	0.640073,
	0.586926,
	0.471773,
	0.397957,
	0.229656,
	0.462915,
	0.690268,
	0.489489,
	0.459962,
	0.710937,
	0.702079,
	0.684363,
	0.684363,
	0.651884,
	0.548541,
	0.589878,
	0.657789,
	0.68141,
	0.675505,
	0.710937,
	0.707984,
	0.675505,
	0.651884,
	0.678457,
	0.702079,
	0.643026,
	0.578068,
	0.536731,
	0.607594,
	0.0288769,
	0.619405,
	0.462915,
	0.705031,
	0.660742,
	0.651884,
	0.68141,
	0.68141,
	0.634168,
	0.572163,
	0.713889,
	0.58102,
	0.62531,
	0.536731,
	0.439294,
	0.693221,
	0.146982,
	0.0584033,
	0.238514,
	0.713889,
	0.687315,
	0.728652,
	0.702079,
	0.707984,
	0.672552,
	0.459962,
	0.51311,
	0.675505,
	0.634168,
	0.687315,
	0.73751,
	0.722747,
	0.690268,
	0.702079,
	0.666647,
	0.699126,
	0.628263,
	0.583973,
	0.533778,
	0.622357,
	0.0406875,
	0.634168,
	0.0967876,
	0,
	0.217846,
	0.643026,
	0.533778,
	0.687315,
	0.530826,
	0.713889,
	0.572163,
	0.645979,
	0.607594,
	0.483583,
	0.173556,
	0.152888,
	0.436341,
	0.610547,
	0.643026,
	1,
	0.728652,
	0.696173,
	0.678457,
	0.690268,
	0.672552,
	0.539683,
	0.548541,
	0.643026,
	0.648931,
	0.634168,
	0.705031,
	0.705031,
	0.666647,
	0.693221,
	0.690268,
	0.672552,
	0.634168,
	0.554447,
	0.583973,
	0.607594,
	0.0406875,
	0.601689,
	0.474725,
	0.678457,
	0.663694,
	0.637121,
	0.696173,
	0.648931,
	0.749321,
	0.678457,
	0.445199,
	0.607594,
	0.415673,
	0.521968,
	0.651884,
	0.572163,
	0.474725,
	0.693221,
	0.285756,
	0.495394,
	0.68141,
	0.660742,
	0.548541,
	0.705031,
	0.696173,
	0.589878,
	0.545589,
	0.62531,
	0.672552,
	0.678457,
	0.707984,
	0.722747,
	0.62531,
	0.666647,
	0.702079,
	0.643026,
	0.643026,
	0.648931,
	0.583973,
	0.640073,
	0.0584033,
	0.46882,
	0.46882,
	0.560352,
	0.702079,
	0.607594,
	0.575115,
	0.746368,
	0.501299,
	0.699126,
	0.400909,
	0.563305,
	0.374336,
	0.318236,
	0.539683,
	0.386933,
	0.391395,
	0.391395,
	0.386933,
	0.386933,
	0.391395,
	0.386933,
	0.393625,
	0.386933,
	0.386933,
	0.382472,
	0.391395,
	0.386933,
	0.391395,
	0.386933,
	0.386933,
	0.391395,
	0.386933,
	0.386933,
	0.391395,
	0.386933,
	0,
	0.4137,
	0.384703,
	0.262023,
	0.360167,
	0.357936,
	0.295481,
	0.360167,
	0.342322,
	0.362397,
	0.349014,
	0.391395,
	0.384703,
	0.353475,
	0.337861,
	0.389164,
	0.398086,
	0.320017,
	0.375781,
	0.324478,
	0.380242,
	0.360167,
	0.386933,
	0.393625,
	0.382472,
	0.41147,
	0.418161,
	0.415931,
	0.41147,
	0.407008,
	0.362397,
	0.342322,
	0.364628,
	0.353475,
	0.353475,
	0.37355,
	0.355706,
	0.349014,
	0.355706,
	0.353475,
	0.00774001,
	0.212951,
	0.393625,
	0.389164,
	0.395856,
	0.391395,
	0.398086,
	0.415931,
	0.353475,
	0.375781,
	0.326709,
	0.342322,
	0.37355,
	0.248639,
	0.386933,
	0.355706,
	0.418161,
	0.355706,
	0.400317,
	0.420392,
	0.418161,
	0.424853,
	0.422622,
	0.418161,
	0.4137,
	0.391395,
	0.364628,
	0.380242,
	0.4137,
	0.422622,
	0.41147,
	0.404778,
	0.41147,
	0.404778,
	0.391395,
	0.400317,
	0.402547,
	0.382472,
	0.366859,
	0.342322,
	0.00550946,
	0.206259,
	0.304403,
	0.395856,
	0.395856,
	0.398086,
	0.380242,
	0.386933,
	0.326709,
	0.400317,
	0.313325,
	0.306634,
	0.206259,
	0.384703,
	0.366859,
	0.380242,
	0.41147,
	0.420392,
	0.282098,
	0.420392,
	0.424853,
	0.418161,
	0.420392,
	0.424853,
	0.404778,
	0.389164,
	0.364628,
	0.395856,
	0.415931,
	0.402547,
	0.409239,
	0.407008,
	0.415931,
	0.407008,
	0.393625,
	0.398086,
	0.404778,
	0.404778,
	0.366859,
	0.366859,
	0.00550946,
	0.17057,
	0.362397,
	0.398086,
	0.400317,
	0.404778,
	0.415931,
	0.402547,
	0.380242,
	0.407008,
	0.41147,
	0.407008,
	0.362397,
	0.275406,
	0.409239,
	0.386933,
	0.366859,
	0.308864,
	0.37132,
	0.415931,
	0.418161,
	0.41147,
	0.418161,
	0.418161,
	0.404778,
	0.37132,
	0.380242,
	0.391395,
	0.407008,
	0.415931,
	0.400317,
	0.422622,
	0.4137,
	0.4137,
	0.4137,
	0.400317,
	0.400317,
	0.391395,
	0.375781,
	0.37355,
	0.00104836,
	0.172801,
	0.389164,
	0.237487,
	0.409239,
	0.315556,
	0.420392,
	0.391395,
	0.357936,
	0.409239,
	0.400317,
	0.246409,
	0.37132,
	0.389164,
	0.404778,
	0.395856,
	0.420392,
	0.4137,
	0.400317,
	0.357936,
	0.409239,
	0.4137,
	0.420392,
	0.415931,
	0.41147,
	0.398086,
	0.386933,
	0.393625,
	0.4137,
	0.407008,
	0.415931,
	0.420392,
	0.418161,
	0.431544,
	0.400317,
	0.37355,
	0.398086,
	0.384703,
	0.404778,
	0.382472,
	0.00104836,
	0.148265,
	0.415931,
	0.404778,
	0.386933,
	0.393625,
	0.386933,
	0.369089,
	0.398086,
	0.398086,
	0.393625,
	0.3334,
	0.284328,
	0.206259,
	0.404778,
	0.402547,
	0.433775,
	0.409239,
	0.217412,
	0.266484,
	0.420392,
	0.409239,
	0.420392,
	0.4137,
	0.402547,
	0.391395,
	0.386933,
	0.402547,
	0.418161,
	0.400317,
	0.407008,
	0.424853,
	0.424853,
	0.427083,
	0.389164,
	0.393625,
	0.41147,
	0.389164,
	0.386933,
	0.375781,
	0,
	0.134881,
	0.402547,
	0.286559,
	0.357936,
	0.404778,
	0.391395,
	0.420392,
	0.375781,
	0.404778,
	0.375781,
	0.382472,
	0.29102,
	0.273175,
	0.279867,
	0.434792,
	0.434792,
	0.434792,
	0.432029,
	0.434792,
	0.434792,
	0.434792,
	0.434792,
	0.434792,
	0.426503,
	0.434792,
	0.434792,
	0.434792,
	0.440318,
	0.434792,
	0.434792,
	0.434792,
	0.434792,
	0.434792,
	0.434792,
	0.434792,
	0.0755968,
	0.202697,
	0.349138,
	0.354664,
	0.329797,
	0.351901,
	0.362953,
	0.307692,
	0.374005,
	0.36019,
	0.354664,
	0.401636,
	0.401636,
	0.343612,
	0.327034,
	0.379531,
	0.374005,
	0.393347,
	0.401636,
	0.304929,
	0.412688,
	0.401636,
	0.393347,
	0.368479,
	0.379531,
	0.374005,
	0.407162,
	0.398873,
	0.401636,
	0.379531,
	0.382294,
	0.376768,
	0.398873,
	0.420977,
	0.42374,
	0.418214,
	0.415451,
	0.393347,
	0.412688,
	0.407162,
	0.089412,
	0.230327,
	0.393347,
	0.42374,
	0.390584,
	0.371242,
	0.346375,
	0.338086,
	0.197171,
	0.401636,
	0.302166,
	0.368479,
	0.371242,
	0.266247,
	0.376768,
	0.329797,
	0.335323,
	0.338086,
	0.346375,
	0.42374,
	0.420977,
	0.401636,
	0.407162,
	0.382294,
	0.362953,
	0.368479,
	0.393347,
	0.351901,
	0.398873,
	0.349138,
	0.374005,
	0.393347,
	0.401636,
	0.409925,
	0.426503,
	0.42374,
	0.412688,
	0.401636,
	0.407162,
	0.420977,
	0.0977011,
	0.26901,
	0.340849,
	0.434792,
	0.346375,
	0.418214,
	0.409925,
	0.329797,
	0.376768,
	0.365716,
	0.293877,
	0.376768,
	0.351901,
	0.362953,
	0.282825,
	0.210986,
	0.407162,
	0.310455,
	0.39611,
	0.39611,
	0.409925,
	0.393347,
	0.393347,
	0.39611,
	0.329797,
	0.362953,
	0.401636,
	0.376768,
	0.39611,
	0.343612,
	0.368479,
	0.385057,
	0.376768,
	0.382294,
	0.420977,
	0.432029,
	0.393347,
	0.39611,
	0.407162,
	0.393347,
	0.0977011,
	0.235853,
	0.365716,
	0.313218,
	0.409925,
	0.409925,
	0.418214,
	0.379531,
	0.39611,
	0.357427,
	0.351901,
	0.368479,
	0.338086,
	0.412688,
	0.349138,
	0.351901,
	0.393347,
	0.42374,
	0.407162,
	0.407162,
	0.401636,
	0.390584,
	0.390584,
	0.401636,
	0.36019,
	0.376768,
	0.39611,
	0.393347,
	0.407162,
	0.374005,
	0.357427,
	0.38782,
	0.385057,
	0.412688,
	0.401636,
	0.432029,
	0.42374,
	0.412688,
	0.42374,
	0.412688,
	0.0728337,
	0.274536,
	0.346375,
	0.432029,
	0.343612,
	0.426503,
	0.401636,
	0.38782,
	0.357427,
	0.349138,
	0.409925,
	0.407162,
	0.329797,
	0.368479,
	0.371242,
	0.393347,
	0.418214,
	0.374005,
	0.365716,
	0.38782,
	0.415451,
	0.418214,
	0.379531,
	0.401636,
	0.374005,
	0.379531,
	0.382294,
	0.401636,
	0.390584,
	0.412688,
	0.362953,
	0.368479,
	0.338086,
	0.412688,
	0.429266,
	0.429266,
	0.437555,
	0.429266,
	0.418214,
	0.429266,
	0.111516,
	0.249668,
	0.371242,
	0.282825,
	0.437555,
	0.390584,
	0.376768,
	0.368479,
	0.368479,
	0.376768,
	0.407162,
	0.390584,
	0.415451,
	0.385057,
	0.36019,
	0.23309,
	0.36019,
	0.180592,
	0.119805,
	0.20546,
	0.415451,
	0.401636,
	0.39611,
	0.398873,
	0.385057,
	0.38782,
	0.407162,
	0.407162,
	0.385057,
	0.412688,
	0.351901,
	0.36019,
	0.371242,
	0.374005,
	0.426503,
	0.440318,
	0.426503,
	0.440318,
	0.412688,
	0.418214,
	0.0977011,
	0.263484,
	0.313218,
	0.263484,
	0.42374,
	0.357427,
	0.354664,
	0.280062,
	0.33256,
	0.318744,
	0.374005,
	0.351901,
	0.379531,
	0.36019,
	0.379511,
	0.393257,
	0.379511,
	0.379511,
	0.379511,
	0.383439,
	0.379511,
	0.379511,
	0.379511,
	0.379511,
	0.379511,
	0.393257,
	0.383439,
	0.379511,
	0.359875,
	0.379511,
	0.379511,
	0.379511,
	0.363802,
	0.379511,
	0.379511,
	0.145839,
	0.361839,
	0.359875,
	0.365766,
	0.357911,
	0.32453,
	0.353984,
	0.36773,
	0.25973,
	0.318639,
	0.352021,
	0.334348,
	0.350057,
	0.381475,
	0.375584,
	0.357911,
	0.338275,
	0.308821,
	0.320602,
	0.318639,
	0.348093,
	0.336311,
	0.314712,
	0.363802,
	0.379511,
	0.381475,
	0.393257,
	0.381475,
	0.322566,
	0.385402,
	0.359875,
	0.359875,
	0.355948,
	0.36773,
	0.359875,
	0.34613,
	0.357911,
	0.359875,
	0.357911,
	0.363802,
	0.10853,
	0.393257,
	0.318639,
	0.340239,
	0.202784,
	0.220457,
	0.373621,
	0.375584,
	0.353984,
	0.357911,
	0.234203,
	0.336311,
	0.159584,
	0.167439,
	0.330421,
	0.192966,
	0.242057,
	0.330421,
	0.228312,
	0.385402,
	0.381475,
	0.385402,
	0.355948,
	0.369693,
	0.361839,
	0.359875,
	0.363802,
	0.342202,
	0.377548,
	0.340239,
	0.328457,
	0.369693,
	0.365766,
	0.381475,
	0.383439,
	0.369693,
	0.375584,
	0.369693,
	0.342202,
	0.353984,
	0.153694,
	0.330421,
	0.247948,
	0.401111,
	0.363802,
	0.322566,
	0.377548,
	0.373621,
	0.291148,
	0.289184,
	0.328457,
	0.361839,
	0.342202,
	0.273475,
	0.36773,
	0.147803,
	0.242057,
	0.289184,
	0.234203,
	0.377548,
	0.383439,
	0.381475,
	0.359875,
	0.355948,
	0.363802,
	0.359875,
	0.363802,
	0.353984,
	0.381475,
	0.352021,
	0.330421,
	0.334348,
	0.385402,
	0.377548,
	0.369693,
	0.357911,
	0.385402,
	0.393257,
	0.350057,
	0.371657,
	0.149766,
	0.381475,
	0.369693,
	0.297039,
	0.375584,
	0.375584,
	0.359875,
	0.336311,
	0.306857,
	0.322566,
	0.251875,
	0.359875,
	0.161548,
	0.308821,
	0.365766,
	0.357911,
	0.369693,
	0.291148,
	0.285257,
	0.359875,
	0.379511,
	0.381475,
	0.352021,
	0.359875,
	0.359875,
	0.365766,
	0.36773,
	0.344166,
	0.363802,
	0.375584,
	0.36773,
	0.363802,
	0.381475,
	0.383439,
	0.371657,
	0.371657,
	0.353984,
	0.363802,
	0.379511,
	0.369693,
	0.159584,
	0.377548,
	0.269548,
	0.393257,
	0.348093,
	0.326493,
	0.369693,
	0.381475,
	0.338275,
	0.353984,
	0.36773,
	0.251875,
	0.359875,
	0.153694,
	0.269548,
	0.338275,
	0.365766,
	0.326493,
	0.36773,
	0.375584,
	0.393257,
	0.383439,
	0.342202,
	0.359875,
	0.36773,
	0.359875,
	0.359875,
	0.316675,
	0.361839,
	0.379511,
	0.36773,
	0.383439,
	0.375584,
	0.369693,
	0.365766,
	0.369693,
	0.379511,
	0.373621,
	0.365766,
	0.377548,
	0.183148,
	0.334348,
	0.369693,
	0.220457,
	0.371657,
	0.291148,
	0.371657,
	0.385402,
	0.359875,
	0.285257,
	0.332384,
	0.240093,
	0.308821,
	0.267584,
	0.393257,
	0.149766,
	0.28133,
	0.293112,
	0.208675,
	0.385402,
	0.383439,
	0.393257,
	0.342202,
	0.359875,
	0.36773,
	0.383439,
	0.36773,
	0.348093,
	0.348093,
	0.371657,
	0.371657,
	0.405039,
	0.36773,
	0.379511,
	0.371657,
	0.377548,
	0.373621,
	0.369693,
	0.385402,
	0.36773,
	0.165475,
	0.377548,
	0.379511,
	0.297039,
	0.353984,
	0.355948,
	0.355948,
	0.375584,
	0.393257,
	0.316675,
	0.283293,
	0.344166,
	0.253839,
	0.279366,
	0.275424,
	0.275424,
	0.275424,
	0.27808,
	0.275424,
	0.275424,
	0.270112,
	0.275424,
	0.275424,
	0.280736,
	0.275424,
	0.275424,
	0.280736,
	0.275424,
	0.275424,
	0.275424,
	0.275424,
	0.275424,
	0.280736,
	0.275424,
	0.275424,
	0,
	0.11341,
	0.224961,
	0.137314,
	0.275424,
	0.0788824,
	0.0656025,
	0.108098,
	0.0443548,
	0.0549787,
	0.0841943,
	0.182465,
	0.29136,
	0.206369,
	0.254177,
	0.296672,
	0.235585,
	0.15325,
	0.280736,
	0.12669,
	0.275424,
	0.259488,
	0.283392,
	0.27808,
	0.286048,
	0.294016,
	0.267456,
	0.27808,
	0.275424,
	0.254177,
	0.267456,
	0.240897,
	0.246209,
	0.219649,
	0.238241,
	0.270112,
	0.27808,
	0.267456,
	0.251521,
	0.272768,
	0,
	0.227617,
	0.142626,
	0.150594,
	0.232929,
	0.0815384,
	0.0629466,
	0.246209,
	0.185121,
	0.171841,
	0.235585,
	0.259488,
	0.209025,
	0.272768,
	0.267456,
	0.256833,
	0.246209,
	0.169185,
	0.187777,
	0.248865,
	0.283392,
	0.240897,
	0.270112,
	0.29136,
	0.29136,
	0.307296,
	0.288704,
	0.296672,
	0.27808,
	0.280736,
	0.272768,
	0.224961,
	0.240897,
	0.240897,
	0.259488,
	0.254177,
	0.2648,
	0.283392,
	0.254177,
	0.275424,
	0,
	0.198401,
	0.0735705,
	0,
	0.0523227,
	0.262144,
	0.0762264,
	0.0735705,
	0.248865,
	0.132002,
	0.275424,
	0.259488,
	0.243553,
	0.270112,
	0.243553,
	0.219649,
	0.13997,
	0.280736,
	0.286048,
	0.256833,
	0.296672,
	0.254177,
	0.272768,
	0.2648,
	0.294016,
	0.294016,
	0.267456,
	0.29136,
	0.275424,
	0.283392,
	0.270112,
	0.240897,
	0.251521,
	0.246209,
	0.238241,
	0.211681,
	0.232929,
	0.262144,
	0.224961,
	0.254177,
	0,
	0.248865,
	0.15325,
	0.0868503,
	0.238241,
	0.15325,
	0.270112,
	0.116066,
	0.0523227,
	0.0416988,
	0.283392,
	0.142626,
	0.294016,
	0.206369,
	0.219649,
	0.288704,
	0.15325,
	0.275424,
	0.179809,
	0.270112,
	0.29136,
	0.283392,
	0.286048,
	0.29136,
	0.280736,
	0.280736,
	0.280736,
	0.296672,
	0.280736,
	0.283392,
	0.27808,
	0.270112,
	0.240897,
	0.267456,
	0.238241,
	0.251521,
	0.270112,
	0.27808,
	0.246209,
	0.254177,
	0,
	0.147938,
	0.283392,
	0.232929,
	0.11341,
	0.275424,
	0.206369,
	0.262144,
	0.203713,
	0.216993,
	0.224961,
	0.206369,
	0.280736,
	0.29136,
	0.270112,
	0.161218,
	0.2648,
	0.286048,
	0.232929,
	0.201057,
	0.29136,
	0.259488,
	0.270112,
	0.275424,
	0.29136,
	0.280736,
	0.283392,
	0.280736,
	0.27808,
	0.296672,
	0.29136,
	0.29136,
	0.270112,
	0.251521,
	0.270112,
	0.251521,
	0.27808,
	0.296672,
	0.262144,
	0.10013,
	0,
	0.132002,
	0.0310749,
	0.023107,
	0.2648,
	0.017795,
	0.0841943,
	0.270112,
	0.246209,
	0.177153,
	0.286048,
	0.124034,
	0.29136,
	0.025763,
	0,
	0.121378,
	0.219649,
	0.272768,
	0.102786,
	0.017795,
	0.296672,
	0.43119,
	0.413608,
	0.413608,
	0.396027,
	0.413608,
	0.413608,
	0.413608,
	0.424597,
	0.413608,
	0.413608,
	0.424597,
	0.413608,
	0.413608,
	0.413608,
	0.413608,
	0.413608,
	0.413608,
	0.413608,
	0.413608,
	0.43119,
	0.413608,
	0.00923039,
	0.231199,
	0.303723,
	0.428992,
	0.396027,
	0.396027,
	0.396027,
	0.255373,
	0.409213,
	0.411411,
	0.398224,
	0.439981,
	0.435585,
	0.437783,
	0.413608,
	0.422399,
	0.437783,
	0.428992,
	0.409213,
	0.398224,
	0.415806,
	0.404817,
	0.420201,
	0.435585,
	0.424597,
	0.413608,
	0.426794,
	0.413608,
	0.439981,
	0.444376,
	0.413608,
	0.404817,
	0.444376,
	0.435585,
	0.40262,
	0.428992,
	0.387236,
	0.360863,
	0.356468,
	0.367456,
	0,
	0.24878,
	0.327898,
	0.428992,
	0.281746,
	0.422399,
	0.424597,
	0.422399,
	0.40262,
	0.428992,
	0.356468,
	0.411411,
	0.352072,
	0.409213,
	0.444376,
	0.393829,
	0.404817,
	0.424597,
	0.398224,
	0.413608,
	0.426794,
	0.424597,
	0.435585,
	0.435585,
	0.411411,
	0.43119,
	0.418004,
	0.424597,
	0.424597,
	0.413608,
	0.428992,
	0.409213,
	0.43119,
	0.435585,
	0.422399,
	0.404817,
	0.40262,
	0.387236,
	0.358666,
	0.389433,
	0.00923039,
	0.360863,
	0.226803,
	0.38284,
	0.385038,
	0.314711,
	0.400422,
	0.422399,
	0.411411,
	0.433388,
	0.358666,
	0.229001,
	0.418004,
	0.437783,
	0.435585,
	0.369654,
	0.428992,
	0.369654,
	0.411411,
	0.40262,
	0.433388,
	0.426794,
	0.422399,
	0.428992,
	0.413608,
	0.450969,
	0.418004,
	0.424597,
	0.422399,
	0.420201,
	0.424597,
	0.418004,
	0.426794,
	0.453167,
	0.426794,
	0.418004,
	0.393829,
	0.38284,
	0.37405,
	0.409213,
	0.0334052,
	0.365259,
	0.237792,
	0.400422,
	0.424597,
	0.409213,
	0.35427,
	0.413608,
	0.415806,
	0.418004,
	0.424597,
	0.308118,
	0.367456,
	0.079557,
	0.0971386,
	0.43119,
	0.343282,
	0.396027,
	0.43119,
	0.407015,
	0.444376,
	0.424597,
	0.420201,
	0.424597,
	0.426794,
	0.398224,
	0.40262,
	0.380643,
	0.407015,
	0.396027,
	0.422399,
	0.404817,
	0.424597,
	0.426794,
	0.40262,
	0.418004,
	0.380643,
	0.37405,
	0.407015,
	0.343282,
	0.0443937,
	0.391631,
	0.187245,
	0.250978,
	0.409213,
	0.387236,
	0.299328,
	0.415806,
	0.407015,
	0.424597,
	0.418004,
	0.418004,
	0.424597,
	0.428992,
	0.415806,
	0.407015,
	0.424597,
	0.407015,
	0.424597,
	0.38284,
	0.398224,
	0.435585,
	0.435585,
	0.428992,
	0.442178,
	0.424597,
	0.426794,
	0.442178,
	0.415806,
	0.418004,
	0.426794,
	0.420201,
	0.420201,
	0.422399,
	0.424597,
	0.404817,
	0.360863,
	0.396027,
	0.396027,
	0.369654,
	0.0356029,
	0.413608,
	0.202628,
	0.187245,
	0.437783,
	0.38284,
	0.215815,
	0.270757,
	0.319107,
	0.424597,
	0.424597,
	0.266362,
	0.35427,
	0.332293,
	0.415806,
	0.398224,
	0.422399,
	0.37405,
	0.38284,
	0.407015,
	0.411411,
	0.428992,
	0.435585,
	0.424597,
	0.435585,
	0.43119,
	0.439981,
	0.435585,
	0.424597,
	0.426794,
	0.411411,
	0.422399,
	0.424597,
	0.413608,
	0.398224,
	0.391631,
	0.332293,
	0.338886,
	0.327898,
	0.389433,
	0.00703268,
	0.272955,
	0.24878,
	0.233396,
	0.627106,
	0.629537,
	0.624675,
	0.629537,
	0.629537,
	0.605227,
	0.629537,
	0.629537,
	0.607658,
	0.629537,
	0.629537,
	0.627106,
	0.622244,
	0.629537,
	0.629537,
	0.629537,
	0.619813,
	0.629537,
	0.629537,
	0.627106,
	0.629537,
	0.150626,
	0.607658,
	0.415607,
	0.439917,
	0.576054,
	0.605227,
	0.525003,
	0.549313,
	0.571192,
	0.568761,
	0.498262,
	0.527434,
	0.583347,
	0.619813,
	0.631968,
	0.610089,
	0.653847,
	0.61252,
	0.629537,
	0.627106,
	0.627106,
	0.631968,
	0.639261,
	0.648985,
	0.631968,
	0.641692,
	0.624675,
	0.607658,
	0.568761,
	0.510417,
	0.571192,
	0.561468,
	0.593072,
	0.619813,
	0.605227,
	0.605227,
	0.641692,
	0.602796,
	0.619813,
	0.639261,
	0.123885,
	0.432624,
	0.525003,
	0.595503,
	0.580917,
	0.522572,
	0.593072,
	0.580917,
	0.549313,
	0.539589,
	0.473952,
	0.602796,
	0.61252,
	0.610089,
	0.578485,
	0.629537,
	0.666002,
	0.471521,
	0.61252,
	0.607658,
	0.61252,
	0.63683,
	0.622244,
	0.66114,
	0.670864,
	0.631968,
	0.639261,
	0.622244,
	0.576054,
	0.512848,
	0.54202,
	0.56633,
	0.568761,
	0.63683,
	0.607658,
	0.590641,
	0.624675,
	0.61252,
	0.617382,
	0.627106,
	0.143333,
	0.464228,
	0.539589,
	0.629537,
	0.61252,
	0.444779,
	0.61252,
	0.580917,
	0.539589,
	0.522572,
	0.425331,
	0.593072,
	0.600365,
	0.66114,
	0.646554,
	0.639261,
	0.648985,
	0.641692,
	0.622244,
	0.605227,
	0.610089,
	0.622244,
	0.629537,
	0.63683,
	0.631968,
	0.634399,
	0.639261,
	0.627106,
	0.58821,
	0.520141,
	0.546882,
	0.561468,
	0.590641,
	0.634399,
	0.622244,
	0.607658,
	0.631968,
	0.622244,
	0.614951,
	0.624675,
	0.119023,
	0.444779,
	0.549313,
	0.605227,
	0.619813,
	0.522572,
	0.595503,
	0.571192,
	0.534727,
	0.534727,
	0.556606,
	0.58821,
	0.583347,
	0.648985,
	0.580917,
	0.624675,
	0.580917,
	0.537158,
	0.622244,
	0.415607,
	0.641692,
	0.58821,
	0.576054,
	0.646554,
	0.648985,
	0.648985,
	0.666002,
	0.624675,
	0.61252,
	0.54202,
	0.546882,
	0.583347,
	0.551744,
	0.578485,
	0.629537,
	0.607658,
	0.631968,
	0.646554,
	0.634399,
	0.583347,
	0.128747,
	0.366987,
	0.614951,
	0.571192,
	0.622244,
	0.597934,
	0.559037,
	0.444779,
	0.527434,
	0.478814,
	0.432624,
	0.634399,
	0.653847,
	0.648985,
	0.653847,
	0.646554,
	0.590641,
	0.515279,
	0.629537,
	0.622244,
	0.648985,
	0.648985,
	0.583347,
	0.627106,
	0.639261,
	0.658709,
	0.641692,
	0.624675,
	0.634399,
	0.610089,
	0.554175,
	0.551744,
	0.563899,
	0.593072,
	0.595503,
	0.61252,
	0.629537,
	0.605227,
	0.653847,
	0.619813,
	0.126316,
	0.396159,
	0.56633,
	0.617382,
	0.58821,
	0.507986,
	0.54202,
	0.473952,
	0.512848,
	0.593072,
	0.520141,
	0.602796,
	0.644123,
	0.639261,
	0.624675,
	0.585779,
	0.576054,
	0.301888,
	0.301888,
	0.308031,
	0.301888,
	0.301888,
	0.301888,
	0.308031,
	0.301888,
	0.301888,
	0.295745,
	0.303936,
	0.301888,
	0.308031,
	0.301888,
	0.301888,
	0.301888,
	0.29165,
	0.308031,
	0.301888,
	0.301888,
	0.29984,
	0.127836,
	0.330555,
	0.242506,
	0.289602,
	0.301888,
	0.312126,
	0.340794,
	0.348984,
	0.312126,
	0.324412,
	0.334651,
	0.340794,
	0.322365,
	0.351032,
	0.328508,
	0.289602,
	0.322365,
	0.289602,
	0.254792,
	0.289602,
	0.293697,
	0.29984,
	0.322365,
	0.330555,
	0.346937,
	0.340794,
	0.346937,
	0.336698,
	0.336698,
	0.338746,
	0.344889,
	0.344889,
	0.336698,
	0.344889,
	0.336698,
	0.324412,
	0.320317,
	0.301888,
	0.312126,
	0.312126,
	0.11555,
	0.342841,
	0.213838,
	0.197457,
	0.324412,
	0.328508,
	0.328508,
	0.316222,
	0.330555,
	0.338746,
	0.340794,
	0.336698,
	0.322365,
	0.305983,
	0.29984,
	0.303936,
	0.316222,
	0.303936,
	0.242506,
	0.29984,
	0.320317,
	0.303936,
	0.318269,
	0.308031,
	0.332603,
	0.332603,
	0.340794,
	0.314174,
	0.338746,
	0.336698,
	0.340794,
	0.340794,
	0.338746,
	0.338746,
	0.340794,
	0.328508,
	0.310079,
	0.314174,
	0.310079,
	0.301888,
	0.121693,
	0.308031,
	0.254792,
	0.213838,
	0.324412,
	0.314174,
	0.340794,
	0.303936,
	0.332603,
	0.344889,
	0.336698,
	0.340794,
	0.324412,
	0.32646,
	0.303936,
	0.271173,
	0.275268,
	0.314174,
	0.230219,
	0.316222,
	0.32646,
	0.328508,
	0.324412,
	0.320317,
	0.320317,
	0.320317,
	0.338746,
	0.342841,
	0.334651,
	0.342841,
	0.35308,
	0.336698,
	0.344889,
	0.346937,
	0.336698,
	0.330555,
	0.314174,
	0.324412,
	0.320317,
	0.324412,
	0.133979,
	0.336698,
	0.26503,
	0.303936,
	0.310079,
	0.29165,
	0.273221,
	0.289602,
	0.344889,
	0.336698,
	0.328508,
	0.340794,
	0.308031,
	0.324412,
	0.32646,
	0.289602,
	0.289602,
	0.289602,
	0.328508,
	0.297793,
	0.320317,
	0.338746,
	0.330555,
	0.320317,
	0.328508,
	0.322365,
	0.328508,
	0.338746,
	0.35308,
	0.334651,
	0.340794,
	0.328508,
	0.342841,
	0.346937,
	0.324412,
	0.316222,
	0.303936,
	0.320317,
	0.310079,
	0.328508,
	0.11555,
	0.271173,
	0.224076,
	0.222029,
	0.314174,
	0.301888,
	0.336698,
	0.293697,
	0.308031,
	0.303936,
	0.340794,
	0.338746,
	0.314174,
	0.297793,
	0.271173,
	0.318269,
	0.248649,
	0.310079,
	0.295745,
	0.308031,
	0.332603,
	0.340794,
	0.332603,
	0.322365,
	0.320317,
	0.322365,
	0.334651,
	0.342841,
	0.351032,
	0.338746,
	0.332603,
	0.336698,
	0.336698,
	0.342841,
	0.32646,
	0.314174,
	0.308031,
	0.328508,
	0.336698,
	0.330555,
	0.113502,
	0.262982,
	0.275268,
	0.289602,
	0.230219,
	0.328508,
	0.340794,
	0.254792,
	0.336698,
	0.303936,
	0.32646,
	0.351032,
	0.336698,
	0.324412,
	0.297793,
	0.293697,
	0.236363,
	0.303936,
	0.328508,
	0.384872,
	0.381266,
	0.381266,
	0.381266,
	0.384872,
	0.381266,
	0.381266,
	0.386675,
	0.381266,
	0.381266,
	0.381266,
	0.381266,
	0.381266,
	0.384872,
	0.384872,
	0.381266,
	0.381266,
	0.386675,
	0.381266,
	0.381266,
	0.381266,
	0.13965,
	0.363235,
	0.29652,
	0.361432,
	0.34881,
	0.365038,
	0.361432,
	0.350613,
	0.37225,
	0.341598,
	0.359629,
	0.354219,
	0.352416,
	0.352416,
	0.347007,
	0.370447,
	0.356022,
	0.352416,
	0.350613,
	0.343401,
	0.356022,
	0.361432,
	0.365038,
	0.37766,
	0.356022,
	0.366841,
	0.374053,
	0.375856,
	0.383069,
	0.374053,
	0.375856,
	0.374053,
	0.365038,
	0.37225,
	0.368644,
	0.354219,
	0.37225,
	0.356022,
	0.350613,
	0.381266,
	0.0855572,
	0.240624,
	0.305536,
	0.307339,
	0.337991,
	0.366841,
	0.370447,
	0.334385,
	0.363235,
	0.361432,
	0.379463,
	0.381266,
	0.359629,
	0.356022,
	0.368644,
	0.354219,
	0.379463,
	0.366841,
	0.356022,
	0.356022,
	0.370447,
	0.374053,
	0.379463,
	0.379463,
	0.366841,
	0.361432,
	0.381266,
	0.379463,
	0.388478,
	0.374053,
	0.368644,
	0.381266,
	0.37225,
	0.375856,
	0.379463,
	0.356022,
	0.356022,
	0.345204,
	0.365038,
	0.370447,
	0.0891634,
	0.208168,
	0.374053,
	0.365038,
	0.365038,
	0.361432,
	0.374053,
	0.354219,
	0.365038,
	0.37766,
	0.390281,
	0.383069,
	0.379463,
	0.37766,
	0.37225,
	0.386675,
	0.379463,
	0.363235,
	0.384872,
	0.374053,
	0.379463,
	0.381266,
	0.370447,
	0.383069,
	0.336188,
	0.368644,
	0.384872,
	0.370447,
	0.386675,
	0.381266,
	0.379463,
	0.379463,
	0.381266,
	0.375856,
	0.365038,
	0.361432,
	0.363235,
	0.368644,
	0.37225,
	0.370447,
	0.0891634,
	0.202759,
	0.276686,
	0.330779,
	0.356022,
	0.305536,
	0.356022,
	0.352416,
	0.386675,
	0.37225,
	0.386675,
	0.384872,
	0.368644,
	0.37766,
	0.356022,
	0.312748,
	0.334385,
	0.356022,
	0.354219,
	0.357825,
	0.365038,
	0.381266,
	0.375856,
	0.375856,
	0.365038,
	0.359629,
	0.37766,
	0.375856,
	0.390281,
	0.375856,
	0.370447,
	0.370447,
	0.379463,
	0.375856,
	0.356022,
	0.34881,
	0.363235,
	0.356022,
	0.345204,
	0.37766,
	0.0963758,
	0.19194,
	0.341598,
	0.368644,
	0.332582,
	0.374053,
	0.379463,
	0.374053,
	0.359629,
	0.34881,
	0.37225,
	0.379463,
	0.343401,
	0.368644,
	0.370447,
	0.359629,
	0.375856,
	0.354219,
	0.339794,
	0.383069,
	0.37225,
	0.383069,
	0.37766,
	0.381266,
	0.361432,
	0.370447,
	0.359629,
	0.37766,
	0.393887,
	0.383069,
	0.37766,
	0.37766,
	0.374053,
	0.383069,
	0.368644,
	0.359629,
	0.37225,
	0.354219,
	0.363235,
	0.365038,
	0.0981789,
	0.19194,
	0.276686,
	0.276686,
	0.347007,
	0.37766,
	0.381266,
	0.366841,
	0.384872,
	0.384872,
	0.383069,
	0.383069,
	0.361432,
	0.328976,
	0.366841,
	0.381266,
	0.375856,
	0.292914,
	0.350613,
	0.368644,
	0.356022,
	0.37766,
	0.375856,
	0.381266,
	0.368644,
	0.365038,
	0.37225,
	0.37766,
	0.57319,
	0.551349,
	0.551349,
	0.578043,
	0.556203,
	0.551349,
	0.551349,
	0.524655,
	0.551349,
	0.568336,
	0.551349,
	0.553776,
	0.568336,
	0.568336,
	0.553776,
	0.568336,
	0.57319,
	0.553776,
	0.553776,
	0.553776,
	0.551349,
	0.0126189,
	0.473694,
	0.405746,
	0.551349,
	0.444574,
	0.522229,
	0.4106,
	0.468841,
	0.216463,
	0.388759,
	0.476121,
	0.522229,
	0.556203,
	0.561056,
	0.548923,
	0.553776,
	0.548923,
	0.546496,
	0.483401,
	0.531936,
	0.524655,
	0.519802,
	0.563483,
	0.548923,
	0.578043,
	0.561056,
	0.561056,
	0.568336,
	0.56591,
	0.497962,
	0.546496,
	0.563483,
	0.548923,
	0.544069,
	0.558629,
	0.558629,
	0.534362,
	0.500388,
	0.398466,
	0.42516,
	0.0660066,
	0.364492,
	0.393613,
	0.505242,
	0.524655,
	0.534362,
	0.405746,
	0.534362,
	0.546496,
	0.510095,
	0.529509,
	0.529509,
	0.478548,
	0.369346,
	0.536789,
	0.514949,
	0.604737,
	0.544069,
	0.553776,
	0.434867,
	0.568336,
	0.578043,
	0.56591,
	0.500388,
	0.505242,
	0.527082,
	0.514949,
	0.536789,
	0.529509,
	0.539216,
	0.534362,
	0.548923,
	0.449427,
	0.546496,
	0.553776,
	0.514949,
	0.536789,
	0.507668,
	0.495535,
	0.466414,
	0.0781402,
	0.42516,
	0.4106,
	0.488255,
	0.558629,
	0.495535,
	0.393613,
	0.298971,
	0.201903,
	0.553776,
	0.497962,
	0.541642,
	0.500388,
	0.490681,
	0.505242,
	0.500388,
	0.490681,
	0.514949,
	0.505242,
	0.536789,
	0.578043,
	0.585323,
	0.539216,
	0.556203,
	0.466414,
	0.490681,
	0.497962,
	0.531936,
	0.548923,
	0.536789,
	0.548923,
	0.563483,
	0.476121,
	0.568336,
	0.563483,
	0.541642,
	0.512522,
	0.490681,
	0.529509,
	0.447001,
	0.0781402,
	0.357212,
	0.187342,
	0.0927005,
	0.330518,
	0.434867,
	0.575616,
	0.379053,
	0.449427,
	0.459134,
	0.490681,
	0.536789,
	0.490681,
	0.332945,
	0.568336,
	0.519802,
	0.534362,
	0.568336,
	0.393613,
	0.39604,
	0.585323,
	0.58047,
	0.58047,
	0.563483,
	0.490681,
	0.512522,
	0.531936,
	0.553776,
	0.57319,
	0.561056,
	0.524655,
	0.597457,
	0.561056,
	0.456707,
	0.468841,
	0.483401,
	0.519802,
	0.490681,
	0.447001,
	0.524655,
	0.0999806,
	0.364492,
	0.114541,
	0.146088,
	0.541642,
	0.531936,
	0.524655,
	0.548923,
	0.568336,
	0.551349,
	0.495535,
	0.539216,
	0.420307,
	0.463988,
	0.468841,
	0.478548,
	0.563483,
	0.56591,
	0.483401,
	0.59503,
	0.568336,
	0.558629,
	0.58047,
	0.56591,
	0.463988,
	0.490681,
	0.531936,
	0.544069,
	0.548923,
	0.553776,
	0.58775,
	0.582897,
	0.551349,
	0.539216,
	0.430014,
	0.476121,
	0.427587,
	0.442147,
	0.490681,
	0.519802,
	0.0732868,
	0.349932,
	0.340225,
	0.476121,
	0.381479,
	0.568336,
	0.582897,
	0.578043,
	0.517375,
	0.544069,
	0.563483,
	0.558629,
	0.442147,
	0.427587,
	0.512522,
	0.478548,
	0.578043,
	0.578043,
	0.544069,
	0.561056,
	0.568336,
	0.539216,
	0.585323,
	0.568336,
	0.456707,
	0.510095,
	0.544069,
	0.536789,
	0.568336,
	0.558629,
	0.541642,
	0.544069,
	0.57319,
	0.57319,
	0.551349,
	0.534362,
	0.485828,
	0.463988,
	0.495535,
	0.548923,
	0.0635799,
	0.308678,
	0.444574,
	0.408173,
	0.466414,
	0.531936,
	0.522229,
	0.510095,
	0.510095,
	0.500388,
	0.556203,
	0.556203,
	0.527082,
	0.308678,
	0.284411,
	0.307852,
	0.307852,
	0.316741,
	0.307852,
	0.307852,
	0.307852,
	0.307852,
	0.307852,
	0.307852,
	0.307852,
	0.307852,
	0.307852,
	0.307852,
	0.307852,
	0.307852,
	0.307852,
	0.307852,
	0.296,
	0.307852,
	0.307852,
	0.301926,
	0.168593,
	0.159704,
	0.171556,
	0.168593,
	0.153778,
	0.26637,
	0.224889,
	0.257481,
	0.257481,
	0.221926,
	0.281185,
	0.313778,
	0.319704,
	0.290074,
	0.275259,
	0.269333,
	0.218963,
	0.189333,
	0.207111,
	0.257481,
	0.260444,
	0.183407,
	0.263407,
	0.304889,
	0.307852,
	0.298963,
	0.307852,
	0.307852,
	0.304889,
	0.298963,
	0.307852,
	0.233778,
	0.275259,
	0.272296,
	0.26637,
	0.304889,
	0.298963,
	0.313778,
	0.301926,
	0.301926,
	0.147852,
	0.260444,
	0.127111,
	0.201185,
	0.210074,
	0.144889,
	0.257481,
	0.24563,
	0.207111,
	0.310815,
	0.248593,
	0.319704,
	0.233778,
	0.224889,
	0.216,
	0.298963,
	0.251556,
	0.298963,
	0.284148,
	0.307852,
	0.319704,
	0.296,
	0.298963,
	0.304889,
	0.296,
	0.272296,
	0.248593,
	0.298963,
	0.296,
	0.307852,
	0.313778,
	0.230815,
	0.263407,
	0.26637,
	0.278222,
	0.298963,
	0.307852,
	0.313778,
	0.304889,
	0.293037,
	0.168593,
	0.263407,
	0.159704,
	0.133037,
	0.221926,
	0.136,
	0.218963,
	0.0945185,
	0.227852,
	0.136,
	0.260444,
	0.307852,
	0.251556,
	0.230815,
	0.284148,
	0.307852,
	0.293037,
	0.224889,
	0.298963,
	0.290074,
	0.307852,
	0.304889,
	0.32563,
	0.304889,
	0.290074,
	0.275259,
	0.242667,
	0.304889,
	0.301926,
	0.313778,
	0.313778,
	0.239704,
	0.251556,
	0.275259,
	0.275259,
	0.296,
	0.304889,
	0.307852,
	0.307852,
	0.307852,
	0.168593,
	0.248593,
	0.130074,
	0.162667,
	0.293037,
	0.207111,
	0.213037,
	0.218963,
	0.233778,
	0.307852,
	0.298963,
	0.272296,
	0.278222,
	0.207111,
	0.221926,
	0.293037,
	0.290074,
	0.233778,
	0.307852,
	0.230815,
	0.319704,
	0.304889,
	0.307852,
	0.296,
	0.290074,
	0.281185,
	0.260444,
	0.313778,
	0.290074,
	0.304889,
	0.307852,
	0.236741,
	0.254519,
	0.284148,
	0.26637,
	0.296,
	0.304889,
	0.307852,
	0.301926,
	0.301926,
	0.150815,
	0.239704,
	0.127111,
	0.307852,
	0.189333,
	0.201185,
	0.227852,
	0.103407,
	0.269333,
	0.24563,
	0.233778,
	0.298963,
	0.221926,
	0.210074,
	0.216,
	0.301926,
	0.210074,
	0.319704,
	0.230815,
	0.298963,
	0.307852,
	0.304889,
	0.307852,
	0.301926,
	0.296,
	0.275259,
	0.248593,
	0.301926,
	0.304889,
	0.304889,
	0.313778,
	0.248593,
	0.251556,
	0.290074,
	0.284148,
	0.298963,
	0.298963,
	0.307852,
	0.307852,
	0.284148,
	0.153778,
	0.260444,
	0.124148,
	0.207111,
	0.275259,
	0.242667,
	0.210074,
	0.174519,
	0.239704,
	0.307852,
	0.257481,
	0.239704,
	0.298963,
	0.204148,
	0.263407,
	0.304889,
	0.248593,
	0.319704,
	0.313778,
	0.198222,
	0.319704,
	0.307852,
	0.284148,
	0.290074,
	0.298963,
	0.275259,
	0.248593,
	0.301926,
	0.307852,
	0.307852,
	0.307852,
	0.224889,
	0.251556,
	0.272296,
	0.275259,
	0.301926,
	0.313778,
	0.301926,
	0.290074,
	0.298963,
	0.147852,
	0.242667,
	0.130074,
	0.301926,
	0.239704,
	0.257481,
	0.248593,
	0.133037,
	0.257481,
	0.287111,
	0.307852,
	0.272296,
	0.269333,
	0.213037,
	0.141926,
	0.384097,
	0.384097,
	0.384097,
	0.38079,
	0.384097,
	0.384097,
	0.38079,
	0.384097,
	0.384097,
	0.384097,
	0.384097,
	0.384097,
	0.387403,
	0.384097,
	0.384097,
	0.384097,
	0.384097,
	0.384097,
	0.390709,
	0.384097,
	0.384097,
	0.0237064,
	0.179104,
	0.374178,
	0.374178,
	0.360952,
	0.367565,
	0.314664,
	0.337808,
	0.35434,
	0.291519,
	0.241924,
	0.202248,
	0.374178,
	0.384097,
	0.241924,
	0.324583,
	0.331195,
	0.364259,
	0.344421,
	0.390709,
	0.384097,
	0.38079,
	0.367565,
	0.364259,
	0.374178,
	0.38079,
	0.370871,
	0.367565,
	0.351033,
	0.337808,
	0.31797,
	0.331195,
	0.364259,
	0.31797,
	0.321276,
	0.35434,
	0.334502,
	0.321276,
	0.308051,
	0.324583,
	0.0270127,
	0.099752,
	0.374178,
	0.38079,
	0.364259,
	0.367565,
	0.344421,
	0.351033,
	0.327889,
	0.208861,
	0.390709,
	0.337808,
	0.364259,
	0.347727,
	0.308051,
	0.38079,
	0.357646,
	0.364259,
	0.347727,
	0.357646,
	0.38079,
	0.370871,
	0.364259,
	0.360952,
	0.38079,
	0.370871,
	0.374178,
	0.35434,
	0.384097,
	0.364259,
	0.347727,
	0.347727,
	0.374178,
	0.331195,
	0.364259,
	0.364259,
	0.38079,
	0.38079,
	0.35434,
	0.324583,
	0.0270127,
	0.126203,
	0.374178,
	0.374178,
	0.374178,
	0.374178,
	0.331195,
	0.364259,
	0.232005,
	0.331195,
	0.321276,
	0.364259,
	0.374178,
	0.397322,
	0.2816,
	0.374178,
	0.374178,
	0.351033,
	0.347727,
	0.364259,
	0.390709,
	0.374178,
	0.364259,
	0.370871,
	0.38079,
	0.397322,
	0.384097,
	0.367565,
	0.384097,
	0.360952,
	0.347727,
	0.347727,
	0.384097,
	0.374178,
	0.364259,
	0.374178,
	0.374178,
	0.384097,
	0.301438,
	0.38079,
	0.0468507,
	0.149347,
	0.387403,
	0.390709,
	0.374178,
	0.38079,
	0.351033,
	0.390709,
	0.2816,
	0.344421,
	0.377484,
	0.337808,
	0.374178,
	0.35434,
	0.331195,
	0.35434,
	0.374178,
	0.387403,
	0.35434,
	0.390709,
	0.384097,
	0.394016,
	0.374178,
	0.38079,
	0.387403,
	0.374178,
	0.400628,
	0.374178,
	0.407241,
	0.38079,
	0.351033,
	0.367565,
	0.387403,
	0.374178,
	0.374178,
	0.387403,
	0.387403,
	0.387403,
	0.31797,
	0.351033,
	0.0468507,
	0.152653,
	0.367565,
	0.288213,
	0.347727,
	0.360952,
	0.311357,
	0.394016,
	0.331195,
	0.205555,
	0.31797,
	0.347727,
	0.38079,
	0.374178,
	0.374178,
	0.390709,
	0.364259,
	0.387403,
	0.374178,
	0.38079,
	0.370871,
	0.394016,
	0.374178,
	0.374178,
	0.384097,
	0.374178,
	0.367565,
	0.38079,
	0.384097,
	0.374178,
	0.360952,
	0.364259,
	0.384097,
	0.367565,
	0.374178,
	0.38079,
	0.377484,
	0.384097,
	0.38079,
	0.38079,
	0.0435444,
	0.159266,
	0.364259,
	0.364259,
	0.367565,
	0.367565,
	0.321276,
	0.341114,
	0.341114,
	0.351033,
	0.35434,
	0.351033,
	0.374178,
	0.364259,
	0.225393,
	0.347727,
	0.364259,
	0.370871,
	0.364259,
	0.324583,
	0.374178,
	0.370871,
	0.374178,
	0.374178,
	0.38079,
	0.38079,
	0.344421,
	0.374178,
	0.347727,
	0.360952,
	0.451929,
	0.451929,
	0.451929,
	0.451929,
	0.458245,
	0.451929,
	0.451929,
	0.45614,
	0.451929,
	0.451929,
	0.451929,
	0.451929,
	0.451929,
	0.46035,
	0.46456,
	0.451929,
	0.451929,
	0.451929,
	0.451929,
	0.451929,
	0.451929,
	0,
	0.188788,
	0.424563,
	0.45614,
	0.45614,
	0.441404,
	0.441404,
	0.401406,
	0.414037,
	0.380355,
	0.420352,
	0.369829,
	0.38246,
	0.447719,
	0.447719,
	0.441404,
	0.401406,
	0.420352,
	0.401406,
	0.407722,
	0.424563,
	0.432983,
	0.424563,
	0.426668,
	0.451929,
	0.437193,
	0.449824,
	0.445614,
	0.443509,
	0.432983,
	0.439299,
	0.447719,
	0.437193,
	0.422458,
	0.45614,
	0.437193,
	0.437193,
	0.435088,
	0.414037,
	0.401406,
	0.0414289,
	0.395091,
	0.323516,
	0.416142,
	0.426668,
	0.376145,
	0.416142,
	0.422458,
	0.445614,
	0.418247,
	0.411932,
	0.298255,
	0.401406,
	0.401406,
	0.359304,
	0.350883,
	0.416142,
	0.317201,
	0.336147,
	0.321411,
	0.432983,
	0.458245,
	0.432983,
	0.432983,
	0.435088,
	0.403511,
	0.371934,
	0.418247,
	0.422458,
	0.449824,
	0.449824,
	0.432983,
	0.428773,
	0.428773,
	0.426668,
	0.439299,
	0.428773,
	0.435088,
	0.409827,
	0.395091,
	0.05827,
	0.401406,
	0.407722,
	0.315096,
	0.441404,
	0.289834,
	0.443509,
	0.435088,
	0.445614,
	0.371934,
	0.409827,
	0.405616,
	0.365619,
	0.401406,
	0.38246,
	0.327727,
	0.369829,
	0.401406,
	0.38667,
	0.424563,
	0.454035,
	0.445614,
	0.439299,
	0.424563,
	0.45614,
	0.401406,
	0.361409,
	0.420352,
	0.432983,
	0.45614,
	0.46456,
	0.443509,
	0.447719,
	0.432983,
	0.432983,
	0.46035,
	0.46035,
	0.447719,
	0.414037,
	0.401406,
	0.00985199,
	0.340357,
	0.357199,
	0.310886,
	0.414037,
	0.380355,
	0.38246,
	0.37404,
	0.432983,
	0.403511,
	0.38667,
	0.409827,
	0.369829,
	0.369829,
	0.29615,
	0.390881,
	0.230891,
	0.323516,
	0.369829,
	0.418247,
	0.420352,
	0.441404,
	0.418247,
	0.416142,
	0.424563,
	0.395091,
	0.350883,
	0.401406,
	0.418247,
	0.422458,
	0.432983,
	0.416142,
	0.430878,
	0.414037,
	0.401406,
	0.407722,
	0.409827,
	0.380355,
	0.403511,
	0.405616,
	0.0287982,
	0.30878,
	0.298255,
	0.416142,
	0.422458,
	0.336147,
	0.376145,
	0.422458,
	0.405616,
	0.405616,
	0.388775,
	0.30036,
	0.371934,
	0.319306,
	0.342463,
	0.350883,
	0.348778,
	0.37404,
	0.409827,
	0.405616,
	0.426668,
	0.424563,
	0.428773,
	0.416142,
	0.424563,
	0.359304,
	0.369829,
	0.395091,
	0.411932,
	0.426668,
	0.426668,
	0.432983,
	0.428773,
	0.418247,
	0.409827,
	0.432983,
	0.409827,
	0.409827,
	0.395091,
	0.395091,
	0.0351136,
	0.245627,
	0.327727,
	0.424563,
	0.376145,
	0.323516,
	0.376145,
	0.38246,
	0.411932,
	0.395091,
	0.38246,
	0.38667,
	0.376145,
	0.369829,
	0.355093,
	0.418247,
	0.395091,
	0.416142,
	0.416142,
	0.420352,
	0.407722,
	0.432983,
	0.422458,
	0.401406,
	0.424563,
	0.390881,
	0.363514,
	0.371934,
	0.411932,
	0.416142,
	0.432983,
	0.424563,
	0.437193,
	0.416142,
	0.411932,
	0.401406,
	0.401406,
	0.403511,
	0.390881,
	0.365619,
	0.00774686,
	0.319306,
	0.243521,
	0.428773,
	0.256152,
	0.424563,
	0.432983,
	0.437193,
	0.403511,
	0.37825,
	0.407722,
	0.357199,
	0.331937,
	0.367724,
	0.365619,
	0.292,
	0.292,
	0.292,
	0.292,
	0.292,
	0.292,
	0.297747,
	0.292,
	0.292,
	0.294873,
	0.292,
	0.292,
	0.297747,
	0.294873,
	0.292,
	0.292,
	0.292,
	0.292,
	0.292,
	0.292,
	0.292,
	0.174177,
	0.266136,
	0.260389,
	0.217283,
	0.257515,
	0.26901,
	0.309242,
	0.314989,
	0.294873,
	0.292,
	0.303494,
	0.263262,
	0.283378,
	0.274757,
	0.283378,
	0.280505,
	0.274757,
	0.274757,
	0.283378,
	0.294873,
	0.312116,
	0.300621,
	0.297747,
	0.292,
	0.277631,
	0.294873,
	0.306368,
	0.300621,
	0.292,
	0.312116,
	0.289126,
	0.248894,
	0.257515,
	0.274757,
	0.257515,
	0.274757,
	0.274757,
	0.303494,
	0.314989,
	0.297747,
	0.110955,
	0.306368,
	0.294873,
	0.266136,
	0.266136,
	0.283378,
	0.274757,
	0.271883,
	0.283378,
	0.294873,
	0.312116,
	0.306368,
	0.303494,
	0.283378,
	0.303494,
	0.303494,
	0.294873,
	0.312116,
	0.277631,
	0.260389,
	0.294873,
	0.297747,
	0.300621,
	0.292,
	0.309242,
	0.303494,
	0.294873,
	0.297747,
	0.283378,
	0.312116,
	0.297747,
	0.294873,
	0.263262,
	0.289126,
	0.294873,
	0.289126,
	0.300621,
	0.280505,
	0.312116,
	0.303494,
	0.108081,
	0.309242,
	0.294873,
	0.286252,
	0.289126,
	0.263262,
	0.260389,
	0.251767,
	0.266136,
	0.266136,
	0.309242,
	0.300621,
	0.309242,
	0.303494,
	0.297747,
	0.312116,
	0.283378,
	0.306368,
	0.294873,
	0.286252,
	0.294873,
	0.297747,
	0.309242,
	0.297747,
	0.312116,
	0.312116,
	0.289126,
	0.314989,
	0.292,
	0.303494,
	0.277631,
	0.286252,
	0.260389,
	0.280505,
	0.266136,
	0.297747,
	0.300621,
	0.300621,
	0.306368,
	0.300621,
	0.113828,
	0.303494,
	0.306368,
	0.292,
	0.292,
	0.271883,
	0.234525,
	0.280505,
	0.289126,
	0.289126,
	0.274757,
	0.283378,
	0.274757,
	0.277631,
	0.300621,
	0.314989,
	0.283378,
	0.312116,
	0.297747,
	0.286252,
	0.289126,
	0.271883,
	0.309242,
	0.297747,
	0.312116,
	0.306368,
	0.294873,
	0.289126,
	0.309242,
	0.320737,
	0.303494,
	0.300621,
	0.274757,
	0.260389,
	0.289126,
	0.266136,
	0.289126,
	0.306368,
	0.312116,
	0.297747,
	0.119576,
	0.303494,
	0.303494,
	0.266136,
	0.154061,
	0.274757,
	0.234525,
	0.303494,
	0.300621,
	0.300621,
	0.320737,
	0.314989,
	0.309242,
	0.277631,
	0.283378,
	0.283378,
	0.274757,
	0.274757,
	0.294873,
	0.283378,
	0.294873,
	0.300621,
	0.306368,
	0.300621,
	0.312116,
	0.309242,
	0.309242,
	0.300621,
	0.283378,
	0.312116,
	0.306368,
	0.300621,
	0.292,
	0.274757,
	0.266136,
	0.274757,
	0.294873,
	0.297747,
	0.312116,
	0.303494,
	0.12245,
	0.314989,
	0.165556,
	0.211535,
	0.191419,
	0.225904,
	0.26901,
	0.309242,
	0.303494,
	0.294873,
	0.314989,
	0.314989,
	0.309242,
	0.312116,
	0.309242,
	0.314989,
	0.274757,
	0.303494,
	0.294873,
	0.283378,
	0.300621,
	0.292,
	0.306368,
	0.260389,
	0.294873,
	0.312116,
	0.309242,
	0.309242,
	0.306368,
	0.314989,
	0.306368,
	0.306368,
	0.292,
	0.277631,
	0.26901,
	0.274757,
	0.289126,
	0.300621,
	0.312116,
	0.312116,
	0.125323,
	0.309242,
	0.179924,
	0.300621,
	0.208661,
	0.234525,
	0.283378,
	0.312116,
	0.303494,
	0.306368,
	0.320737,
	0.300621,
	0.303494,
	0.309242,
	0.314989,
	0.814392,
	0.814392,
	0.814392,
	0.814392,
	0.775399,
	0.825027,
	0.814392,
	0.814392,
	0.839206,
	0.775399,
	0.814392,
	0.814392,
	0.814392,
	0.814392,
	0.814392,
	0.814392,
	0.814392,
	0.814392,
	0.814392,
	0.839206,
	0.814392,
	0,
	0.442184,
	0.587522,
	0.63715,
	0.715137,
	0.690323,
	0.725771,
	0.736406,
	0.793123,
	0.732861,
	0.761219,
	0.708047,
	0.63715,
	0.64424,
	0.803758,
	0.647784,
	0.63715,
	0.796668,
	0.796668,
	0.825027,
	0.867565,
	0.768309,
	0.743495,
	0.764764,
	0.786033,
	0.810847,
	0.793123,
	0.786033,
	0.676143,
	0.764764,
	0.786033,
	0.736406,
	0.807302,
	0.828571,
	0.828571,
	0.810847,
	0.693867,
	0.800213,
	0.786033,
	0.786033,
	0.0168025,
	0.364197,
	0.725771,
	0.796668,
	0.708047,
	0.729316,
	0.793123,
	0.743495,
	0.683233,
	0.736406,
	0.690323,
	0.732861,
	0.768309,
	0.810847,
	0.743495,
	0.821482,
	0.690323,
	0.693867,
	0.583977,
	0.64424,
	0.810847,
	0.686778,
	0.828571,
	0.796668,
	0.793123,
	0.757675,
	0.768309,
	0.810847,
	0.775399,
	0.743495,
	0.743495,
	0.750585,
	0.786033,
	0.793123,
	0.828571,
	0.817937,
	0.768309,
	0.764764,
	0.796668,
	0.75413,
	0.0168025,
	0.186955,
	0.814392,
	0.63715,
	0.782489,
	0.828571,
	0.782489,
	0.661964,
	0.598157,
	0.697412,
	0.424459,
	0.40319,
	0.683233,
	0.332294,
	0.296845,
	0.764764,
	0.488267,
	0.445728,
	0.743495,
	0.697412,
	0.786033,
	0.775399,
	0.786033,
	0.796668,
	0.782489,
	0.789578,
	0.778944,
	0.782489,
	0.722226,
	0.541439,
	0.654874,
	0.683233,
	0.796668,
	0.825027,
	0.817937,
	0.810847,
	0.828571,
	0.817937,
	0.764764,
	0.786033,
	0,
	0.261397,
	0.846296,
	0.541439,
	0.541439,
	0.789578,
	0.764764,
	0.690323,
	0.548529,
	0.619426,
	0.52726,
	0.541439,
	0.771854,
	0.817937,
	0.669054,
	0.552074,
	0.396101,
	0.697412,
	0.619426,
	0.722226,
	0.796668,
	0.768309,
	0.796668,
	0.793123,
	0.796668,
	0.821482,
	0.846296,
	0.793123,
	0.793123,
	0.669054,
	0.63715,
	0.651329,
	0.825027,
	0.846296,
	0.821482,
	0.817937,
	0.810847,
	0.814392,
	0.796668,
	0.775399,
	0,
	0.194045,
	0.796668,
	0.690323,
	0.832116,
	0.810847,
	0.729316,
	0.654874,
	0.456363,
	0.750585,
	0.661964,
	0.807302,
	0.775399,
	0.530805,
	0.768309,
	0.732861,
	0.775399,
	0.771854,
	0.771854,
	0.786033,
	0.786033,
	0.796668,
	0.768309,
	0.796668,
	0.764764,
	0.786033,
	0.814392,
	0.821482,
	0.757675,
	0.729316,
	0.736406,
	0.700957,
	0.807302,
	0.817937,
	0.835661,
	0.821482,
	0.810847,
	0.803758,
	0.796668,
	0.697412,
	0,
	0.275576,
	0.775399,
	0.775399,
	0.828571,
	0.693867,
	0.732861,
	0.814392,
	0.743495,
	0.725771,
	0.743495,
	0.612336,
	0.757675,
	0.817937,
	0.75413,
	0.768309,
	0.786033,
	0.796668,
	0.757675,
	0.768309,
	0.771854,
	0.810847,
	0.786033,
	0.828571,
	0.832116,
	0.768309,
	0.821482,
	0.786033,
	0.764764,
	0.732861,
	0.73995,
	0.803758,
	0.789578,
	0.821482,
	0.803758,
	0.807302,
	0.817937,
	0.793123,
	0.817937,
	0.786033,
	0,
	0.208224,
	0.817937,
	0.817937,
	0.764764,
	0.732861,
	0.782489,
	0.750585,
	0.683233,
	0.722226,
	0.782489,
	0.672598,
	0.796668,
	0.828571,
	0.796668};

}

#endif
